// 3DOEditor.cpp : Defines the class behaviors for the application.
//
#include "stdafx.h"
#include "3DOEditor.h"

#include "MainFrm.h"
#include "ChildFrm.h"
#include "3DOEditorDoc.h"
#include "3DOEditorView.h"
#include "DatDocument.h"
#include "DatView.h"
#include "LpDocument.h"
#include "LpView.h"
#include "FileVersion.h"
#include "Credits.h"
#include "Register.h"

#include "TrkInfo.h"

#include "shlobj.h"

#include "TipDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BOOL CheckRegistrationName();

#include "Palette.h"

/////////////////////////////////////////////////////////////////////////////
// CMy3DOEditorApp

BEGIN_MESSAGE_MAP(CMy3DOEditorApp, CWinApp)
	ON_COMMAND(CG_IDS_TIPOFTHEDAY, ShowTipOfTheDay)
	//{{AFX_MSG_MAP(CMy3DOEditorApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_HELP_CREDITS, OnHelpCredits)
	ON_COMMAND(ID_STARTWINMIP, OnStartwinmip)
	ON_COMMAND(IDC_STARTGPL, OnStartgpl)
	ON_COMMAND(ID_EDIT_EDITWINMIPLOCATION, OnEditWinMipLocation)
	ON_COMMAND(ID_EDIT_EDITGPLLOCATION, OnEditGPLLocation)
	ON_COMMAND(ID_FILE_IMPORTPALETTE, OnFileImportpalette)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMy3DOEditorApp construction

CMy3DOEditorApp::CMy3DOEditorApp()
{
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CMy3DOEditorApp object

CMy3DOEditorApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CMy3DOEditorApp initialization

BOOL CMy3DOEditorApp::InitInstance()
{
	// Standard initialization

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	SetRegistryKey("TrackEditorSoftware");

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Register document templates
	LoadImages();


	pPckTemplate = new CMultiDocTemplate(
		IDR_MYDATTYPE2,
		RUNTIME_CLASS(CDatDocument),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(DatView));
	AddDocTemplate(pPckTemplate);

	pDatTemplate = new CMultiDocTemplate(
		IDR_MYDATTYPE,
		RUNTIME_CLASS(CDatDocument),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(DatView));
	AddDocTemplate(pDatTemplate);

	pDocTemplate = new CMultiDocTemplate(
		IDR_MY3DOETYPE,
		RUNTIME_CLASS(CMy3DOEditorDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CMy3DOEditorView));
	AddDocTemplate(pDocTemplate);

	pTrkTemplate = new CMultiDocTemplate(
		IDR_MYTRKETYPE,
		RUNTIME_CLASS(CDatDocument),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(DatView));
	AddDocTemplate(pTrkTemplate);


	pLpTemplate = new CMultiDocTemplate(
		IDR_MYLPTYPE,
		RUNTIME_CLASS(CLpDocument),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CLpView));
	AddDocTemplate(pLpTemplate);

	pTrkInfoTemplate = new CMultiDocTemplate(
		    IDR_MYTRKETYPE,
			RUNTIME_CLASS(CDatDocument),
			RUNTIME_CLASS(CChildFrame),
			RUNTIME_CLASS(CTrkInfo));
	AddDocTemplate(pTrkInfoTemplate);

	

	// create main MDI Frame window
	pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	

	// Dispatch commands specified on the command line
	//if (!ProcessShellCommand(cmdInfo))
	//	return FALSE;
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();

	CheckCurrentEditorVersion();

	// CG: This line inserted by 'Tip of the Day' component.
	ShowTipAtStartup();

	return TRUE;
}

void CMy3DOEditorApp::LoadImages()
{
	if (pImageList!=NULL) delete pImageList;

	pImageList = new CImageList();

	int size = (IDB_MAX - IDB_MIN);
	pImageList->Create(16, 15, TRUE, size, 2);	

	CBitmap bitmap;
	for (UINT iCnt=IDB_MIN; iCnt<=IDB_MAX; iCnt++)
	{
		bitmap.LoadBitmap(iCnt);
		pImageList->Add(&bitmap, (COLORREF)0xFFFFFF);
		bitmap.DeleteObject();
	}	
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual void OnCancel();
	virtual BOOL OnInitDialog();
	afx_msg void OnOk2();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	ON_BN_CLICKED(IDOK2, OnOk2)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CMy3DOEditorApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CMy3DOEditorApp commands

void CAboutDlg::OnCancel() 
{
	// TODO: Add extra cleanup here
	PostQuitMessage(0);
	
	CDialog::OnCancel();
}

BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	char VersionString[10];

	CFileVersion fileVersion;

	// get the name of the programme
	char ModuleName[256];
	GetModuleFileName(0,ModuleName,256);

	fileVersion.Open(ModuleName);

	//wsprintf(VersionString,"%d.%d.%d.%d.%d",MAJOR_VERSION,MINOR_VERSION,DEBUG_VERSION,DEBUG_SUB_VERSION,BUILDNUMBER);
	wsprintf(VersionString,"%s",fileVersion.GetProductVersion());
	SetDlgItemText(IDC_VERSION_STR,VersionString);
	//CString ver(VersionString);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CMy3DOEditorApp::OnHelpCredits() 
{
	CCredits *dlg = new CCredits();
	dlg->DoModal();
}

CView * CMy3DOEditorApp::OnC3doView(CDocument *pDoc)
{
    CFrameWnd* pNewFrame  = pDocTemplate->CreateNewFrame(pDoc, NULL);
	pDocTemplate->InitialUpdateFrame(pNewFrame, pDoc);
	return pNewFrame->GetActiveView();
}

void CMy3DOEditorApp::OnLpView(CDocument *pDoc)
{
    CFrameWnd* pNewFrame  = pLpTemplate->CreateNewFrame(pDoc, NULL);
	pLpTemplate->InitialUpdateFrame(pNewFrame, pDoc);
}

void CMy3DOEditorApp::OnStartwinmip() 
{
	static char buffer[512];
	//char   workingDir[512];
	CString gp2Location = getWinMipLocation();

	//GetCurrentDirectory(512,workingDir);

	//SetCurrentDirectory(gp2Location);
	wsprintf(buffer,"%s\\Winmip.exe",gp2Location);
		  
	WinExec(buffer,SW_SHOW);
	//SetCurrentDirectory(workingDir);
}

void CMy3DOEditorApp::OnEditWinMipLocation() 
{
	char fileName[256];

	strcpy(fileName,"*.*");
	
	BROWSEINFO browser;
	char buffer[MAX_PATH];

	browser.hwndOwner = NULL;
	browser.pszDisplayName = buffer;
	browser.lpszTitle = "Select WinMip Location";
	browser.lpfn = NULL;
	browser.pidlRoot = NULL;
	browser.ulFlags = BIF_RETURNONLYFSDIRS;

	LPITEMIDLIST files;

	files = SHBrowseForFolder(&browser);

	if (files !=NULL)
	{
		char filename[256];

		if (SHGetPathFromIDList(files,filename))
		{
		 CString strSection       = "Preferences";
         CString strStringItem    = "WinMipLocation";
         CWinApp* pApp = AfxGetApp();

		 pApp->WriteProfileString(strSection, strStringItem, CString(filename));
		}
		else
		{
	     AfxMessageBox("Failed to Locate WinMip directory");
		}
   	}
	else
	{
		AfxMessageBox("Failed to find WinMip directory");
	}
} 

void CMy3DOEditorApp::OnEditGPLLocation() 
{
	char fileName[256];

	strcpy(fileName,"*.*");
	
	BROWSEINFO browser;
	char buffer[MAX_PATH];

	browser.hwndOwner = NULL;
	browser.pszDisplayName = buffer;
	browser.lpszTitle = "Select GPL Location";
	browser.lpfn = NULL;
	browser.pidlRoot = NULL;
	browser.ulFlags = BIF_RETURNONLYFSDIRS;

	LPITEMIDLIST files;

	files = SHBrowseForFolder(&browser);

	if (files !=NULL)
	{
		char filename[256];

		if (SHGetPathFromIDList(files,filename))
		{
		 CString strSection       = "Preferences";
         CString strStringItem    = "GPLLocation";
         CWinApp* pApp = AfxGetApp();

		 pApp->WriteProfileString(strSection, strStringItem, CString(filename));
		}
		else
		{
	     AfxMessageBox("Failed to Locate GPL directory");
		}
   	}
	else
	{
		AfxMessageBox("Failed to find GPL directory");
	}
}


void CMy3DOEditorApp::OnStartgpl() 
{
	static char buffer[512];
	//char   workingDir[512];
	CString gp2Location = getGPLLocation();

	//GetCurrentDirectory(512,workingDir);

	//SetCurrentDirectory(gp2Location);
	wsprintf(buffer,"%s\\gpl.exe",gp2Location);
		  
	WinExec(buffer,SW_SHOW);
	//SetCurrentDirectory(workingDir);
}


CString CMy3DOEditorApp::getWinMipLocation()
	{
	 CString strSection       = "Preferences";
     CString strStringItem    = "WinMipLocation";
	 CWinApp* pApp = AfxGetApp();

	 CString circuitsDir = pApp->GetProfileString(strSection, strStringItem,"c:\\Program Files\\WinMip\\");
	 return circuitsDir;
	}


CString CMy3DOEditorApp::getGPLLocation()
	{
	 CString strSection       = "Preferences";
     CString strStringItem    = "GPLLocation";
	 CWinApp* pApp = AfxGetApp();

	 CString circuitsDir = pApp->GetProfileString(strSection, strStringItem,"c:\\Serria\\gpl\\");
	 return circuitsDir;
	}


void CAboutDlg::OnOk2() 
{
	CRegister *dlg = new CRegister();
	dlg->DoModal();
	if (dlg) delete dlg;
}

static char lpszPalFilter[] = "Palette (*.pal)|*.pal|All Files (*.*)|*.*";
extern int GetFileLength(FILE *fp);

void CMy3DOEditorApp::OnFileImportpalette() 
{
	   char fileName[256];
	   strcpy(fileName,"unnamed");
	   CFileDialog *fdlg = new CFileDialog(TRUE,"*",(LPCSTR)&fileName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,lpszPalFilter);

	   int result = fdlg->DoModal();

	   if (result == IDOK)
	   {
		   FILE *fpin = FOPEN(fdlg->GetFileName(),"rb");
	       if (fpin==NULL)
		   {
		   CString msg;
		   msg.Format("Cannot Open Palette %s ",fdlg->GetFileName());
		   AfxMessageBox(msg);
		   return;
		   }
		   int len = GetFileLength(fpin);

		   for(int i=0;i<len/3;i++)
		   {
			  int r = fgetc(fpin);
			  int g = fgetc(fpin);
			  int b = fgetc(fpin);

			  if (i > 256) break;
			  colorIndex[i].r = r;
			  colorIndex[i].g = g;
			  colorIndex[i].b = b;
		   }

		   FCLOSE(fpin);
	   }
}

void CMy3DOEditorApp::OnTablesTrackview(CDocument *pDoc) 
{
	CFrameWnd* pNewFrame  = pTrkInfoTemplate->CreateNewFrame(pDoc, NULL);
	pTrkInfoTemplate->InitialUpdateFrame(pNewFrame, pDoc);
}

void CMy3DOEditorApp::ShowTipAtStartup(void)
{
	// CG: This function added by 'Tip of the Day' component.

	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);
	if (cmdInfo.m_bShowSplash)
	{
		CTipDlg dlg;
		if (dlg.m_bStartup)
			dlg.DoModal();
	}

}

void CMy3DOEditorApp::ShowTipOfTheDay(void)
{
	// CG: This function added by 'Tip of the Day' component.

	CTipDlg dlg;
	dlg.DoModal();

}



#include <afxinet.h>

BOOL GetVersionPage(int &Major,int &Minor,int &Debug)
{
	//this code excerpt also demonstrates how to use //try/catch exception handling
	//assumes server, port and URL names have been initialized
	CInternetSession session("My Session");
	CHttpConnection* pServer = NULL;
	CHttpFile* pFile = NULL;
	DWORD dwRet;
	char szBuff[1024];
	
	try{   
			CString strServerName="members.xoom.com"; 
			CString strObject="/paulhoad/gpl/ver.html";
			INTERNET_PORT nPort=80;
			pServer = session.GetHttpConnection(strServerName);
			pFile = pServer->OpenRequest(CHttpConnection::HTTP_VERB_GET, strObject);
			//pFile->AddRequestHeaders(szHeaders);   
			pFile->SendRequest();
			pFile->QueryInfoStatusCode(dwRet);   
			
			if (dwRet == HTTP_STATUS_OK)   {
				UINT nRead = pFile->Read(szBuff, 1023);       
				while (nRead > 0)  {     
					sscanf(szBuff,"%d\n%d\n%d\n",&Major,&Minor,&Debug);
					nRead = 0;
				}   
			}   
			delete pFile;   
			delete pServer;
		}
		catch (CInternetException* ){   
		    //catch errors from WinInet
			//psz->ReportError();
			cerr << "Unable to check online connection error" << endl;
			session.Close();
			return FALSE;
		}
		session.Close();
		return TRUE;
}

BOOL CMy3DOEditorApp::CheckCurrentEditorVersion()
{
   CFileVersion fileVersion;
   // get the name of the programme
   char ModuleName[256];
   GetModuleFileName(0,ModuleName,256);

   fileVersion.Open(ModuleName);
   CString versionNumber = fileVersion.GetProductVersion();
   CString buff;
   int Major,Minor,Debug;
   Major = 0;
   Minor = 0;
   Debug = 0;
   int cmonth = GetProfileInt("Upgrade","CheckMonth",-1);
   int cday = GetProfileInt("Upgrade","CheckDay",-1);
   int timeChecked = GetProfileInt("Upgrade","CheckTimes",-1);
   timeChecked++;
   WriteProfileInt("Upgrade","CheckTimes",timeChecked);
   CTime t = CTime::GetCurrentTime();
   int month = t.GetMonth();
   int day   = t.GetDay();
   if (cmonth != month || cday != day || timeChecked >= 10)
   {
	 WriteProfileInt("Upgrade","CheckMonth",month);
	 WriteProfileInt("Upgrade","CheckDay",day);
	 timeChecked = 0;
	 WriteProfileInt("Upgrade","CheckTimes",timeChecked);
	 cerr << "Checking with online version..." << endl;
     if (GetVersionPage(Major,Minor,Debug))
	 {
	   int PMajor,PMinor,PDebug;
	   sscanf(versionNumber,"%d.%d.%d",&PMajor,&PMinor,&PDebug);
	   BOOL upgrade = FALSE;

	   if (PMajor <= Major) 
	   {
		  if (PMinor <= PMinor) 
		  {			  
				if (PDebug < Debug) 
				{
					upgrade = TRUE;
				}
		  }
	   }

	   if (upgrade)
	   {
		   CString ProVer;
		   CString CurVer;
		   ProVer.Format(" (Version %d.%d.%d)",PMajor,PMinor,PDebug);
		   CurVer.Format(" (Version %d.%d.%d)",Major,Minor,Debug);
		   CString msg;
		   msg += "Your version ";
		   msg += ProVer;
		   msg += " of the GPL Editor is out of date.";
		   msg += CurVer;
		   msg += " has now been released.";
		   msg += " Shall I open Your browser so you can download the next version?";
		   int result = AfxMessageBox(msg,MB_YESNO);
		   if (result == IDYES)
		   {
			   ShellExecute(NULL,"open", "http://members.xoom.com/paulhoad/gpl/index.html",NULL, NULL, SW_SHOWNORMAL);
		   }
		   else
		   {
			   cerr << "You have an out of date version" << endl;
		   }
	   }
	   else
	   {
		   cerr << "You have an upto date version" << endl;
	   }
	 }
     return TRUE;
   }
   else
   {
	   cerr << "Version check upto date [" << timeChecked << "]" << endl;
	   return FALSE;
   }
}
