#include "stdafx.h"
#include "stdlib.h"
#include "C3do.h"
#include "stdio.h"
#include "Point3D.h"
//#include "Polygon.h"
#include "math.h"

#include "3DOEditor.h"
#include "3DOEditorDoc.h"
#include "3DOEditorView.h"
#include "CGPLFile.h"
#include "DataValue.h"
#include "Section.h"

#include "ImageFile.h"
#include "CMip.h"
#include "Cam.h"
#include "BMP.h"
#include "Trk.h"
#include "MipError.h"
#include "fstream.h"
#include "stdio.h"

#include <gl\glaux.h>
#include <gl\gl.h>
#include <gl\glu.h>

extern int FileNameToIcon(char *name);
extern Node * getChildNode(int cmdval,int sectionOffset,Node *parent,FILE *fp,C3do *obj);
extern void totals();
extern void resettotals();


#define BEGIN_INDEX 4


void setRandomColor()
{
	static int lastColor=0;
	int colorIdx = (lastColor % numRandomColors);
	t_Color currentColor = RandomColors[colorIdx];
	lastColor++;
	glColor3f(currentColor.r,currentColor.g,currentColor.b);
}

void setGLColor(float r,float g,float b)
{
	//static  float lastr = -1.0f;
	//static  float lastg = -1.0f;
	//static  float lastb = -1.0f;

	//if (r != lastr || g != lastg || b !=lastb)
	//{
       glColor3f(r,g,b);
	//   lastr = r;
	//   lastg = g;
	//   lastb = b;
	//}
}

C3do::~C3do()
	{
		if (strs)     
		{
			delete strs;
			strs = NULL;
		}
		if (sections )
		{
			delete sections;
			sections = NULL;
		}

		//if (primTree) delete primTree;

		for(int i=0;i<numPts;i++)
		{
			delete ptsArray[i];
		}
		delete []ptsArray;

		if (objArray) {
			delete objArray;
			objArray = NULL;
		}

		if (trackBuffer) delete trackBuffer;

		topDrawlocation = NULL;
}

extern char * getline(FILE *fp);

BOOL C3do::ReadPtAliasName(const char *filename)
{
	char  TDatDrive[256];
	char  TDatDir[256];
	char  TDatFile[256];
	char  TDatExt[256];
	char  AliasFile[256];

	_splitpath(filename,TDatDrive,TDatDir,TDatFile,TDatExt);
    _makepath(AliasFile,TDatDrive,TDatDir,TDatFile,"nam");

    FILE *fp = FOPEN(AliasFile,"r");
	if (fp)
	{
        while(!feof(fp))
		{
			char *line = getline(fp);
			char name[256];
			int id;
			memset(name,0,256);

			sscanf(line,"%d %[^\0]\0",&id,name);
			if (ptsArray && id<=numPts)
			{
				ptsArray[id]->setPointName(name);
				ptAliases->addElement(new PointAlias(id,name));
			}
			free(line);
		}
		FCLOSE(fp);
	}
	return TRUE;
}


BOOL C3do::UpdateOrAddAlias(int id,char *_alias)
{
	ptsArray[id]->setPointName(_alias);
	BOOL found=FALSE;
	for(int i=0;i<ptAliases->size();i++)
	{
	   PointAlias *alias = (PointAlias*)ptAliases->elementAt(i);
	   if (id == alias->id)
	   {
		   found = TRUE;
		   alias->alias = strdup(_alias);
		   break;
	   }	
	}

	if (found == FALSE)
	{
		ptAliases->addElement(new PointAlias(id,_alias));
	}

	return FALSE;
}

BOOL C3do::WritePtAliasName(const char *filename)
{
	char  TDatDrive[256];
	char  TDatDir[256];
	char  TDatFile[256];
	char  TDatExt[256];
	char  AliasFile[256];

	_splitpath(filename,TDatDrive,TDatDir,TDatFile,TDatExt);
    _makepath(AliasFile,TDatDrive,TDatDir,TDatFile,"nam");

	if (ptAliases && ptAliases->size() >= 1)
	{
     FILE *fp = FOPEN(AliasFile,"wb");
	 if (fp)
	 {
		for(int i=0;i<ptAliases->size();i++)
		{
			PointAlias *alias = (PointAlias*)ptAliases->elementAt(i);
			fprintf(fp,"%d %s\n",alias->id,alias->alias);
		}
        FCLOSE(fp);
	 }
	}
	return TRUE;
}

BOOL C3do::Read(const char *filename)
{
	FILE *fp = FOPEN(filename,"rb");

	if (fp == NULL)
	{
		return FALSE;
	}
	Read(fp);
	ReadPtAliasName(filename);
	//FCLOSE(fp);
	return TRUE;
}

BOOL C3do::WriteICR2(FILE *fp)
{
	WritePCDWord(fp,ICR2DataSize);
	WritePCDWord(fp,RootOffset);
	WritePCDWord(fp,ICR2MipFiles);
	WritePCDWord(fp,ICR2PmpFiles);
	WritePCDWord(fp,ICR2ThreeDOFiles);
	for(int i=0;i<(ICR2MipFiles+ICR2PmpFiles+ICR2ThreeDOFiles);i++)
	{
		CImageFile *file = (CImageFile*)strs->elementAt(i);
		char filename[10];
		memset(filename,0,10);
		strncpy(filename,file->filename,10);
		fwrite(filename,1,8,fp);
	}
	int Start=ftell(fp);
	WritePCDWord(fp,ICR2Zero);
	for(i=0;i<numPts;i++)
	{
		Point3D *pt = (Point3D*)ptsArray[i];
		pt->WriteICR2(fp);
	}

	int sectionOffset = ftell(fp);
	int rootPos = primTree->Write(fp,sectionOffset+12);
	fseek(fp,4,SEEK_SET);
	int NewRootOffset = rootPos-Start;
	if (NewRootOffset != RootOffset)
	{
		cerr << "Root Offset Changed" << endl;
	}
	WritePCDWord(fp,NewRootOffset);
	return TRUE;
}

BOOL C3do::ReadICR2(FILE *fp)
{
	//00-03   Data Size (filesize - length of header)
    //04-07   Unknown Size
    //08-11   # of MIP files used by this 3DO file
    //12-15   # of PMP files used by this 3DO file
    //16-19   # of 3DO files used by this 3DO file
    //20-XX   file names - XX is 19 + 8*number of files
	ICR2DataSize     = ReadPCDWord(fp);
	RootOffset   = ReadPCDWord(fp);
	ICR2MipFiles     = ReadPCDWord(fp);
	ICR2PmpFiles     = ReadPCDWord(fp);
	ICR2ThreeDOFiles = ReadPCDWord(fp);
	for(int i=0;i<(ICR2MipFiles+ICR2PmpFiles+ICR2ThreeDOFiles);i++)
	{
		char filename[10];
		memset(filename,0,10);
		fread(filename,1,8,fp);
		CImageFile *file = new CImageFile(datFileParent,filename);
		strs->addElement(file);
	}
	Section *section = new Section(fp,this);
	strcpy(section->info.id,STRS_ID);
	section->info.length = (ICR2MipFiles+ICR2PmpFiles+ICR2ThreeDOFiles)*8;
    section->info.unk1   = 0;
	sections->addElement(section);
	int Start = ftell(fp);
	ICR2Zero = ReadPCDWord(fp);
	if (ICR2Zero!=0)
	{
		//AfxMessageBox("ICR2Zero not zero");
	}
	ICR2DataSectionStart = ftell(fp);
	


	numPts = 0;
	ptsArray = (Point3D**)malloc(100*sizeof(Point3D*));

	/*
	for(i=0;i<numPts;i++)
	{
		ptsArray[i] = new Point3D();
		ptsArray[i]->ReadICR2(fp);
		if (ptsArray[i]->pt.unk != 0)
		{
           numPts = i;
		   break;
		}
	}
	*/

	Section *pts = new Section(fp,this);
	strcpy(pts->info.id,XYZ_ID);
	pts->info.length = numPts*20;
    pts->info.unk1   = 0;
	sections->addElement(pts);


	Section *prim = new Section(fp,this);
	strcpy(prim->info.id,PRIM_ID);
    //RootOffset;
	prim->RootOffset = RootOffset;
	prim->offset  = Start;

	int result = IDYES;

	if (ask3do)
	{
		result =AfxMessageBox("Do You want to read the Track 3DO Tree this time? (NO Recommended)\n\n(Note: To improve loading speed the track.3do file can be ignored when you want to work on other files. To load the tree close and open the file again this time select YES",MB_YESNO);
	}

	if (result == IDYES)
	{
	 primTree = prim->readPrim(prim->offset,0);
	
	 if (primTree)
	 {
	  topDrawlocation->removeAllElements();
	  topDrawlocation->addElementReference(primTree->getRoot());
	 }
	}

	sections->addElement(prim);
	return TRUE;
}

extern int GetFileLength(FILE *fp);

BOOL C3do::Read(FILE *fp)
{
	int begin = ftell(fp);
	fseek(fp,begin,SEEK_SET);	
	BOOL shallLoad=TRUE;
	firstGLDraw = TRUE;

	fread(&Header,1,sizeof(t_Header),fp);
	
	if (!strncmp(Header.id,"4OD3",4))
	{
		version = t_GPL;
	}
	else if (!strncmp(Header.id,"3OD3",4))
	{
		version = t_N2;
	}
	else
	{
		version = t_ICR2;
		fseek(fp,begin,SEEK_SET);
		return ReadICR2(fp);
	}

	int afterHeader = ftell(fp);
	trackBufferSize = Header.FileSize+12;
	//trackBufferSize = GetFileLength(fp);
	trackBuffer = new unsigned char[trackBufferSize];
	fseek(fp,0,SEEK_SET);
	fread(trackBuffer,trackBufferSize,1,fp);
	fseek(fp,afterHeader,SEEK_SET);
	//trackBufferPos = afterHeader;
	/*
	if (LOWORD(Header.MagicNumber)!=20276)
	{
		if (Header.MagicNumber==860114739)
		{
		 version = t_N2;
		}
		else
		{
		 version = t_ICR2;
		 fseek(fp,begin,SEEK_SET);
		 return ReadICR2(fp);
		}
	}
	else
	{
		version = t_GPL;
	}
	*/
	

	int pos = begin+sizeof(t_Header);
	while(pos < (begin+Header.FileSize))
	{
		Section *section = new Section(fp,this);
		pos = section->ReadHeader(fp);
		if (pos == -1) break;
		sections->addElement(section);
	}

	if (verbose)  cout << endl;
	
	for(int i=0;i<sections->size();i++)
	{
	 Section *section = (Section*)sections->elementAt(i);

	 if (section->isXYZ() || section->isPERP())
	 {
	  
	  fseek(fp,section->offset,SEEK_SET);
	  int size = section->info.length/16;

	  ptsArray = (Point3D**)malloc(size*sizeof(Point3D*));
	  if (ptsArray==NULL)
	  {
		  AfxMessageBox("Failed to malloc Memory");
		  return FALSE;
	  }
	  numPts = size;
	  BOOL isN2 = section->isPERP();
	  if (verbose)  cout << "Reading " << size << " Points...";
	  if (verbose)  cout.flush();
	  for(int i=0;i<size;i++)
	  {
		Point3D *pt= new Point3D();
		pt->object = this;
		pt->idx = i;
		if (isN2)
		  pt->ReadN2(fp);
		else
		  pt->Read(fp);
		ptsArray[i] = pt;
	  }
	  if (verbose) cout << "Done" << endl;
	  if (verbose) cout.flush();
	 }
	 else if (section->isPLAN())
	 {
		 if (verbose) cout << "Reading Plan...";
	     if (verbose) cout.flush();
		 fseek(fp,section->offset,SEEK_SET);
		 //trackBufferPos = section->offset;
	     int count = 0;
	     int size = section->info.length/4;
		 numPlanes = size/4;
		 Plans = (CPlanes**) malloc(numPlanes*sizeof(CPlanes*));
		 

		 for(int i=0;i<numPlanes;i++)
		 {
			Plans[i] = new CPlanes();
			float A = ReadPCFloat(fp);
			float B = ReadPCFloat(fp);
			float C = ReadPCFloat(fp);
			float D = ReadPCFloat(fp);
			Plans[i]->A.data = A;
			Plans[i]->B.data = B;
			Plans[i]->C.data = C;
			Plans[i]->D.data = D;

			Plans[i]->A.setDesc("Plan A");
			Plans[i]->B.setDesc("Plan B");
			Plans[i]->C.setDesc("Plan C");
			Plans[i]->D.setDesc("Plan D");
		 }	
		 if (verbose) cout << "Done" << endl;
	     if (verbose) cout.flush();
	 }
	 else if (section->isNORM())
	 {
		 if (verbose) cout << "Reading Norm...";
	     if (verbose) cout.flush();
		 fseek(fp,section->offset,SEEK_SET);
		 //trackBufferPos = section->offset;
	     int count = 0;
	     int size = section->info.length/4;
		 numNormals = size/4;
		 Normals = (CNormals**) malloc(numNormals*sizeof(CNormals*));
		 

		 for(int i=0;i<numNormals;i++)
		 {
			Normals[i] = new CNormals();
			float A = ReadPCFloat(fp);
			float B = ReadPCFloat(fp);
			float C = ReadPCFloat(fp);
			float D = ReadPCFloat(fp);
			Normals[i]->A.data = A;
			Normals[i]->B.data = B;
			Normals[i]->C.data = C;
			Normals[i]->D.data = D;

			Normals[i]->A.setDesc("Normal X");
			Normals[i]->B.setDesc("Normal Y");
			Normals[i]->C.setDesc("Normal Z");
			Normals[i]->D.setDesc("Normal Unk");
		 }	
		 if (verbose) cout << "Done" << endl;
	     if (verbose) cout.flush();
	 }
	 else if (section->isSTR())
	 {
	  if (verbose) cout << "Reading Strings...";
	  if (verbose) cout.flush();
	  fseek(fp,section->offset,SEEK_SET);
	  int count = 0;
	  int size = section->info.length;
	  
	  char name[256];
	  int  pos=0;
	  //int offset = size-section->offset;
	  int offset = ftell(fp)-section->offset;
	  while(count < size)
	  {
		char c =fgetc(fp);
		name[pos++] = c;
		if (c == 0)
		{
			name[pos]='\0';
			CImageFile *file = new CImageFile(datFileParent,name);
			file->offset = offset;
			strs->addElement(file);
			offset = ftell(fp)-section->offset;
	        pos = 0;
			count++;
			continue;
		}
		count++;
	  }
	  if (verbose) cout << "Done" << endl;
	  if (verbose) cout.flush();
	 }
	 else if (section->isPRIM())
	 {
		 int result=IDYES;
		 resettotals();

		 if (ask3do)
		 {
			 result =AfxMessageBox("Do You want to read the Track 3DO Tree this time? (NO Recommended)\n\n(Note: To improve loading speed the track.3do file can be ignored when you want to work on other files. To load the tree close and open the file again this time select YES",MB_YESNO);
		 }
		 
		 fseek(fp,section->offset,SEEK_SET);
		 //trackBufferPos = section->offset;
		 RootBegin = section->offset;
	     int count = 16;
	     int size = section->info.length;
		 section->RootOffset = ReadPCDWord(fp);
		 RootOffset = section->RootOffset;
		 if (result==IDYES)
		 {
		  primTree = section->readPrim(section->offset,0);
		  SecondPass(primTree);
		  if (primTree)
		  {
		   topDrawlocation->removeAllElements();
			topDrawlocation->addElementReference(primTree->getRoot());
		  }
		 }
		 else
		 {
			 shallLoad=FALSE;
		 }
	 }
	 else if (section->isHAND())
	 {
		 if (verbose) cout << "Reading Hand...";
	     if (verbose) cout.flush();
		 fseek(fp,section->offset,SEEK_SET);
	     int count = 0;
	     int size = section->info.length/4;
		 numHandles = size/2;
		 Handles = (CHandles**) malloc(numHandles*sizeof(CHandles*));
		 extras  = (Node**) malloc(numHandles*sizeof(Node*));

		 for(int i=0;i<numHandles;i++)
		 {
			Handles[i] = new CHandles();
			int str_offset  = ReadPCDWord(fp);
			int offset_tree = ReadPCDWord(fp);

			Handles[i]->String.data = (float)str_offset;
			Handles[i]->Offset.data = (float)offset_tree;

			int foff = ftell(fp);
			if (shallLoad)
			{
				CString name = findStrName(str_offset);
				if (stricmp(name,"") && stricmp(name,"Unk"))
				{
			       extras[i] = section->readPrim(RootBegin,(int)(Handles[i]->Offset.data));
				}
				else
				{
				   extras[i] = NULL;
				}
			}
			else
			{
				extras[i] = NULL;
			}
		    fseek(fp,foff,SEEK_SET);
		 }	
		 if (verbose) cout << "Done" << endl;
	     if (verbose) cout.flush();
	 }
	 else
	 {
		 cout << "Unknown Section " << section->GetName() << endl;
		 
	 }
	 
	}
	return TRUE;
}

BOOL C3do::Write(const char *filename)
{
	FILE *fp = FOPEN(filename,"wb");

	if (fp == NULL)
	{
		return FALSE;
	}
	Write(fp);
	WritePtAliasName(filename);
	FCLOSE(fp);
	return TRUE;
}

BOOL C3do::Write(FILE *fp)
{
	fwrite(trackBuffer,trackBufferSize,1,fp);
	fseek(fp,0,SEEK_SET);

	if (isICR2Version())
	{
		return WriteICR2(fp);
	}
	fwrite(&Header,1,sizeof(t_Header),fp);

	for(int i=0;i<sections->size();i++)
	{
	 Section *section = (Section*)sections->elementAt(i);
	 int sectionOffset = ftell(fp);
	 

	 // Write out those points
	 if (section->isXYZ() || section->isPERP())
	 {
	  section->info.length = numPts*16;
	  section->WriteHeader(fp);
	  BOOL isN2 = section->isPERP();
	  for(int i=0;i<numPts;i++)
	  {
		if (isN2)
			ptsArray[i]->WriteN2(fp);
		else
			ptsArray[i]->Write(fp);
	  }
	 }
	 else if (section->isSTR())
	 {
	  //calculate the positon
	  int sectioncount=0;
	  int start = ftell(fp);
	  for(int i=0;i<strs->size();i++)
	  {
		CImageFile *str = (CImageFile*)strs->elementAt(i);
		str->offset = sectioncount;
		sectioncount+=str->WriteSize();		
	  }
	  sectioncount++;
	  start+=sectioncount;
	  //int remain = (12+start)%4;
	  //sectioncount+=remain;
	  section->info.length =sectioncount;
	  // rewind to start of stringsections
	  //fseek(fp,start,SEEK_SET);
	  section->WriteHeader(fp);
	  
	  for(i=0;i<strs->size();i++)
	  {
		CImageFile *str = (CImageFile*)strs->elementAt(i);
		str->Write(fp);		
	  }
	  // string table teminator
	  fputc(0xFF,fp);
	  int remain = 4-(ftell(fp)%4);
	  if (remain <=3)
	  {
	   for(i=0;i<remain;i++)
	   {
          fputc(0x20,fp);
	   }
	  }
	 }
	 else if (section->isPLAN())
	 {
	  //int size = section->info.length/4;
	  section->info.length = numPlanes*16;
	  section->WriteHeader(fp);
	  for(int i=0;i<numPlanes;i++)
	  {
		float A = Plans[i]->A.data;
		float B = Plans[i]->B.data;
		float C = Plans[i]->C.data;
		float D = Plans[i]->D.data;
		fwrite(&A,1,sizeof(float),fp);
		fwrite(&B,1,sizeof(float),fp);
		fwrite(&C,1,sizeof(float),fp);
		fwrite(&D,1,sizeof(float),fp);
	  }
	 }
	 else if (section->isNORM())
	 {
	  //int size = section->info.length/4;
	  section->info.length = numNormals*16;
	  section->WriteHeader(fp);
	  for(int i=0;i<numNormals;i++)
	  {
		float A = Normals[i]->A.data;
		float B = Normals[i]->B.data;
		float C = Normals[i]->C.data;
		float D = Normals[i]->D.data;
		fwrite(&A,1,sizeof(float),fp);
		fwrite(&B,1,sizeof(float),fp);
		fwrite(&C,1,sizeof(float),fp);
		fwrite(&D,1,sizeof(float),fp);
	  }
	 }
	 else if (section->isPRIM())
	 {
		int start = ftell(fp);
		if (primTree)
		  primTree->ClearWriteStatus();

		for(int i=0;i<numHandles;i++)
		{
		 if (extras[i])
		 {
			 extras[i]->ClearWriteStatus();
			 extras[i]->written = FALSE;
		 }
		}


		WriteAsOverLay = FALSE;

	    if (SeenANodeFifteen)
		{
		 int askWrite = AfxMessageBox("Is this a Track 3do File",MB_YESNO);
		 if (askWrite==IDYES)
		 {
			WriteAsOverLay = TRUE;
		 }
		}
		
		if (!WriteAsOverLay)
		{		
		section->WriteHeader(fp);

		// save the root offset Position for later
		WriteRoofOffsetPosition = ftell(fp);
		// write out that dummy root offset.
		WritePCDWord(fp,0xFEFEFEFE);
		int FirstTreeItemOffset = ftell(fp);


		// If you have a handle section containing the collision box then
		// it must be save first and the offset to the command saved in the
		// handles section.
		int currentOffset = sectionOffset+12;

		for(int i=0;i<numHandles;i++)
		{
		 if (extras[i])
		 {
		  if (extras[i]->written == FALSE)
		  {
		   extras[i]->written = TRUE;
		   int collisionPos = extras[i]->Write(fp,sectionOffset+12);
		   Handles[i]->Offset.data = (float)collisionPos-currentOffset;
		   extras[i]->writingOffset = collisionPos;
		  }
		  else
		  {
           AfxMessageBox("Already Written");
		  }		  
		  //currentOffset=collisionPos+4;
          //currentOffset=ftell(fp);
		 }
		}

	    int rootPos = primTree->Write(fp,sectionOffset+12);
		int end = ftell(fp);
		fseek(fp,start,SEEK_SET);
		int newlen = (end-WriteRoofOffsetPosition);
		if (newlen != section->info.length)
		{
			CString msg;
			msg.Format("section length changed %d to %d",newlen,section->info.length);
			AfxMessageBox(msg);
		}
		section->info.length = newlen;
		section->WriteHeader(fp);
		fseek(fp,WriteRoofOffsetPosition,SEEK_SET);
		//WritePCDWord(fp,rootPos-section->getObject()->primTree->getOffset());
		int NewRootOffset = rootPos-start-12;
		if (NewRootOffset != section->RootOffset)
		{
			cerr << "Root Offset Changed" << endl;
		}
		WritePCDWord(fp,NewRootOffset);
		fseek(fp,end,SEEK_SET);
		}
		else
		{
			//int curpos = ftell(fp);
			//section->WriteHeader(fp);
			//WritePCDWord(fp,section->RootOffset);
			primTree->OverLay(fp);
		}
	 }
	 else if (section->isHAND())
	 {
		 if (WriteAsOverLay)
		 {
			 fseek(fp,section->offset-12,SEEK_SET);
		 }
		 int start = ftell(fp);
		 section->WriteHeader(fp);
		 int size = section->info.length/4;
	     for(int i=0;i<numHandles;i++)
	     {
		    int data1 = (int)Handles[i]->String.data;
			int data2 = (int)Handles[i]->Offset.data;
		    fwrite(&data1,1,sizeof(int),fp);
			fwrite(&data2,1,sizeof(int),fp);
	     }
	 }
	 else
	 {
		AfxMessageBox("Unable to Write Section No Writer Present For this type");
	 }
	

	}
	int foff = ftell(fp);

	if (isN2Version())
	{
	 // pad to a div 4 length N2 not sure about GPL
	 int remainder = 4-(foff%4);
	 if (remainder != 4)
	 {
	  for(i=0;i<remainder;i++)
	  {
		fputc(0x20,fp);
	  }
	 }
	}

	fflush(fp);
	fseek(fp,0,SEEK_SET);
	Header.FileSize = GetFileLength(fp)-0x0c;
	fwrite(&Header,1,sizeof(t_Header),fp);
	return TRUE;
}

void C3do::Explore()
{
	GetFile();

    if (dataLoaded)
	{
	   //help?
	   CMy3DOEditorDoc *TdoDocument = new CMy3DOEditorDoc();
	   pDoc = TdoDocument;
	   TdoDocument->setObject(this);
	   ((CMy3DOEditorApp*)AfxGetApp())->OnC3doView(TdoDocument);
	}
	else
	{
	   AfxMessageBox("File Cannot be Loaded");
	}
}

//void C3do::DrawGL(HDC hdc,int showPoints,int showTextures,BOOL m_LeftButtonDown)
void C3do::DrawGLText(HDC hdc,CView *view)
{
	    CMy3DOEditorView *doview = (CMy3DOEditorView*)view;
		glPushMatrix();
		float scale = 3.0f;
		glScalef(scale,scale,scale);
	    glTranslated(-midx,-midy,-midz);

        if (doview->GetDocument()->showPointNumbers)
		{
		 for(int i=0;i<numPts;i++)
		 {
		
		 GLdouble wx = ptsArray[i]->getX();
		 GLdouble wy = ptsArray[i]->getY();
		 GLdouble wz = ptsArray[i]->getZ();
		 

		 DrawText2D(hdc,doview,wx,wy,wz,i);		
		 }
		}
		glPopMatrix();
}

void C3do::GetDist(CView *view)
{
	CMy3DOEditorView *doview = (CMy3DOEditorView*)view;
	minx=miny=minz = 10E8;	
	maxx=maxy=maxz = -10E8;
	for(int i=0;i<numPts;i++)
	 {
	  Point3D *pt = ptsArray[i];
	  
	  GLdouble wx,wy,wz;
	  wx = (double)pt->getX();
	  wy = (double)pt->getY();
	  wz = (double)pt->getZ();
	  minx = min(wx,minx);
	  miny = min(wy,miny);
	  minz = min(wz,minz);
	  maxx = max(wx,maxx);
	  maxy = max(wy,maxy);
	  maxz = max(wz,maxz);
	 }
	 //firstGLDraw=FALSE;

	 double m1 = max(fabs(minx),max(fabs(miny),fabs(minz)));
	 double m2 = max(fabs(maxx),max(fabs(maxy),fabs(maxz)));
	 doview->GetDocument()->dist = max(50.0,6.0*max(m1,m2));
}

void C3do::ReMaxMin()
{
	minx=miny=minz = 10E8;	
	maxx=maxy=maxz = -10E8;

	//int numPts = ptsArray->GetCount();
	for(int i=0;i<numPts;i++)
	 {
	  Point3D *pt = ptsArray[i];
	  //POSITION pos = ptsArray->FindIndex(i);
	  //Point3D *pt = (Point3D*)ptsArray->GetAt(pos);

	  GLdouble wx,wy,wz;
	  wx = (double)pt->getX();
	  wy = (double)pt->getY();
	  wz = (double)pt->getZ();
	  minx = min(wx,minx);
	  miny = min(wy,miny);
	  minz = min(wz,minz);
	  maxx = max(wx,maxx);
	  maxy = max(wy,maxy);
	  maxz = max(wz,maxz);
	 }

  midx = (maxx+minx)/2.0;
  midy = (maxy+miny)/2.0;
  midz = (maxz+minz)/2.0;
}

void C3do::DrawGL(HDC hdc,CView *view,int currentTexture)
{
	CMy3DOEditorView *doview = (CMy3DOEditorView*)view;
	glPushMatrix();

	if (firstGLDraw || doview->m_AltDown)
	{
	ReMaxMin();
	firstGLDraw=FALSE;
	}

	if (doview->viewDirection != t_XYZ)
	{
		ReMaxMin(); 


	}

	//glPointSize(5.0f); 
	float scale = 3.0f;

	glScalef(scale,scale,scale);
	glTranslated(-midx,-midy,-midz);
	
	if (doview->m_LeftButtonDown && doview->viewDirection==t_XYZ)
	{
	  glBegin(GL_LINES);
	  glVertex3d(minx,miny,minz);
	  glVertex3d(minx,maxy,minz);
	  glVertex3d(minx,maxy,minz);
	  glVertex3d(maxx,maxy,minz);
	  glVertex3d(maxx,maxy,minz);
	  glVertex3d(maxx,miny,minz);
	  glVertex3d(maxx,miny,minz);
	  glVertex3d(minx,miny,minz);
	  glEnd();
	  glBegin(GL_LINES);
	  glVertex3d(minx,miny,maxz);
	  glVertex3d(minx,maxy,maxz);
	  glVertex3d(minx,maxy,maxz);
	  glVertex3d(maxx,maxy,maxz);
	  glVertex3d(maxx,maxy,maxz);
	  glVertex3d(maxx,miny,maxz);
	  glVertex3d(maxx,miny,maxz);
	  glVertex3d(minx,miny,maxz);
	  glEnd();

	  glBegin(GL_LINES);
	  glVertex3d(minx,miny,minz);
	  glVertex3d(minx,miny,maxz);
	  glEnd();

	  glBegin(GL_LINES);
	  glVertex3d(maxx,miny,minz);
	  glVertex3d(maxx,miny,maxz);
	  glEnd();

	  glBegin(GL_LINES);
	  glVertex3d(maxx,maxy,minz);
	  glVertex3d(maxx,maxy,maxz);
	  glEnd();

	  glBegin(GL_LINES);
	  glVertex3d(minx,maxy,minz);
	  glVertex3d(minx,maxy,maxz);
	  glEnd();
	  
	  glPopMatrix();
	  return;
	}

    if (topDrawlocation!=NULL && doview->GetDocument()->show3do)
	{
		  //topDrawlocation->ClearDrawnStatus();
		  for(int i=0;i<topDrawlocation->size();i++)
		  {
			  Node * topDraw = (Node*) topDrawlocation->elementAt(i);
			  topDraw->DrawGL(hdc,this,doview,currentTexture);
		  }
	}

	if (doview->GetDocument()->showPoints)
	{
	 glPointSize(5.0f);
	 glBegin(GL_POINTS);
	 for(int i=0;i<numPts;i++)
	 {
	  Point3D *pt = (Point3D*)ptsArray[i];	  
	 
	  setGLColor(1.0f,0.0f,0.0f);
	  GLdouble wx,wy,wz;
	  wx = (double)pt->getX();
	  wy = (double)pt->getY();
	  wz = (double)pt->getZ();
	  glVertex3d(wx,wy,wz);
	 }
	 glEnd();
	}
	glPointSize(1.0f);
	
	if (doview->GetDocument()->showObjects)
	{
		 glPointSize(3.0f);
		 setGLColor(0.0f,1.0f,0.0f);
		 glBegin(GL_POINTS);
		 int objSize = objArray->GetCount(); 
		 for(int i=0;i<objSize;i++)
		 {
			POSITION pos = objArray->FindIndex(i);
		    Point3D *ptA = (Point3D*)objArray->GetAt(pos);
			
            GLdouble wx,wy,wz;
	        wx = (double)ptA->getX();
	        wy = (double)ptA->getY();
	        wz = (double)ptA->getZ();
			glVertex3d(wx,wy,wz);
		 }
		 glEnd();
		 glPointSize(1.0f);
	}

	if (selectedPt)
	{
		glPointSize(8.0f);
		setGLColor(0.0f,0.0f,1.0f);
		glBegin(GL_POINTS);
		GLdouble wx,wy,wz;
	    wx = (double)selectedPt->getX();
	    wy = (double)selectedPt->getY();
	    wz = (double)selectedPt->getZ();
		glVertex3d(wx,wy,wz);
		glEnd();
		setGLColor(0.0f,0.0f,0.0f);
		glPointSize(1.0f);
	}

	if (datFileParent && doview->GetDocument()->showCameras)
	{
	  CCam *cam = datFileParent->GetMainCameraFile();
	  if (cam)
	  {
		  cam->DrawGL(hdc,view);
	  }
	}

	if (datFileParent && doview->GetDocument()->showTrk)
	{
	    CTrk *tfile = (CTrk*)datFileParent->GetMainTrkFile();
		if (tfile)
		{
			if (ptsArray)
			{
			double x=ptsArray[BEGIN_INDEX]->getX();
			double y=ptsArray[BEGIN_INDEX]->getY();
			double z=ptsArray[BEGIN_INDEX]->getZ();
			tfile->DrawGL((CMy3DOEditorView*)view,x,y,z);
			}
		}
		else
		{
			cerr << "No Trk to draw" << endl;
		}
	}

	glPopMatrix();
}

#define PROFILE(x,y,b) x = AfxGetApp()->GetProfileInt(CString(#y), CString(#x),b);

void C3do::LoadTree(HTREEITEM parent,CObjectTree *tree)
{
	tree->m_ObjectTree.SetRedraw(FALSE);
	PROFILE(sortRepositoryType,3doPreferences,0);

	if (isGPLVersion() || isN2Version() || isICR2Version())
	{
	 int size = sections->size();	
	 if (size)
	 {
	  for(int i=0;i<size;i++)
	  {
	   Section *section = (Section*)sections->elementAt(i);
	   section->LoadTree(parent,tree,this); 
	  } 
	 }
	 else
	 {
	  subTree = tree->insertInfoNode(parent, (LPSTR)(LPCSTR)"subTree",TO_ID(IDB_OFFSET));
	 }
	}

	theNodeRepository.sortRepositoryType = sortRepositoryType;
	HTREEITEM repos = tree->insertInfoNode(parent, (LPSTR)(LPCSTR)"Node Repository",TO_ID(IDB_OFFSET));
    tree->m_ObjectTree.SetItemData(repos,(DWORD)&theNodeRepository);
	theNodeRepository.subTree = tree->insertInfoNode(repos, (LPSTR)(LPCSTR)"subTree",TO_ID(IDB_OFFSET));
	//theNodeRepository.LoadTree(repos,tree);
	tree->m_ObjectTree.SetRedraw(TRUE);
}

void Section::parsePrimTree(FILE *fp,HTREEITEM parent, CObjectTree *tree)
{
   CString buffer;
  
   buffer.Format("3DO Tree Root %d",RootOffset);
   HTREEITEM treeParent = tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_OFFSET));

   cerr << "Loading Prim Tree..." << endl;
   cerr.flush();   
   int start = GetTickCount();
   if (getObject()->primTree)
      getObject()->primTree->LoadTree(treeParent,tree);
   int end   = GetTickCount();
   
   for(int i=0;i<getObject()->numHandles;i++)
   {
    if (getObject()->extras[i])
    {
     int offsetColision = getObject()->Handles[i]->Offset.getVal();
	 CString name =  getObject()->findStrName(getObject()->Handles[i]->String.getVal());
     buffer.Format("%s Tree Root %d",name,offsetColision);
     HTREEITEM treeParentColision = tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_OFFSET));
     getObject()->extras[i]->LoadTree(treeParentColision,tree);
    }
   }

   cerr << "Creating 3DO prim Expandable Tree took " << (end-start) << " milliseconds "<< endl;
   cerr.flush();
}

int Node::getCommandVal(FILE *fp)
{
	   int cmdval;
	   if (object->getVersion() == t_N2)
	   {
		   cmdval = ReadPCDWord(fp);
		   int val = fgetc(fp);
		   //cerr << cmdval << "," << val << endl;
	   }
	   else if (object->getVersion() == t_ICR2)
	   {
		   int pos = ftell(fp);
		   cmdval = ReadPCDWord(fp);
	   }
	   else
	   { 
		   cmdval = ReadPCDWord(fp);
	   }
	   return cmdval;
}

int Node::getCommandValObj(FILE *fp,C3do *obj)
{
	   int cmdval;
	   if (obj->getVersion() == t_N2)
	   {
		   cmdval = ReadPCDWord(fp);
		   int val = fgetc(fp);
		   //cerr << cmdval << "," << val << endl;
	   }
	   else if (obj->getVersion() == t_ICR2)
	   {
		   int pos = ftell(fp);
		   cmdval = ReadPCDWord(fp);
	   }
	   else
	   { 
		   cmdval = ReadPCDWord(fp);
	   }
	   return cmdval;
}

void Node::sendCommandVal(FILE *fp,int cmdval)
{
	   if (object->getVersion() == t_N2)
	   {
		   WritePCDWord(fp,cmdval);
		   fputc(0x0,fp);
	   }
	   else
	   { 
		   WritePCDWord(fp,cmdval);
	   }
}

Node * Section::readPrim(int ReadOffset,int HandleOffset)
{
   if (HandleOffset)
	 fseek(fp,ReadOffset+HandleOffset,SEEK_SET);
   else
     fseek(fp,ReadOffset+RootOffset,SEEK_SET);

   //object->trackBufferPos = ftell(fp);

   try
   {
#ifdef _DEBUG
	   cerr << "Reading 3DO..";
	   int start = GetTickCount();
	   if (object->getVersion() == t_N2) cerr << " N2 3DO Detected" << endl;
#endif
	   Node *parent = getChildType(ReadOffset,NULL,fp,object);
#ifdef _DEBUG
	   int end   = GetTickCount();
	   cerr << "Reading 3DO prim tree took " << (end-start) << " milliseconds "<< endl;
	   totals();
	   cerr.flush();
#endif
	   return parent;
   }
   catch(C3doErrorException *e)
   {
	   AfxMessageBox(e->toString());
   }
   return NULL;
   
}

void Section::readCmd(HTREEITEM parent, CObjectTree *tree)
{
   AfxMessageBox("Dont readCmd Anymore");

   
}

/*
void DOPolygon::LoadTree(HTREEITEM parent,CObjectTree *tree, C3do *tdo)
    {
		if (textureidx == -1 && tdo)
		{
		   if (tdo->strs)
		   {
			 if (strid)
			 {
			  int ssize = tdo->strs->size();
			  for(int i=0;i<ssize;i++)
			  {
			   CImageFile *file = (CImageFile*)tdo->strs->elementAt(i);
			   if (file->offset == strid)
			   {
			    textureidx = i;
			    break;
			   }
			  }
			 }
			 else
			 {
			   textureidx = 0;
			 }
		 }
		}

		if (points)
		{
		  
		 char *str;

		 if (tdo->strs && strid!=-1)
		 {
		   CImageFile *file = (CImageFile*)tdo->strs->elementAt(strid);
		   if (file)
		   {
		     str = (LPSTR)file->filename;
		   }
		   else
		   {
			 str = strdup("????");
		   }
		 }
		 else
		 {
			str = strdup("???");
		 }
	     int psize = points->size();
		 CString buffer;
		 buffer.Format("Polygon pts=%d texture=%d color=%X strnum=%d texture=%s",psize,textureidx,colorref,strid,str);
		 tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_OFFSET));
		}
	}
*/


int ReadPCDWord(FILE *fp)
{
	    int PCword=0;
		int len = fread(&PCword,1,4,fp);		
		return PCword;	     
}

float ReadPCFloat(FILE *fp)
{
	float data;

	fread(&data,1,sizeof(float),fp);
	return data;
}

int ReadPCDWordRev(FILE *fp)
{
        unsigned char data[4];
        int  PCword;

		fread(data,1,4,fp);

        PCword = data[0]*256*256*256 + 
			     data[1]*256*256+
			     data[2]*256 +
                 data[3];
        return 0xFFFFFFFF & PCword;
}



int ReadPCWord(FILE *fp)
{
        unsigned char data[2];
        int  PCword;

		fread(data,1,2,fp);

        PCword = data[1]*256 +
                 data[0];
        return 0x0000FFFF & PCword;
}

void WritePCWord(FILE *fp,int value)
{
        unsigned char data[2];

		data[1] = (int)(value/256);
		data[0] = value-(data[1]*256);

		fwrite(data,1,2,fp);
}

void WritePCFloat(FILE *fp,float data)
{
	int idx = fwrite(&data,1,4,fp);
	if (idx!=4) AfxMessageBox("Write Error");
}

void WritePCDWord(FILE *fp,int value)
{
        unsigned char data[4];

		data[3] = HIBYTE(HIWORD(value));
		data[2] = LOBYTE(HIWORD(value));
		data[1] = HIBYTE(LOWORD(value));
		data[0] = LOBYTE(LOWORD(value));

		fwrite(data,1,4,fp);
}

void C3do::UpdateTree(Node *updateNode)
{
	    CString buffer;
	    CObjectTree *tree = ((CMy3DOEditorApp*)AfxGetApp())->m_MainTree;
		HTREEITEM tp = tree->m_ObjectTree.GetParentItem(FileNode);
		tree->m_ObjectTree.DeleteItem(FileNode);
		FileNode = NULL;
		buffer.Format("3DO File %s",filename);
	    FileNode = tree->insertInfoNode(tp, (LPSTR)(LPCSTR)buffer,TO_ID(FileNameToIcon((LPSTR)(LPCSTR)filename)));
		tree->m_ObjectTree.SetItemData(FileNode,(DWORD)this);
		try {
			LoadTree(FileNode,tree);
		}
		catch(C3doErrorException *e)
		{
			AfxMessageBox(e->toString());
		}
		if (updateNode)
		{
		 Node *root = updateNode->getRoot(); 
		 tree->m_ObjectTree.Expand(FileNode,TVE_EXPAND);

		 for(int i=0;i<sections->size();i++)
		 {
			 Section *sec = (Section*)sections->elementAt(i);
			 if (sec && sec->isPRIM())
			 {
				 tree->m_ObjectTree.Expand(sec->SectionNode,TVE_EXPAND);				 
				 HTREEITEM rootParentItem = tree->m_ObjectTree.GetChildItem(sec->SectionNode);
				 int count = 1+numHandles;
				 
				 while(count)
				 {
			       count--;  
				   tree->m_ObjectTree.Expand(rootParentItem,TVE_EXPAND);				 
				   HTREEITEM rootItem = tree->m_ObjectTree.GetChildItem(rootParentItem);
				   if (rootItem)
				   {
					 Node *rootWord = (Node *)tree->m_ObjectTree.GetItemData(rootItem);
					 if (rootWord == root){
						 tree->m_ObjectTree.SetRedraw(FALSE);
						 updateNode->ExpandToRoot(tree);
						 tree->m_ObjectTree.EnsureVisible(updateNode->treeNode);
						 tree->m_ObjectTree.SetRedraw(TRUE);
						 break;
					 }

				   }
				   rootParentItem = tree->m_ObjectTree.GetNextSiblingItem(rootParentItem);
				 }
			 }
		 }
		 //tree->m_ObjectTree.Expand(t->treeNode,TVE_EXPAND);
		 //HTREEITEM htree = updateNode->treeNode;
		 //tree->m_ObjectTree.Select(htree,TVGN_CARET);
		}
}

void C3do::addSection(Section *sec)
{
		sections->addElement(sec);
}

void CGplFile::SaveAs(LPCSTR filename)
{
	   if (filename==NULL)
	   {
	   char fileName[256];
	   strcpy(fileName,"unnamed");
	   char *ext = strdup((LPSTR)(LPCSTR)GetDefaultExtension());
	   CFileDialog *fdlg = new CFileDialog(FALSE,"*",(LPCSTR)&fileName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,lpszGPLFilter);

	   int result = fdlg->DoModal();

	   if (result == IDOK)
	   {
	    FILE *fpout = FOPEN(fdlg->GetFileName(),"wb");
	    if (fpout==NULL)
		{
		   CString msg;
		   msg.Format("Cannot Open %s for writing",fdlg->GetFileName());
		   AfxMessageBox(msg);
		   return;
		}
	    Write(fpout);
		if (isA(t_tdo))
		{
			C3do *tdoFile = (C3do*)this;
			tdoFile->WritePtAliasName(fdlg->GetFileName());
		}
	    fflush(fpout);
	    FCLOSE(fpout);
	   }
	   if (fdlg) delete fdlg;
	   free(ext);
	   }
	   else
	   {
		FILE *fpout = FOPEN(filename,"wb");
	    if (fpout==NULL)
		{
		   CString msg;
		   msg.Format("Cannot Open %s for writing",filename);
		   AfxMessageBox(msg);
		   return;
		}
	    Write(fpout);
	    fflush(fpout);
	    FCLOSE(fpout);
	   }
   }



int C3do::GetVertexPoint(FILE *fp,int vertexOffset,BOOL texture)
{
	int foff = ftell(fp);
	int idx = 0;

	fseek(fp,ICR2DataSectionStart+vertexOffset-4,SEEK_SET);

	Point3D *pt = new Point3D();
	pt->object = this;
	pt->ReadICR2(fp,texture);
	addPoint(pt);
	idx = numPts-1;
	pt->idx = idx;

	fseek(fp,foff,SEEK_SET);
	return idx;
}


void C3do::SecondPass(Node *root)
{
	cerr << "Second Pass...." << NodePtrs->size() << " Unconnected Nodes" << endl;
	if (root)
	{
	 int posRoot = root->getOffset();
	 for(int i=0;i<NodePtrs->size();i++)
	 {
		NodePtr *ptr = (NodePtr*)NodePtrs->elementAt(i);
		int fpos = posRoot+ptr->ptr;
		Node *fptr = theNodeRepository.FindNode(fpos);
		if (fptr)
		{
			ptr->pointToNode = fptr;
		}
		else
		{
			cerr << "Cannot find Unconnected Node " << ptr->ptr << endl;
		}
	 }
	}
}

/*
int C3do::ReadPCWord(FILE *fp)
{
	//int pos = ftell(fp);
	int res = ::ReadPCWord(fp);
	//eek(fp,2,SEEK_CUR);
	trackBufferPos+=2;
	return res;
}

int C3do::ReadPCDWord(FILE *fp)
{
	//int pos = ftell(fp);
	//int res = ::ReadPCDWord(fp);
	
	int *data = (int*)&trackBuffer[trackBufferPos];
	//fseek(fp,4,SEEK_CUR);
	trackBufferPos+=4;
	return *data;
}

float C3do::ReadPCFloat(FILE *fp)
{
	//int pos = ftell(fp);
	//int res = ::ReadPCDWord(fp);
	float *data = (float*)&trackBuffer[trackBufferPos];
	trackBufferPos+=4;
	//fseek(fp,4,SEEK_CUR);
	return *data;	
}


int C3do::FRead(void *data,int size,int number,FILE *fp)
{
	//int pos = ftell(fp);
	int memsize = size*number;
	memcpy(data,&trackBuffer[trackBufferPos],memsize);
	trackBufferPos+=memsize;
	//fseek(fp,memsize,SEEK_CUR);
	return memsize;
}
*/

bool isPolygon(Node *ptr)
{
	int type = ptr->type;

	if (type == 0x2) return true;
	if (type == 0x80000001) return true;
	if (type == 0x80000002) return true;	
	if (type == 0x81b) return true;	
	if (type == 0x81c) return true;	
	if (type == 0x81d) return true;	
	if (type == 0x81e) return true;	
	if (type == 0x81f) return true;	
	if (type == 0x820) return true;	
	if (type == 0x821) return true;	
	return false;
}


void C3do::ExportDataTxt(CString filename)
{
	ofstream file(filename);

	file << "# 3DO Export" << endl;

	file << numPts << endl;
	for(int i=0;i<numPts;i++)
	{
		file << "p " ;
		Point3D *ptA = ptsArray[i];
		file << ptA->getX() << " " << ptA->getY() << " " << ptA->getZ() << endl;
	}


	POSITION pos = theNodeRepository.repository->GetStartPosition();

	while(pos !=NULL)
	{
		 int key;
		 Node *ptr;
		 theNodeRepository.repository->GetNextAssoc(pos,key,ptr);
		 if (isPolygon(ptr))
		 {
			 NodePolygon *polygon = (NodePolygon*)ptr;
			 file << "v " ;
			 file << polygon->numberOfVerticies << " " ;
			 file << polygon->color << " ";
			 for(int i=0;i<polygon->numberOfVerticies;i++)
			 {
				 file << polygon->verticies[i] << " ";
			 }
			 file << endl;
		 }
	}

	file << "v -1" << endl;

	file.close();
}

//extern char * getline(FILE *fp);

void C3do::ExportSubTree(Node *top)
{
	ofstream file("testnode.txt");

	top->Print(file);

	file.close();
}


void C3do::ImportDataTxt(CString filename)
{
	FILE *fp = fopen(filename,"rb");

	if (fp==NULL)
	{
		AfxMessageBox("Cannot Open Import File");
		return;
	}

	int replace = AfxMessageBox("Do you want to replace or add the Points (Yes for replace/ No for add)",MB_YESNO);


	char *comment = getline(fp);

	char *pts = getline(fp);
	int npts;
	sscanf(pts,"%d",&npts);

	for(int i=0;i<npts;i++)
	{
		float x,y,z;
		char *pts = getline(fp);
		sscanf(pts,"%*s %f %f %f",&x,&y,&z);
		Point3D *newpt = new Point3D(x,y,z);
		if (replace == IDYES)
		{
		 if (i >= numPts)
		 {
		  addPoint(newpt);
		 }
		 else{
		  ptsArray[i] = newpt;
		 }
		}
		else
		{
		 addPoint(newpt);
		}
	}

	POSITION pos = theNodeRepository.repository->GetStartPosition();

	//pts = getline(fp);
	//int vpoly;
	//sscanf(pts,"%d",&vpoly);
	while(true)
	{
		int vs,col,ipos;
		char *pts = getline(fp);
		sscanf(pts,"%*s %d%n",&vs,&ipos);
		if (vs == -1) break;
		pts+=ipos;
		sscanf(pts,"%d%n",&col,&ipos);
		pts+=ipos;
		int *vx = new int[vs];
		for(int j=0;j<vs;j++)
		{
			int v;
			sscanf(pts,"%d%n",&v,&ipos);
			pts+=ipos;
			vx[j] = v;
		}

		if (pos)
		{
		  int key;
		  Node *ptr;
		  do {          
		    theNodeRepository.repository->GetNextAssoc(pos,key,ptr);
		  } while(pos!=NULL  && !isPolygon(ptr));

		  //ptr should be a polygon
		  if (ptr && isPolygon(ptr))
		  {
			  NodePolygon *poly = (NodePolygon*)ptr;
			  poly->numberOfVerticies = vs;
			  poly->verticies = new int[poly->numberOfVerticies];
			  for(int j=0;j<vs;j++)
			  {
				  poly->verticies[j] = vx[j];
			  }
		  }
		}
	}

	while(pos)
	{
		int key;
		  Node *ptr;
		  do {          
		    theNodeRepository.repository->GetNextAssoc(pos,key,ptr);
		  } while(pos!=NULL  && !isPolygon(ptr));

		  //ptr should be a polygon
		  if (ptr && isPolygon(ptr))
		  {
			  NodePolygon *poly = (NodePolygon*)ptr;
			  poly->numberOfVerticies = 3;
			  poly->verticies = new int[poly->numberOfVerticies];
			  poly->verticies[0] = 0;
			  poly->verticies[1] = 1;
			  poly->verticies[2] = 2;
		  }
	}
		

	CString msg;
	msg.Format("Added %d Points",npts);
	AfxMessageBox(msg);
}

void C3do::ExportDataObj(CString filename)
{
	ofstream file(filename);

	//file << "# 3DO Export" << endl;


	//file << numPts << endl;
	//file.precision(3);
	//file.width(3);
	file.setf(ios::fixed,ios::floatfield);
	for(int i=0;i<numPts;i++)
	{
		file << "v " ;
		Point3D *ptA = ptsArray[i];
		file << ptA->getX() << " " << ptA->getY() << " " << ptA->getZ() << endl;
	}


	POSITION pos = theNodeRepository.repository->GetStartPosition();

	while(pos !=NULL)
	{
		 int key;
		 Node *ptr;
		 theNodeRepository.repository->GetNextAssoc(pos,key,ptr);
		 if (isPolygon(ptr))
		 {
			 NodePolygon *polygon = (NodePolygon*)ptr;
			 file << "f" ;
			 //file << polygon->numberOfVerticies << " " ;
			 //file << polygon->color << " ";
			 for(int i=0;i<polygon->numberOfVerticies;i++)
			 {
				 file << " ";
				 file << (1+polygon->verticies[i]);
			 }
			 file << endl;
		 }
	}

	//file << "v -1" << endl;

	file.close();
}