
#ifndef _INCLUDED_3DO_H_
#define _INCLUDED_3DO_H_

#include "CGPLFile.h"
#include "Section.h"
#include "Point3D.h"
//#include "Polygon.h"

#include "Pallette.h"
#include "ImageFile.h"
#include "Planes.h"
#include "NodeRepository.h"

typedef struct 
{
  char id[4];
  int unk1;
  int FileSize;
}t_Header;

#define MAX_POLYGONS 10
#define MAX_PTS      10

class Node;
class DataValue;
class Section;

typedef enum { t_GPL, t_N2, t_ICR2 } t_Version;

class C3do: public CGplFile
{
	t_Version version;
	
public:
	t_Version getVersion()
	{
		return version;
	}

	C3do(CDocument *Doc):pDoc(Doc),CGplFile(t_tdo)
	{
		strs     = new Vector("strings",__LINE__);
	    sections = new Vector("Sections",__LINE__);
		Plans = NULL;
		ptsArray = NULL;
		numPolygons = 0;
		firstGLDraw = TRUE;
		objArray     = new CObList();
		//mallocedPolygons = 8;
		//polygonArray = (DOPolygon**)malloc(mallocedPolygons*sizeof(DOPolygon*));
		//linesArray = NULL;
		numLines = 0;
		//mallocedLines = 1;
		//linesArray = (DOLine**)malloc(mallocedLines*sizeof(DOLine*));
		topDrawlocation = new Vector("TopDrawLocations",__LINE__);
		numPts = 0;
		ask3do = FALSE;
		primTree = NULL;
		extras = NULL;
		showOffset = TRUE;
		version = t_GPL;
		selection = NULL;
		selectedPt = NULL;
		sortRepositoryType = 0;
		ptAliases = new Vector("aliases",__LINE__);
		NodePtrs = new Vector("unconnected",__LINE__);
		WriteAsOverLay = FALSE;
		SeenANodeFifteen =FALSE;
		trackBuffer = NULL;
		Normals = NULL;
	}

	BOOL isGPLVersion()
	{
		return (version == t_GPL); 
	}

	BOOL isN2Version()
	{
		return (version == t_N2); 
	}

	BOOL isICR2Version()
	{
		return (version == t_ICR2); 
	}

	~C3do();


	CString GetDefaultExtension()
	{
		return CString(".3do");
	}

	int GetVertexPoint(int vertexOffset,BOOL texture);

	BOOL ReadPtAliasName(const char *filename);
	BOOL WritePtAliasName(const char *filename);
	BOOL Read(const char *filename);
	BOOL Read(FILE *fp);
	BOOL ReadICR2(FILE *fp);
	BOOL Write(FILE *fp);
	BOOL Write(const char *filename);
	BOOL WriteICR2(FILE *fp);
	void GetDist(CView *view);

	BOOL WriteAsOverLay;
	BOOL SeenANodeFifteen;

	BOOL UpdateOrAddAlias(int id,char *alias);

	void ReMaxMin();

	void addSection(Section *sec);

	Vector *sections;
	Vector *strs;
	Vector *ptAliases;
	BOOL	ask3do;
	
	CPlanes **Plans;
	int numPlanes;

	CHandles **Handles;
	int numHandles;

	CNormals **Normals;
	int numNormals;

	t_Header   Header;

	Vector * topDrawlocation;
	Node * primTree;
	Node * selection;
	Node ** extras;

	C3do * FindObjectFile(CString name)
	{
	   if (datFileParent)
	   {
	     return (C3do*)datFileParent->FindObjectFile(name);
	   }
	   else
	   {
		 C3do *tdo = new C3do(NULL);
		 CString filename = name;
		 filename+=".3do";
		 if (tdo->Read(filename))
		 {
			 return tdo;
		 }    
	   }
	   return NULL;
	}

	
	Point3D   **ptsArray;
	int       numPts;

	Point3D *selectedPt;
	
	CObList *objArray;
	
	void addObj(float x,float y,float z)
	{
		Point3D *pt = new Point3D();
		
		pt->setX(x);
		pt->setY(y);
		pt->setZ(z);
		objArray->AddTail(pt);
	}

	void addPoint(Point3D *pt)
	{
		numPts++;
		ptsArray = (Point3D**)realloc(ptsArray,numPts*sizeof(Point3D*));
		ptsArray[numPts-1] = pt;
		pt->idx = numPts-1;
	}

	
	void addPlane(CPlanes *pt)
	{
		numPlanes++;
		Plans = (CPlanes**)realloc(Plans,numPlanes*sizeof(CPlanes*));
		Plans[numPlanes-1] = pt;
	}

	void addNormal(CNormals *pt)
	{
		if (numNormals == 0)
		{
		 numNormals++;
		 Normals = (CNormals**)malloc(numNormals*sizeof(CNormals*));
		}
		else
		{
		numNormals++;
		Normals = (CNormals**)realloc(Normals,numNormals*sizeof(CNormals*));
		}
		Normals[numNormals-1] = pt;
	}

	CImageFile * addString(char *filename)
	{
		int offset=0;
		for(int i=0;i<strs->size();i++)
		{
			CImageFile *f = (CImageFile*)strs->elementAt(i);
			offset+=f->WriteSize();
		}
		CImageFile *file = new CImageFile(datFileParent,strdup(filename));
		file->offset = offset;
		strs->addElement(file);
		return file;
	}


	void GetMinMax(float *minx,float *miny,float *maxx,float *maxy)
	{
		if (numPts)
		{
		*minx = 10E6f;
		*miny = 10E6f;
		*maxx = -10E6f;
		*maxy = -10E6f;
		
		for(int i=0;i<numPts;i++)
		{
		   *minx = min(*minx,ptsArray[i]->getX());
		   *miny = min(*miny,ptsArray[i]->getY());		   
		   *maxx = max(*maxx,ptsArray[i]->getX());
		   *maxy = max(*maxy,ptsArray[i]->getY());  
		}
		}
		else
		{
			*minx = (float)-1024.0;
		    *miny = (float)-1024.0;
		    *maxx = (float)1024.0;
		    *maxy = (float)1024.0;
		}
	}

	void getTextureFile()
	{
		for(int i=0;i<strs->size();i++)
		{

		}
	}

	void Expand(CObjectTree *tree,HTREEITEM node)
	{
		   if (subTree){
			   tree->m_ObjectTree.DeleteItem(subTree);
			   GetFile();
			   LoadTree(node,tree);
			   subTree = NULL;
		   }
	}

	void UpdateTree(Node *updateNode);

	

	void Explore();
	
	void DrawGL(HDC hdc,CView *view,int currentTexture);
	void DrawGLText(HDC hdc,CView *view);

	void LoadTree(HTREEITEM parent,CObjectTree *tree);

	BOOL firstGLDraw;

	double minx,miny,minz;
	double maxx,maxy,maxz;
	double midx,midy,midz;

	void SecondPass(Node *root);

	/*
	void addPolygon(DOPolygon *poly)
	{
		if (numPolygons >= mallocedPolygons)
		{		   
		   mallocedPolygons=mallocedPolygons*2;
		   TRACE1("Reallocing Polygons to %d\n",mallocedPolygons);
		   polygonArray = (DOPolygon**)realloc(polygonArray, mallocedPolygons*sizeof(DOPolygon*));
		}
		polygonArray[numPolygons]=poly;
		numPolygons++;
	}

	void addLine(DOLine *line)
	{
		if (numLines >= mallocedLines)
		{		   
		   mallocedLines=mallocedLines*2;
		   TRACE1("Reallocing Lines to %d\n",mallocedLines);
		   linesArray = (DOLine**)realloc(linesArray, mallocedLines*sizeof(DOLine*));
		}
		linesArray[numLines++] = line;
	}
	*/

	CString findStrName(int strid)
	{
		if (strs)
		{
	          int ssize = strs->size();
			  for(int i=0;i<ssize;i++)
			  {
			   CImageFile *file = (CImageFile*)strs->elementAt(i);
			   if (file->offset == strid)
			   {
			    return CString(file->filename);
			   }
			  }
		}
		return CString("Unk");
	}

	int findStrNameId(int strid)
	{
		if (strs)
		{
	          int ssize = strs->size();
			  for(int i=0;i<ssize;i++)
			  {
			   CImageFile *file = (CImageFile*)strs->elementAt(i);
			   if (file->offset == strid)
			   {
			    return i;
			   }
			  }
		}
		return -1;
	}


	int mallocedPolygons;
	int mallocedLines;

	int getNumPolygons()
	{
		return numPolygons;
	}

	int WriteRoofOffsetPosition;
	int RootOffset;
	int RootBegin;

	BOOL showOffset;

	NodeRepository theNodeRepository;
	Vector * NodePtrs;

	CDocument *pDoc;

	int sortRepositoryType;
	int ICR2DataSectionStart;

	// ICR2 stuff
	int GetVertexPoint(FILE *fp,int vertexOffset,BOOL texture);
	int ICR2DataSize;
	int ICR2MipFiles;
	int ICR2PmpFiles;
	int ICR2ThreeDOFiles;
	int ICR2Zero;

	unsigned char *trackBuffer;
	int            trackBufferSize;
	//int			   trackBufferPos;

	//int ReadPCWord(FILE *fp);
	//int ReadPCDWord(FILE *fp);
	//float ReadPCFloat(FILE *fp);
	//int FRead(void *data,int size,int number,FILE *fp);

	void ExportDataTxt(CString filename);
	void ImportDataTxt(CString filename);
	void ExportSubTree(Node *top);
	void ExportDataObj(CString filename);

private:
	int numPolygons;
	int numLines;
	
	
};


#endif