#include "stdafx.h"
#include "CMip.h"
#include "stdio.h"
#include "Palette.h"

#include "MipPreview.h"

extern int ReadPCDWord(FILE *fp);

BOOL CMip::Read(const char *filename)
{
	FILE *fp = FOPEN(filename,"rb");

	if (fp == NULL)
	{
		CString msg;
		msg.Format("Cannot find file: %s",filename);
		//AfxMessageBox(msg);
		return FALSE;
	}
	Read(fp);
	FCLOSE(fp);
	return TRUE;
}

char * getSectionHeader(FILE *fp)
{
	int foff = ftell(fp);
	char *sectionId = (char*)malloc(5*sizeof(char));
	memset(sectionId,0,5);
	fread(sectionId,1,4,fp);

	// rewind
	fseek(fp,foff,SEEK_SET);
	return sectionId;
}

BOOL CMip::ReadICR2Mip(FILE *fp)
{
	int filesize = ReadPCDWord(fp); // always zero
	ver = ReadPCDWord(fp); // always zero
	logx      = ReadPCDWord(fp);
	logy      = ReadPCDWord(fp);
	numimages = ReadPCDWord(fp);
	color     = ReadPCDWord(fp);

	for(int i=0;i<numimages;i++)
	{
		int ImageVal       = ReadPCDWord(fp);
		int NImageVal      = ReadPCDWord(fp);
		int ImageStart     = ReadPCDWord(fp);
		cerr << "ImageStart "<< i << " " << ImageVal << "," << NImageVal << endl;
	}
	return TRUE;
}

BOOL CMip::Read(FILE *fp)
{
	if (dataLoaded) return TRUE;

	int foff = ftell(fp);

	t_Mip_Header Header;
	fread(&Header,1,sizeof(t_Mip_Header),fp);

	if (Header.id[0]!='M' && Header.id[1]!='I')
	{
		if (Header.id[1]!='P' && Header.id[2]!='I')
		{
		 
		 cerr << "Must be an ICR2 Mip" << endl;
		 version = t_MIPICR2;
		 // move back to the beginning
		 fseek(fp,foff,SEEK_SET);
		}
		else
		{
		 cerr << "N2 Mip File" << endl;
		 version = t_MIPN2;
		}
	}
	else
	{
		version = t_MIPGPL;
	}
	
	// ICR2 has a different header
	if (isICR2Version())
	{		
		return ReadICR2Mip(fp);
	}
	else
	{
		ver       = getc(fp);
		logx      = ReadPCDWord(fp);
		logy      = ReadPCDWord(fp);
		color     = ReadPCDWord(fp);
		unknown   = getc(fp);
		numimages = ReadPCDWord(fp);
		unknown   = getc(fp);
		unknown   = getc(fp);
	
	}
	
	t_CMap_Header CHeader;
	t_BMap BHeader;
	t_BMPH IHeader;

	BOOL  LoadedCMAP = FALSE;
	BOOL  LoadedDATA = FALSE;

	while(!LoadedDATA)
	{
	char *sectionId = getSectionHeader(fp);

	if (!strcmp(sectionId,"PAMC"))
	{	
     fread(&CHeader,1,sizeof(t_CMap_Header),fp);
	 RGBQUAD *pal = new RGBQUAD[256];
	 for(int i=0;i<16;i++)
	 {
			pal[i].rgbRed   = CHeader.palette[i].r;
			pal[i].rgbGreen = CHeader.palette[i].g;
			pal[i].rgbBlue  = CHeader.palette[i].b;
	 }
	 bitmapData.SetPalette(pal);
	 LoadedCMAP = TRUE;
	 delete pal;	 
	}
	else if (!strcmp(sectionId,"PAMB"))
	{
	 fread(&BHeader,1,sizeof(t_BMap),fp);
	}
	else if (!strcmp(sectionId,"DHMB"))
	{
	 fread(&IHeader,1,12,fp);
	 IHeader.ver       = getc(fp);
	 IHeader.x         = ReadPCDWord(fp);
	 IHeader.y         = ReadPCDWord(fp);
	 IHeader.width     = ReadPCDWord(fp);
     IHeader.unknown   = getc(fp);		
	 IHeader.unknown   = getc(fp);		
	 IHeader.unknown   = getc(fp);
	}
	else if (!strcmp(sectionId,"ATAD"))
	{
	    t_DATA DHeader;
		fread(&DHeader,1,sizeof(t_DATA),fp);
		int dataSize = (IHeader.x*IHeader.y);
		unsigned char *image = NULL;

		if (dataSize >= 0xFFFFFF)
		{
			TRACE1("Mallocing Large Data %d",dataSize);
		}

		image = new unsigned char[dataSize];		
		fread(image,1,dataSize,fp);
		BOOL useRgb = ((IHeader.ver >= 2) ? TRUE : FALSE);
		unsigned char *image8 = NULL;

		if (useRgb)
		{
		  image8 = new unsigned char[3*dataSize];
		}
		else
		{
		  image8 = new unsigned char[2*dataSize];
		}

		//0 cmap 4bit 
		//1 cmap 4bit 
		//2 ?? ?? 
		//3 rgb 565 
		//4 prgb 1555 
		//5 rgba 4444 
		

		int count = 0;
		if (IHeader.ver == 0 || IHeader.ver == 1)
		{
		 for(int i=0;i<2*dataSize;i+=2)
		 {	
			image8[i+1]   = 1+(0x0F & image[count]);
			image8[i] = 1+((0xF0 & image[count]) >> 4);			
			count++;
		 }
		}
		else if (IHeader.ver == 4)
		{
		  for(int i=0;i<dataSize;i++)
		  {
			  int value = image[i];
			  int p = (0x4000 & value) >> 15;
			  int r = (0x7C00 & value) >> 10;
			  int g = (0x3E0 & value) >> 5;
			  int b = (0x1F & value);
			  image8[count]   = r*7;
			  image8[count+1] = g*7;
			  image8[count+2] = b*7;
			  if (count >= (3*dataSize)) break;
			  count+=3;
		  }
		}
		else if (IHeader.ver == 5)
		{
		  for(int i=0;i<dataSize;i++)
		  {
			  int value = image[i];
			  int p = (0xF000 & value) >> 12;
			  int r = (0xF00 & value) >> 8;
			  int g = (0xF0 & value) >> 4;
			  int b = (0x0F & value);
			  image8[count]   = r*0xF;
			  image8[count+1] = g*0xF;
			  image8[count+2] = b*0xF;
			  if (count >= (3*dataSize)) break;
			  count+=3;
		  }
		}
		else if (IHeader.ver == 3)
		{
		  for(int i=0;i<dataSize;i++)
		  {
			  if (isN2Version())
			  {
			  int value = image[i];
			  int r = colorIndex[value].r;
			  int g = colorIndex[value].g;
			  int b = colorIndex[value].b;
			  if (value != 0 && value != 31)
			  {
				  if (r == 0 && g == 0 && b ==0 )
				  {
					  cerr << "(" << value <<")";
				  }
			  }
			  image8[count]   = b;
			  image8[count+1] = g;
			  image8[count+2] = r;
			  if (count >= (3*dataSize)) break;
			  count+=3;
			  }
			  else
			  {
			  int value = image[i];
			  int r = (0xF800 & value) >> 11;
			  int g = (0x7E0 & value) >> 5;
			  int b = (0x1F & value);
			  image8[count]   = r*0x7;
			  image8[count+1] = g*0x4;
			  image8[count+2] = b*0x7;
			  if (count >= (3*dataSize)) break;
			  count+=3;
			  }
		  }
		}
		else if (IHeader.ver == 2)
		{
		  for(int i=0;i<dataSize;i++)
		  {
			  int value = image[i];
			  int r = colorIndex[value].r;
			  int g = colorIndex[value].g;
			  int b = colorIndex[value].b;
			  image8[count]   = b;
			  image8[count+1] = g;
			  image8[count+2] = r;
			  if (count >= (3*dataSize)) break;
			  count+=3;
		  }
		}
		else
		{
		  CString msg;
		  msg.Format("Cannot Handle this type of imagefile ver=%d",IHeader.ver);
		  AfxMessageBox(msg);
		}
		bitmapData.Create((unsigned short)IHeader.x,(unsigned short)IHeader.y,image8,useRgb);
		delete image8;
		delete image;
		LoadedDATA = TRUE;
	 }
	 free(sectionId);
	 }	 
    return TRUE;
}


void CMip::Explore()
{
	GetFile();

    if (dataLoaded)
	{
	   CMipPreview *dlg = new CMipPreview();
	   dlg->nWidth  = bitmapData.ImageWidth();
	   dlg->nHeight = bitmapData.ImageHeight();
	   bitmapData.CreateDIB(dlg->bmp);
	   dlg->DoModal();
	}
	else
	{
	   AfxMessageBox("File Cannot be Loaded");
	}
}