// DatDocument.cpp : implementation file
//

#include "stdafx.h"
#include "3DOEditor.h"
#include "DatDocument.h"
#include "ObjectTree.h"
#include "MipList.h"
#include "CGPLFile.h"
#include "Progress.h"
#include "ProjectSetting.h"
#include "3doSelector.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static char lpszTabFilter[] = "Comma Separated Text(*.csv)|*.csv|All Files (*.*)|*.*|";

/////////////////////////////////////////////////////////////////////////////
// CDatDocument

IMPLEMENT_DYNCREATE(CDatDocument, CDocument)

CDatDocument::CDatDocument()
{
	datFile   = NULL;
	trackFile = NULL;
	FileNode  = NULL;
}

BOOL CDatDocument::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	return TRUE;
}

CDatDocument::~CDatDocument()
{
	if (FileNode) ((CMy3DOEditorApp*)AfxGetApp())->m_MainTree->m_ObjectTree.DeleteItem(FileNode);
	FileNode = NULL;
	if (datFile) delete datFile;
}


BEGIN_MESSAGE_MAP(CDatDocument, CDocument)
	//{{AFX_MSG_MAP(CDatDocument)
	ON_COMMAND(ID_VIEW_MIPLIST, OnViewMiplist)
	ON_COMMAND(ID_EDIT_UNPACKALL, OnEditUnpackall)
	ON_COMMAND(ID_FILE_EXPORTTRKTOEXCEL, OnFileExporttrktoexcel)
	ON_COMMAND(IDC_BUILD, OnBuild)
	ON_COMMAND(ID_FILE_SAVEALL, OnFileSaveall)
	ON_COMMAND(ID_BUILD_SETTINGS, OnBuildSettings)
	ON_COMMAND(ID_FILE_EXPORTCAMTOEXCEL, OnFileExportcamtoexcel)
	ON_COMMAND(ID_EDIT_UPDATETRACK, OnEditUpdatetrack)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDatDocument diagnostics

#ifdef _DEBUG
void CDatDocument::AssertValid() const
{
	CDocument::AssertValid();
}

void CDatDocument::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDatDocument serialization

void CDatDocument::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDatDocument commands

BOOL CDatDocument::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;
	
	// TODO: Add your specialized creation code here
	HTREEITEM RootNode = ((CMy3DOEditorApp*)AfxGetApp())->m_MainTree->m_ObjectTree.GetRootItem();

	CString buffer;
	buffer.Format("TrackFile File %s",lpszPathName);
	FileNode = ((CMy3DOEditorApp*)AfxGetApp())->m_MainTree->insertInfoNode(RootNode, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_FILE));

	datFile = new CDat();

	CProgress *dlg = new CProgress();
	if (datFile->isPackFileName(lpszPathName))
	{
	  dlg->UpdateProgress("Reading Pck Project file",0);
	}
	else
	{
	  dlg->UpdateProgress("Reading Dat file",0);
	}

	datFile->Read(lpszPathName);
	datFile->LoadTree(((CMy3DOEditorApp*)AfxGetApp())->m_MainTree,FileNode);

	dlg->UpdateProgress("Reading Track.3do (Graphical Definition)",30);
	// get the main track .3do file
	trackFile =  (C3do*)datFile->GetMainTrackFile();
	if (trackFile== NULL)
	{
		cerr << "Cannot Find Main Track.3do file (Non track file type)" << endl;
	}
	else
	{
		trackFile->ask3do = TRUE;
		trackFile->GetFile();
		trackFile->ask3do = FALSE;
	}

		dlg->UpdateProgress("Reading Track.cam (Cameras)",70);
	cameraFile =  (CCam*)datFile->GetMainCameraFile();
	if (cameraFile== NULL)
	{
		cerr << "Cannot Find Main Camera file" << endl;
	}
	else
	{
		cameraFile->GetFile();
	}

	dlg->UpdateProgress("Reading Track.trk (Track Definition)",80);
	trkFile =  (CTrk*)datFile->GetMainTrkFile();
	if (trkFile== NULL)
	{
		cerr << "Cannot Find Main Trk file" << endl;
	}
	else
	{
		trkFile->GetFile();
	}

	dlg->UpdateProgress("Reading Track.trk (Track Definition)",100);
	if (dlg) delete dlg;

	C3doSelector *fdlg = new C3doSelector(this);
	int result = fdlg->DoModal();

	if (result == IDOK)
	{
		if (fdlg->selected_file)
		    fdlg->selected_file->Explore();
	}

	if (fdlg) delete fdlg;

	
	return TRUE;
}

void CDatDocument::OnViewMiplist() 
{
	if (datFile)
	{
	 CMipList *dlg = new CMipList(datFile);

	 
	 dlg->DoModal();
	 if (dlg) delete dlg;
	}
}

BOOL CDatDocument::OnSaveDocument(LPCTSTR lpszPathName) 
{
	if (datFile)
	{
	  char ddrive[256];
	  char ddir[256];
	  char dfile[256];
	  char dext[256];
	  char tempFile[1024];

	  _splitpath(lpszPathName,ddrive,ddir,dfile,dext);
	  _makepath(tempFile,ddrive,ddir,dfile,"tmp");

	  if (datFile->isPackFile())
	  {
		datFile->Write(lpszPathName);
	  }
	  else
	  {
	    datFile->Write(tempFile);
	  }
	  return TRUE;
	}
	return FALSE;
}



void CDatDocument::OnEditUnpackall() 
{
	datFile->UnPackAll();
}

void CDatDocument::OnFileExporttrktoexcel() 
{
	CTrk *trk = datFile->GetMainTrkFile();
	char fileName[256];

	strcpy(fileName,"track.csv");
	CFileDialog *fdlg = new CFileDialog(FALSE,"csv",(LPCSTR)&fileName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,lpszTabFilter);

	int result = fdlg->DoModal();

	if (result!=IDCANCEL)
	{
	CString filename = fdlg->GetFileName();
	FILE *fp = FOPEN(filename,"w");

	if (fp == NULL){
		AfxMessageBox("Cannot open track.csv");
	    return;
    }

	trk->ExportCSV(fp);

	FCLOSE(fp);
	}
	if (fdlg) delete fdlg;
}

void CDatDocument::OnBuild() 
{
	if (!datFile->isPackFile() || datFile->outputfile==NULL)
	{
		AfxMessageBox("Can only build a new track from a .pck project file");
	}
	else
	{
		CString msg;
	    msg.Format("%s%s%s",datFile->DatDrive,datFile->DatDir,datFile->outputfile);
	    OnSaveDocument(msg);
	}
}

void CDatDocument::OnFileSaveall() 
{
	datFile->SaveModified();
}

void CDatDocument::OnBuildSettings() 
{
	CProjectSetting *dlg = new CProjectSetting(this);
	dlg->DoModal();
	
	if (dlg) delete dlg;
}



void CDatDocument::OnFileExportcamtoexcel() 
{
	CCam *cam = datFile->GetMainCameraFile();

	char fileName[256];

	strcpy(fileName,"cameras.csv");
	CFileDialog *fdlg = new CFileDialog(FALSE,"csv",(LPCSTR)&fileName,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,lpszTabFilter);

	int result = fdlg->DoModal();

	if (result!=IDCANCEL)
	{
	CString filename = fdlg->GetFileName();
	FILE *fp = FOPEN(filename,"w");

	if (fp == NULL){
		AfxMessageBox("Cannot open cameras.csv");
	    return;
    }

	cam->ExportCSV(fp);

	FCLOSE(fp);
	}
	if (fdlg) delete fdlg;
}


void CDatDocument::OnEditUpdatetrack() 
{
	CTrk *trk = datFile->GetMainTrkFile();
	if (trk)
	{
		trk->updateTrack();
	}
	else
	{
		AfxMessageBox("No Track Present");
	}
}
