// DatView.cpp : implementation file
//

#include "stdafx.h"
#include "CGplFile.h"
#include "C3do.h"
#include "3DOEditor.h"
#include "DatDocument.h"
#include "DatView.h"

#include "Point3D.h"
#include "Display.h"
//#include "Polygon.h"
#include "MainFrm.h"
#include "math.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define BEGIN_INDEX 4

/////////////////////////////////////////////////////////////////////////////
// DatView

IMPLEMENT_DYNCREATE(DatView, CScrollView)

#define PROFILE(x,y,b) x = AfxGetApp()->GetProfileInt(CString(#y), CString(#x),b);
#define WR_PROFILE(x,y) AfxGetApp()->WriteProfileInt(CString(#y), #x,x);

extern double DegToRad(double angle);

CPoint GetTrackPoint(double currrad,double angle,double length)
{
		   CPoint pt;
		   double anglelen     = length*angle;
           double anglerad     = DegToRad(anglelen);
           double sectorRadius = 0;
           double inrad        = DegToRad(anglelen/2.0);
           sectorRadius        = (length/anglerad);
           double chordrad     = DegToRad((anglelen/2.0));
           double len          = 2*(sectorRadius*sin(inrad));

           pt.x =  (long)(len*cos((currrad+inrad)));
           pt.y =  (long)(len*sin((currrad+inrad))); 
		   return pt;
}


DatView::DatView()
{
	display = new Display();
	display->setScale(0.2);
	selected = NULL;

	/*
	showPoints = TRUE;
    showLines = FALSE;
	showFilled = FALSE;
	showCameras = FALSE;
	showObjects = FALSE;
	showAxis = TRUE;

	showTrackNumbers = FALSE;
	showWalls = TRUE;
	showTrk = TRUE;
	show3do = TRUE;
	*/

	PROFILE(showPoints,Preferences,FALSE);
	PROFILE(showLines,Preferences,FALSE);
	PROFILE(showFilled,Preferences,FALSE);
	PROFILE(showCameras,Preferences,TRUE);
	PROFILE(showObjects,Preferences,FALSE);
	PROFILE(showAxis,Preferences,TRUE);
	PROFILE(showTrackNumbers,Preferences,FALSE);
	PROFILE(showTrk,Preferences,TRUE);
	PROFILE(show3do,Preferences,FALSE);
	PROFILE(showWalls,Preferences,TRUE);

}

DatView::~DatView()
{
	WR_PROFILE(showPoints,Preferences)
	WR_PROFILE(showLines,Preferences)
	WR_PROFILE(showFilled,Preferences)
	WR_PROFILE(showCameras,Preferences)
	WR_PROFILE(showObjects,Preferences)
	WR_PROFILE(showAxis,Preferences)
	WR_PROFILE(showTrackNumbers,Preferences)
	WR_PROFILE(showTrk,Preferences)
	WR_PROFILE(show3do,Preferences)
	WR_PROFILE(showWalls,Preferences)
	delete display;
}


BEGIN_MESSAGE_MAP(DatView, CScrollView)
	//{{AFX_MSG_MAP(DatView)
	ON_COMMAND(ID_ZOOMIN, OnZoomin)
	ON_COMMAND(ID_ZOOMOUT, OnZoomout)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_SHOW_POINTS, OnShowPoints)
	ON_COMMAND(ID_SHOWLINES, OnShowlines)
	ON_UPDATE_COMMAND_UI(ID_SHOWLINES, OnUpdateShowlines)
	ON_UPDATE_COMMAND_UI(ID_SHOW_POINTS, OnUpdateShowPoints)
	ON_COMMAND(ID_SHOWFILLED, OnShowfilled)
	ON_UPDATE_COMMAND_UI(ID_SHOWFILLED, OnUpdateShowfilled)
	ON_COMMAND(ID_ZOOMHOME, OnZoomhome)
	ON_WM_MBUTTONUP()
	ON_COMMAND(ID_SHOWOBJECTS, OnShowobjects)
	ON_UPDATE_COMMAND_UI(ID_SHOWOBJECTS, OnUpdateShowobjects)
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(ID_SHOWWALLS, OnShowwalls)
	ON_UPDATE_COMMAND_UI(ID_SHOWWALLS, OnUpdateShowwalls)
	ON_COMMAND(ID_SHOWTRACKNUMBERS, OnShowtracknumbers)
	ON_UPDATE_COMMAND_UI(ID_SHOWTRACKNUMBERS, OnUpdateShowtracknumbers)
	ON_COMMAND(ID_SHOWTRACK, OnShowtrack)
	ON_UPDATE_COMMAND_UI(ID_SHOWTRACK, OnUpdateShowtrack)
	ON_COMMAND(ID_SHOWTHREEDO, OnShowthreedo)
	ON_UPDATE_COMMAND_UI(ID_SHOWTHREEDO, OnUpdateShowthreedo)
	ON_COMMAND(ID_VIEW_LPCHARTS, OnViewLpcharts)
	ON_COMMAND(ID_SHOW_AXIS, OnShowAxis)
	ON_UPDATE_COMMAND_UI(ID_SHOW_AXIS, OnUpdateShowAxis)
	ON_COMMAND(ID_SHOWCAMERAS, OnShowcameras)
	ON_UPDATE_COMMAND_UI(ID_SHOWCAMERAS, OnUpdateShowcameras)
	ON_COMMAND(ID_EDIT_CREATE3DOFROMTRK, OnEditCreate3dofromtrk)
	ON_COMMAND(ID_TABLES_TRACKVIEW, OnTablesTrackview)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// DatView drawing

void DatView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	CSize sizeTotal;
	// TODO: calculate the total size of this view
	sizeTotal.cx = sizeTotal.cy = 1000;
	SetScrollSizes(MM_TEXT, sizeTotal);

	
	float minx,miny,maxx,maxy;

	CDatDocument *pDoc = (CDatDocument*)GetDocument();
	{
	 if (pDoc->trackFile!=NULL)
	 {
		C3do *TrackFile = (C3do*)pDoc->trackFile;
		if (TrackFile->ptsArray!=NULL)
		{
	     TrackFile->GetMinMax(&minx,&miny,&maxx,&maxy);
		 //display->setOffsetXY(minx,miny);
		 display->setOffsetXY(minx,maxy);
		 double dx = maxx-minx;
		 double dy = maxy-miny;
		 double mdd = max(dx,dy);
		 sizeTotal.cx = sizeTotal.cy = (long)(mdd*display->getScale());
		 SetScrollSizes(MM_TEXT, sizeTotal);
		}
	 }
	}

	
}

void DatView::OnDraw(CDC* pDC)
{
	CDatDocument *pDoc = (CDatDocument*)GetDocument();
	display->setContext(pDC);
	
	if (show3do && pDoc->trackFile!=NULL)
	{
		C3do *TrackFile = (C3do*)pDoc->trackFile;
		Vector *topDrawlocation = TrackFile->topDrawlocation;


		if (topDrawlocation!=NULL)
		{
		  for(int i=0;i<topDrawlocation->size();i++)
		  {
			  Node * topDraw = (Node*) topDrawlocation->elementAt(i);
			  topDraw->Draw(TrackFile,display,this);
		  }
		}
		else
		{
		 /*
		 int psize = TrackFile->getNumPolygons();
		 for(int i=0;i<psize;i++)
		 {
			DOPolygon *poly = (DOPolygon*)TrackFile->polygonArray[i];
			
			int ppsize = poly->points->size();

			if (ppsize > 1)
			{
			Point3D *pt = (Point3D*)poly->points->elementAt(0);
			if (pt)
			{
			 double x=pt->getX()-display->ofx;
		     double y=pt->getY()-display->ofy;

		     x = (x*display->getScale());
		     y = -1*(y*display->getScale());
			 poly->npoly[0].x = (int)x;
			 poly->npoly[0].y = (int)y;
			}
			}

			for(int j=1;j<ppsize;j++)
			{ 
               Point3D *pt = (Point3D*)poly->points->elementAt(j);
			   if (pt)
			   {
			   double x=pt->getX()-display->ofx;
		       double y=pt->getY()-display->ofy;

		       x = (x*display->getScale());
		       y = -1*(y*display->getScale());
	      	   //if (showLines ) display->pDC->LineTo((int)x,(int)y);
			   poly->npoly[j].x = (int)x;
			   poly->npoly[j].y = (int)y;
			   }
			}
			if (showFilled || showLines )
			{
			CBrush *oldbrush;
			if (showLines ) display->pDC->SelectStockObject(BLACK_PEN);
			else  display->pDC->SelectStockObject(NULL_PEN);
			if (showFilled) 
			{
			 CBrush *polygonBrush = new CBrush(poly->colorref);
			 oldbrush = display->pDC->SelectObject(polygonBrush);
			 display->pDC->Polygon(poly->npoly,ppsize);
			 display->pDC->SelectObject(oldbrush);
			 delete polygonBrush;
			}
			}
			//DOPolygon *poly = (DOPolygon*)TrackFile->polygonArray->GetNext(pos);
		 }
		
		display->pDC->SelectStockObject(BLACK_PEN);

		if (showLines)
		{
		 for(int i=0;i<psize;i++)
		 {
			DOPolygon *poly = (DOPolygon*)TrackFile->polygonArray[i];		  
			
			if (showLines )
			{
			display->pDC->SelectStockObject(BLACK_PEN);
			display->pDC->SelectStockObject(NULL_BRUSH);
			display->pDC->Polygon(poly->npoly,poly->points->size());
			}
			//DOPolygon *poly = (DOPolygon*)TrackFile->polygonArray->GetNext(pos);
		 }
		}
		*/
		}



		display->pDC->SelectStockObject(BLACK_PEN);

		if (showPoints && TrackFile->ptsArray)
		{
		 int size = TrackFile->numPts;
		 for(int i=0;i<size;i++)
		 {	
			TrackFile->ptsArray[i]->Draw2D(display);

			if (TrackFile->ptsArray[i]->SrbCount > 0)
			{
			 double x=TrackFile->ptsArray[i]->getX();
		     double y=TrackFile->ptsArray[i]->getY();

		     display->drawBitmap(IDB_SRB,x,y);
			}
		 }
		}

		if (showObjects)
		{
		 int objSize = TrackFile->objArray->GetCount(); 
		 for(int i=0;i<objSize;i++)
		 {
			//TrackFile->objArray[i]->Draw2D(display);
			POSITION pos = TrackFile->objArray->FindIndex(i);
		    Point3D *pt = (Point3D*)TrackFile->objArray->GetAt(pos);

			double x=pt->getX();
		    double y=pt->getY();

		    //x = (x*display->getScale());
		    //y = -1*(y*display->getScale());
			 
			display->drawBitmap(IDB_OBJECT,x,y); 
		 }
		}

		
		
		if (selected)
			 selected->Draw2DSelected(display);	
	}

	if (showTrk && pDoc->trkFile!=NULL)
	{
	   double x;
	   double y;

	   x = 1024;
	   y = -1024;
	   if (pDoc->trackFile!=NULL)
	   {
	    C3do *TrackFile = (C3do*)pDoc->trackFile;
	    if (TrackFile->ptsArray && TrackFile->numPts > 0)
		{
		 x=TrackFile->ptsArray[BEGIN_INDEX]->getX();
		 y=TrackFile->ptsArray[BEGIN_INDEX]->getY();
		 //if (!TrackFile->isN2Version())
		 //{
	     // x=TrackFile->ptsArray[8]->getX();
	     // y=TrackFile->ptsArray[8]->getY();
		 //}
		 //else
		 //{
		 // x=TrackFile->ptsArray[5]->getX();
	     // y=TrackFile->ptsArray[5]->getY();
		 //}
		}
	   }
	   pDoc->trkFile->Draw2D(display,this,x,y);
	}

	if (showCameras)
	{
		 CCam *CameraFile = (CCam*)pDoc->cameraFile;
		 if (CameraFile && pDoc->trkFile)
		 {
		   CameraFile->Draw(display,pDoc->trkFile);
		 }
	}
}

/////////////////////////////////////////////////////////////////////////////
// DatView diagnostics

#ifdef _DEBUG
void DatView::AssertValid() const
{
	CScrollView::AssertValid();
}

void DatView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// DatView message handlers

void DatView::OnZoomin() 
{
	double scale = display->getScale();
	scale*=1.2;
	display->setScale(scale);
	UpdateScroll();
	InvalidateRect(NULL,TRUE);
}

void DatView::OnZoomout() 
{
	double scale = display->getScale();
	scale*=0.8;
	display->setScale(scale);
	UpdateScroll();
	InvalidateRect(NULL,TRUE);
}


void DatView::UpdateScroll()
{
    float minx,miny,maxx,maxy;
	CSize sizeTotal;


	CDatDocument *pDoc = (CDatDocument*)GetDocument();
	{
	 if (pDoc->trackFile!=NULL)
	 {
		C3do *TrackFile = (C3do*)pDoc->trackFile;
	    TrackFile->GetMinMax(&minx,&miny,&maxx,&maxy);
		display->setOffsetXY(minx,maxy);
		double dx = maxx-minx;
		double dy = (maxy-miny);
		double mdd = max(dx,dy);
		sizeTotal.cx = sizeTotal.cy = 2*(long)(mdd*display->getScale());
		SetScrollSizes(MM_TEXT, sizeTotal);
	 }
	}
}

void DatView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (nFlags & MK_CONTROL)
	{
	  OnZoomin();
	  OnZoomin();
	  OnZoomin();
	  OnZoomin();
	}
	else
	{
	 CPoint pt;
	 CPoint offset = GetDeviceScrollPosition();

	 pt.x = point.x+offset.x;
	 pt.y = point.y+offset.y;
	 dragging = TRUE;
	 FindSelection(pt);
	 InvalidateRect(NULL,TRUE);
	}
	CScrollView::OnLButtonDown(nFlags, point);
}

void DatView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	dragging = FALSE;
	selected = NULL;
	//selectedIndex=-1;
	CScrollView::OnLButtonUp(nFlags, point);
}

void DatView::OnMouseMove(UINT nFlags, CPoint point) 
{
    CPoint pt;
	CPoint offset = GetDeviceScrollPosition();

	pt.x = point.x+offset.x;
	pt.y = point.y+offset.y;

	if (selected && dragging)
	{
	  selected->setLocation(display,pt);
	  InvalidateRect(NULL,TRUE);
	}

	double x = pt.x/display->getScale();
	double y = pt.y/display->getScale();

	x = x+ display->ofx;
	y = y+ display->ofy;

	CString msg;

	//if (selected)
	//{
	msg.Format("%2.2f,%2.2f = Pt[%d]",x,y,selectedIndex);
	//}
	//else
	//{
	//  msg.Format("%2.2f,%2.2f",x,y);
	//}

	((CMy3DOEditorApp*)AfxGetApp())->pMainFrame->SetStatusText(msg);
	CScrollView::OnMouseMove(nFlags, point);
}

void DatView::FindSelection(CPoint point)
{
    CDatDocument *pDoc = (CDatDocument*)GetDocument();

	if (pDoc->trackFile!=NULL)
	{
		C3do *TrackFile = (C3do*)pDoc->trackFile;
		//int size = TrackFile->ptsArray->GetCount();
		int size = 	TrackFile->numPts;
	    for(int i=0;i<size;i++)
		{
			//POSITION pos = TrackFile->ptsArray->FindIndex(i);
		    //Point3D *pt = (Point3D*)TrackFile->polygonArray->GetAt(pos);
			Point3D *pt = TrackFile->ptsArray[i];

			if (pt->isOver(display,point))
			{
				selected = pt;
				selectedIndex = i;
			}
		}		
	}
	
}


void DatView::OnShowPoints() 
{
	showPoints = !showPoints;
	InvalidateRect(NULL,TRUE);
}

void DatView::OnShowlines() 
{
	showLines = !showLines;
	InvalidateRect(NULL,TRUE);
}

void DatView::OnUpdateShowlines(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(showLines);
}

void DatView::OnUpdateShowPoints(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(showPoints);
}

void DatView::OnShowfilled() 
{
	showFilled = !showFilled;
	InvalidateRect(NULL,TRUE);
}

void DatView::OnUpdateShowfilled(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(showFilled);
}

void DatView::OnZoomhome() 
{
	double scale =0.2;
	display->setScale(scale);
	UpdateScroll();
	InvalidateRect(NULL,TRUE);
}

void DatView::OnMButtonUp(UINT nFlags, CPoint point) 
{
	if (nFlags & MK_CONTROL)
	{
	  OnZoomout();
	  OnZoomout();
	  OnZoomout();
	  OnZoomout();
	}
	else
	{
	  OnZoomout();
	}



	CScrollView::OnRButtonUp(nFlags, point);
}

void DatView::OnShowobjects() 
{
	showObjects = !showObjects;
	InvalidateRect(NULL,TRUE);
}

void DatView::OnUpdateShowobjects(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(showObjects);
}

void DatView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	if (selected)
	{
	  selected->Explore(NULL);
	}
	else
	{
	 CPoint pt;
	 CPoint offset = GetDeviceScrollPosition();

	 pt.x = point.x+offset.x;
	 pt.y = point.y+offset.y;
	 FindSelection(pt);
	 if (selected ) selected->Explore(NULL);
	}

	CScrollView::OnLButtonDblClk(nFlags, point);
}

void DatView::OnShowwalls() 
{
	showWalls = !showWalls;
	InvalidateRect(NULL,TRUE);
}

void DatView::OnUpdateShowwalls(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(showWalls);
}

void DatView::OnShowtracknumbers() 
{
	showTrackNumbers = !showTrackNumbers;
	InvalidateRect(NULL,TRUE);
}

void DatView::OnUpdateShowtracknumbers(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(showTrackNumbers);
}

void DatView::OnShowtrack() 
{
	showTrk = !showTrk;
	InvalidateRect(NULL,TRUE);
}

void DatView::OnUpdateShowtrack(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(showTrk);
}

void DatView::OnShowthreedo() 
{
	show3do = !show3do;
	InvalidateRect(NULL,TRUE);
}

void DatView::OnUpdateShowthreedo(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(show3do);
}

void DatView::OnViewLpcharts() 
{
	CDatDocument *pDoc = (CDatDocument*)GetDocument();

    if (pDoc && pDoc->datFile)
	{
	   if(pDoc->datFile->LpDocument)
	   {
		  ((CMy3DOEditorApp*)AfxGetApp())->OnLpView(pDoc->datFile->LpDocument);
	   }
	}
}

void DatView::OnShowAxis() 
{
	showAxis = !showAxis;
	InvalidateRect(NULL,TRUE);
}

void DatView::OnUpdateShowAxis(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(showAxis);
}

void DatView::OnShowcameras() 
{
	showCameras = !showCameras;
	InvalidateRect(NULL,TRUE);
}

void DatView::OnUpdateShowcameras(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(showCameras);
}

void DatView::OnEditCreate3dofromtrk() 
{
	CDatDocument *pDoc = (CDatDocument*)GetDocument();

    if (pDoc && pDoc->trkFile)
	{
	   pDoc->trkFile->Create3do("toval.3do");
	}
}

void DatView::OnTablesTrackview() 
{
	((CMy3DOEditorApp*)AfxGetApp())->OnTablesTrackview(GetDocument());
}
