
#ifndef _INCLUDED_DISPLAY_H
#define _INCLUDED_DISPLAY_H

#include "stdafx.h"

class Display
{
public:
	CBrush *redBrush;
	CBrush *blueBrush;
	CBrush *greyBrush;
	CPen   *greyPen;

	Display()
	{
		ofx = 0.0f;
		ofy = 0.0f;
		redBrush  = new CBrush(RGB(0xFF,0,0));
		blueBrush = new CBrush(RGB(0,0,0xFF));
		greyBrush = new CBrush(RGB(0xC0,0xC0,0xC0));
		greyPen = new CPen(PS_SOLID,1,RGB(0xC0,0xC0,0xC0));
	}

	~Display()
	{
	    delete redBrush;
		delete blueBrush;
		delete greyBrush;
		delete greyPen;
	}

	void SelectRedBrush()
	{
		pDC->SelectObject(redBrush);
	}

	void SelectBlueBrush()
	{
		pDC->SelectObject(blueBrush);
	}

	void SelectGreyBrush()
	{
		pDC->SelectObject(greyBrush);
	}

	void SelectGreyPen()
	{
		pDC->SelectObject(greyPen);
	}

	void DefaultBrush()
	{
		pDC->SelectStockObject(BLACK_BRUSH);
	}

	void DefaultPen()
	{
		pDC->SelectStockObject(BLACK_PEN);
	}

	void setScale(double sc)
	{
		scale = sc;
	}

	double getScale()
	{
		return scale;
	}

	void setContext(CDC *pdc)
	{
		pDC = pdc;
	}

	void setOffsetXY(float x,float y)
	{
	   ofx = x;
	   ofy = y;
	}

	void drawBitmap(int id,double x1,double y1)
    {
	double xa=x1-ofx;
	double ya=y1-ofy;
	
    xa = (xa*getScale());
	ya = -1*(ya*getScale());
	CBitmap bmp, *poldbmp;
    CDC memdc;

	// Load the bitmap resource
    bmp.LoadBitmap( id );
 
    // Create a compatible memory DC
    memdc.CreateCompatibleDC( pDC );
 
    // Select the bitmap into the DC
    poldbmp = memdc.SelectObject( &bmp );
 
	
	// Copy (BitBlt) bitmap from memory DC to screen DC
    pDC->BitBlt( (int)xa, (int)ya, 16, 15, &memdc, 0, 0, SRCAND );
	
    memdc.SelectObject( poldbmp );
	memdc.DeleteDC();
    }

	void drawLine(double x1,double y1,double x2,double y2)
	{
		double xa=x1-ofx;
	    double ya=y1-ofy;
		double xb=x2-ofx;
	    double yb=y2-ofy;

	    xa = (xa*getScale());
	    ya = -1*(ya*getScale());

		xb = (xb*getScale());
	    yb = -1*(yb*getScale());

		pDC->MoveTo((int)xa,(int)ya);
		pDC->LineTo((int)xb,(int)yb);
	}

	void drawRectangle(double x1,double y1,double x2,double y2)
	{
		double xa=x1-ofx;
	    double ya=y1-ofy;
		double xb=x2-ofx;
	    double yb=y2-ofy;

	    xa = (xa*getScale());
	    ya = -1*(ya*getScale());

		xb = (xb*getScale());
	    yb = -1*(yb*getScale());

		pDC->Rectangle((int)xa,(int)ya,(int)xb,(int)yb);
	}

	double getScreenX(double x1)
	{
		double xa=x1-ofx;
		xa = (xa*getScale());
		return xa;
	}

	double getScreenY(double y1)
	{
		double ya=y1-ofy;
		ya = -1*(ya*getScale());
		return ya;
	}

	void drawPolygon(double x1,double y1,double x2,double y2,double x3,double y3,double x4,double y4)
	{
	   POINT pts[4];

	   pts[0].x = (long)getScreenX(x1);
	   pts[0].y = (long)getScreenY(y1);

	   pts[1].x = (long)getScreenX(x2);
	   pts[1].y = (long)getScreenY(y2);

	   pts[2].x = (long)getScreenX(x3);
	   pts[2].y = (long)getScreenY(y3);

	   pts[3].x = (long)getScreenX(x4);
	   pts[3].y = (long)getScreenY(y4);

	   pDC->Polygon(pts,4);
	}

	void drawText(double x1,double y1,LPCSTR text)
	{
		   int sx = (int)getScreenX(x1);
	       int sy = (int)getScreenY(y1);

		   pDC->SelectStockObject(ANSI_VAR_FONT);
		   pDC->SetBkMode(TRANSPARENT);
		   pDC->TextOut(sx,sy,text);
	}

	void drawArc(double x,double y,double x1,double y1,double cx,double cy,double angle1,double angle2,double radius)
    {
	double sx = getScreenX(x1);
	double sy = getScreenY(y1);
	double ex = getScreenX(x);
	double ey = getScreenY(y);
	double csx = getScreenX(cx);
	double csy = getScreenY(cy);
	double rad = (double)(radius*scale);

	int irad = (int) rad;

	if (irad == 0) return;

	if (sx == ex && sy == ey) return;
	if ((angle1+angle2) == angle1) return;

	int scx,scy,ecx,ecy;

	if (rad > 0)
	{
	scx =  (int)(csx-rad);
	scy =  (int)(csy-rad);
	ecx =  (int)(csx+rad);
	ecy =  (int)(csy+rad);
	}
	else
	{
	scx =  (int)(csx+rad);
	scy =  (int)(csy+rad);
	ecx =  (int)(csx-rad);
	ecy =  (int)(csy-rad);
	}

	if (scx == ecx) 
		return;

	

	if ((angle1+angle2) > angle1)
	{
	 pDC->Arc((int)scx,(int)scy,(int)ecx,(int)ecy,(int)ex,(int)ey,(int)sx,(int)sy);
	}
	else
	 pDC->Arc((int)scx,(int)scy,(int)ecx,(int)ecy,(int)sx,(int)sy,(int)ex,(int)ey);
	
    }


	CDC *pDC;
	double scale;
	float ofx;
	float ofy;

};

#endif