
#include "stdafx.h"
#include "ImageFile.h"
#include "ObjectTree.h"
#include "resource.h"
#include "Vector.h"
#include "BMP.h"
#include "MipPreview.h"

#include "CMip.h"
#include "CSrb.h"

#include <gl\glaux.h>
#include <gl\gl.h>
#include <gl\glu.h>

#include "ImageFileEditor.h"


extern int ReadPCDWord(FILE *fp);

CImageFile::CImageFile(CDat *datfile,LPCSTR buffer):
Observer(t_Image)
{
	filename = strdup(buffer);
	sections = NULL;
	bits=NULL;
	valid = FALSE;
	bmp = NULL;
	datFileParent = datfile;
}

CImageFile::~CImageFile()
{
	if (filename) free(filename);
	if (bits)     free(bits);
}

typedef enum
{
	t_Mip,t_Srb,t_3do,t_Unk
} t_FileType;

t_FileType getFileType(const char *filename,CString &newfilename)
{
	FILE *fp;

	newfilename.Format("%s.MIP",filename);
	fp = FOPEN(newfilename,"rb");
	if (fp != NULL) 
	{
		FCLOSE(fp);
		return t_Mip;
	}

	

	newfilename.Format("%s.SRB",filename);
	fp = FOPEN(newfilename,"rb");
	if (fp != NULL) 
	{
		FCLOSE(fp);
		return t_Srb;
	}

	newfilename.Format("%s.3DO",filename);
	fp = FOPEN(newfilename,"rb");
	if (fp != NULL) 
	{
		FCLOSE(fp);
		return t_3do;
	}

	return t_Unk;
}

BOOL CImageFile::Open()
{

	CString newfilename;
	t_FileType fileType = getFileType(filename,newfilename);


	switch(fileType)
	{
	   case t_3do:
		   AfxMessageBox("Don't have a 3do handler");
		   break;
	   case t_Mip:
		   {
		   CMip *mipfile = new CMip();
		   mipfile->Read(newfilename);
		   bmp = &(mipfile->bitmapData);
		   }
		   break;
	   case t_Srb:
		   {
		   CSrb *srbfile = new CSrb();
		   //srbfile->Read(newfilename);
		   //bmp = srbfile->cmip->bitmapData;
		   }
		   break;
	}
	
	return TRUE;
}

void CImageFile::LoadTree(HTREEITEM node,CObjectTree *tree)
{
	if (Open())
	{
		//CString buffer;
		//buffer.Format("FileSize %d",Header.fileSize);
		//tree->insertInfoNode(node, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_OFFSET));
		//buffer.Format("Width %d",logx);
		//tree->insertInfoNode(node, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_OFFSET));
		//buffer.Format("Height %d",logy);
		//tree->insertInfoNode(node, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_OFFSET));
		//buffer.Format("Images %d",numimages);
		//tree->insertInfoNode(node, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_OFFSET));
		//buffer.Format("Size=%d NumImages=%d DefaultColor=%X",Header.fileSize,Header.NumImages,Header.DefaultColor);
		//tree->insertInfoNode(node, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_OFFSET));
		/*
		for(int i=0;i<sections->size();i++)
		{
		   Block *blk = (Block*)sections->elementAt(i);
		   buffer.Format("Sec[%d]: %s %d ",i,blk->getId(),blk->getSize());
		   tree->insertInfoNode(node, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_OFFSET));
		}
		*/
	}
	else
	{
	  CString buffer;
	  buffer.Format("Couldn't Find File:%s",filename);
	  tree->insertInfoNode(node, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_FILE));
	}

}

unsigned char *TextureMap(CDC *pDC ,CBitmap &cbmp,int width,int height,unsigned char *lpvBits)
{ 
  if (lpvBits == NULL)
  {
  BITMAP bmp;
  
  cbmp.GetBitmap( &bmp );

  if (bmp.bmWidth > 1024) return NULL;
  if (bmp.bmHeight > 1024) return NULL;

  CDC*          bitmapDc = new CDC;
  bitmapDc -> CreateCompatibleDC(pDC);

  CBitmap* oldBitMap = (CBitmap*) bitmapDc -> SelectObject( &cbmp );

  //unsigned char *lpvBits = new unsigned char[bmp.bmWidth * bmp.bmHeight *3];
  lpvBits = new unsigned char[bmp.bmWidth * bmp.bmHeight *3];
  BITMAPINFO *bmpinfo = new BITMAPINFO;

  bmpinfo->bmiHeader.biSize          = sizeof(BITMAPINFOHEADER);
  bmpinfo->bmiHeader.biWidth         = bmp.bmWidth;
  bmpinfo->bmiHeader.biHeight        = bmp.bmHeight;
  bmpinfo->bmiHeader.biPlanes        = 1;
  bmpinfo->bmiHeader.biBitCount      = 24;
  bmpinfo->bmiHeader.biCompression   = BI_RGB;
  bmpinfo->bmiHeader.biSizeImage     = bmp.bmWidth * bmp.bmHeight * 3;
  bmpinfo->bmiHeader.biXPelsPerMeter = 0;
  bmpinfo->bmiHeader.biYPelsPerMeter = 0;
  bmpinfo->bmiHeader.biClrImportant  = 0;
  bmpinfo->bmiHeader.biClrUsed       = 0;

  if (!::GetDIBits( bitmapDc -> GetSafeHdc(), HBITMAP( cbmp ), 0,bmp.bmHeight, lpvBits, bmpinfo, DIB_RGB_COLORS ))
  {
   AfxMessageBox("GetDIBits error");
  }

  unsigned int size = (bmpinfo->bmiHeader.biSizeImage);

  for(unsigned int i=0;i<size;i+=3)
  {
	  unsigned char *ptr =lpvBits+i;
	  unsigned char *r = ptr;
	  unsigned char *g = ptr+1;
	  unsigned char *b = ptr+2;

	  unsigned char red = *r;
	  unsigned char blue = *b;
	  *b = red;
	  *r = blue;
  }

  bitmapDc -> SelectObject( oldBitMap );
  delete bitmapDc;
  delete bmpinfo;
  }

  GLint val;
  glGetIntegerv(GL_MAX_TEXTURE_SIZE,&val);

  glTexImage2D(GL_TEXTURE_2D, 0, 3, width, height, 0,GL_RGB, GL_UNSIGNED_BYTE, lpvBits);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);  
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);  
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);  
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);  
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);  
  glEnable(GL_TEXTURE_2D);  
  glShadeModel(GL_SMOOTH);  

  
  //delete[] lpvBits;
  
  return lpvBits;
}

void Mak256SquareImage(CBitmap &jamBitmap,CBitmap &currentImg,int nWidth,int nHeight,int hr,int vr,int ow,int oh)
{
	   
       CDC memdcA,memdcB;
	   CBitmap *poldbmps,*poldbmpd;
	   // Create a compatible memory DC
       memdcA.CreateCompatibleDC( NULL );
	   memdcB.CreateCompatibleDC( NULL );
	   
 
	   // Select the bitmap into the DC
       poldbmps = memdcA.SelectObject(&jamBitmap);	   
	   poldbmpd = memdcB.SelectObject(&currentImg);

	   int subImageSizeH = (ow/hr);
	   int subImageSizeV = (oh/vr);

	   /*
	   for(int i=0;i<subImageSizeH;i++)
	   {
	       for(int j=0;j<subImageSizeV;j++)
		   {
			 int xpos = (i*subImageSizeH);
			 int ypos = (j*subImageSizeV);
	         memdcB.StretchBlt( xpos,ypos, subImageSizeH,subImageSizeV, &memdcA, 0,0, nWidth, nHeight,SRCCOPY );
		   }
	   }
	   
	   */
	   memdcB.StretchBlt( 0,0, ow,oh, &memdcA, 0,0, nWidth,nHeight,SRCCOPY );
	   

	   memdcA.SelectObject( poldbmps );
	   memdcB.SelectObject( poldbmpd );
}

BOOL CImageFile::Edit()
{
	return Rename();
}

void CImageFile::Explore()
{
	CMip *textureMip=NULL;

	if (datFileParent)
	{
		textureMip = (CMip*)datFileParent->FindImageFile(filename);
		if (textureMip)
		{
			textureMip->GetFile();
		}
		if (textureMip==NULL)
		{
			CSrb *SrbFile = (CSrb*)datFileParent->FindSrbImageFile(filename);
			if (SrbFile)
			{
			   SrbFile->GetFile();
			   textureMip = SrbFile->cmip;
			}

		}
	}

	if (textureMip)
	{
	   CMipPreview *dlg = new CMipPreview();
	   dlg->nWidth  = textureMip->bitmapData.ImageWidth();
	   dlg->nHeight = textureMip->bitmapData.ImageHeight();
	   textureMip->bitmapData.CreateDIB(dlg->bmp);
	   dlg->DoModal();
	}
	else
	{
	   AfxMessageBox("No Image Loaded");
	}
}

BOOL CImageFile::Rename()
{
	CImageFileEditor *dlg = new CImageFileEditor();

	dlg->m_Edit = filename; 
	int result = dlg->DoModal();
	

	if (result == IDOK)
	{
		filename = strdup(dlg->m_Edit);
		return TRUE;
	}
	return FALSE;
}

