#ifndef _INCLUDED_NODE_H
#define _INCLUDED_NODE_H

#include "ObjectTree.h"
#include "resource.h"
#include "Observer.h"
#include "Display.h"
#include "C3do.h"
#include "DataValue.h"

class Node;
class CMip;
class CSrb;
//class C3do;
class NodePlane;

Node * getChildType(int offset,Node *parent,FILE *fp,C3do *object=NULL);
int ReadPCDWord(FILE *fp);
float ReadPCFloat(FILE *fp);
void  WritePCDWord(FILE *fp,int number);



inline CString ToRgb(int color)
{
	CString msg;
	int red= (color & 0x00FF0000) >> 16;
	int green = (color & 0x0000FF00) >> 8;
	int blue=(color & 0x000000FF);
	msg.Format("(%d,%d,%d)",red,green,blue);
	return msg;
}

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
class Node: public Observer
{
   
   public:
   Node(int secOffset=-1,Node *parent=NULL,int type=-1):sectionOffset(secOffset),parent(parent),numberOfChildren(0),bitmapid(IDB_OFFSET),type(type),Observer(t_tree)
   {
	   object = NULL;
	   children = NULL;
	   writingOffset = -1;
	   treeNode = NULL;
	   string_offset = 0;
	   refCount = 0;
	   written = FALSE;
	   drawn = FALSE;
	   root = NULL;
	   depth = -1;
   }

   void DeleteNode(Node *ptr);
   virtual ~Node();

   virtual void Explore()
   {
	   Edit();
   }
      
   

   int getCommandVal(FILE *fp);
   static int getCommandValObj(FILE *fp,C3do *obj);
   void sendCommandVal(FILE *fp,int val);

   virtual void Read(FILE *fp)
   {
	   int foff   = ftell(fp);
	   numberOfChildren = 1;
	   children = (Node**)malloc(sizeof(Node*));
	   children[0] = getChildType(sectionOffset,this,fp);
	   
   }

   virtual int Write(FILE *fp,int secOffset)=0;
   virtual int OverLay(FILE *fp)
   {
	   CString msg;
	   msg.Format("No Overlay Writer for T-%x info please report",type);
	   cerr << msg << endl;
	   return -1;
   }

   int WriteAllChildren(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   for(int i=0;i<numberOfChildren;i++)
	   { 
		   if (children[i] && children[i]->written==FALSE){
			   children[i]->written = TRUE;
			   children[i]->writingOffset = children[i]->Write(fp,sectionOffset);			   
		   }
	    }
	   return foff;
   }

   void ClearWriteStatus()
   {
	   for(int i=0;i<numberOfChildren;i++)
	   {		   
		   if (children[i] && children[i]->written==TRUE)
		   {
			   children[i]->written=FALSE;
			   children[i]->ClearWriteStatus();			   
		   }
	   }
   }

   virtual void ClearDrawnStatus();

   CString Export();

   void ExportData(ostream &out);

   void Print(ostream &out)
   {
	  out << toString();
	  int depth = GetDepth();
	  out << endl;
	  for(int j=0;j<depth;j++)
	  {
		out << "  ";
	  }
	  out << "{" << endl;
	  
	  for(int i=0;i<numberOfChildren;i++)
	  {
		  if (children[i])
		  {
			  int depth = children[i]->GetDepth();
			  for(int j=0;j<depth;j++)
			  {
				  out << "  ";
			  }
			  children[i]->Print(out);
			  out << endl;
		  }
	  }
	  for(j=0;j<depth;j++)
	  {
		out << "  ";
	  }
	  out << "}" << endl;
	  
   }


   int GetDepth()
   {
	   Node *par = this;
	   int count = 0;
	   while(par!=NULL)
	   {
		   par=par->getParent();
		   count++;		  
	   }
	   return count;
   }

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d Children=%d",type,numberOfChildren);
	   return buffer;
   }

   void Expand(CObjectTree *tree,HTREEITEM parent)
   {
		if (subTree){
			   tree->m_ObjectTree.DeleteItem(subTree);
			   for(int i=0;i<numberOfChildren;i++)
	           {
                 if (children[i])
		            children[i]->LoadTree(parent,tree);
	           }
			   subTree = NULL;
		}
   }

   virtual HTREEITEM LoadTree(HTREEITEM branch,CObjectTree *tree)
   {
	   treeNode = tree->insertInfoNode(branch, (LPSTR)(LPCSTR)toString(),TO_ID(bitmapid));
	   tree->m_ObjectTree.SetItemData(treeNode,(DWORD)this);
	   if (numberOfChildren)
	   {
		   subTree = tree->insertInfoNode(treeNode, "subTree",TO_ID(bitmapid));
	   }
	   return treeNode;
   }

   void addChild(Node *node);

   

   int getOffset()
   {
	   if (sectionOffset == -1)
	   {
		   AfxMessageBox("Section Offset not set");
	   }
	   return sectionOffset;
   }
  
   virtual void Draw(C3do* object,Display *display,CView *doview)
   {
	   for(int i=0;i<numberOfChildren;i++)
	   {
		   children[i]->Draw(object,display,doview);
	   }
   }

   virtual void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   
   BOOL DrawDisplayClass(CView *doview,int display_Class);
   

   Node *getRoot()
   {
	   if (root == NULL)
	   {
	     Node *t=this;
	     while(t->getParent())
	     { 
		  t = t->getParent();
	     }
		 root = t;
	   }
	   return root;
   }

   Node *getParent()
   {
	   return parent;
   }

   virtual int getNodePlaneNumber()
   {
		return -1;
   }

   virtual int getPlaneNumber()
   {
	   Node *t=this;
	   while(t)
	   { 
		  int _planeNumber =  t->getNodePlaneNumber();
		  if (_planeNumber != -1)
			  return _planeNumber;
		  else
		      t = t->getParent();
	   }
	   return -1;
   }

   virtual NodePlane *getPlaneNode()
   {
	   Node *t=this;
	   while(t)
	   { 
		  if (t->type >= 0x6 && t->type <= 0xB)
		  {
			  return (NodePlane*)t;
		  }
		  else
		      t = t->getParent();
	   }
	   return NULL;
   }

   void ExpandToRoot(CObjectTree *tree);

   C3do *getObject()
   {
	   if (object == NULL)
	   {
	    Node *root = getRoot();
		object = root->object;
	    return root->object;
	   }
	   return object;
   }

   virtual int getTextureStringOffset();
   virtual BOOL Edit()
   {
	   CString msg;
	   msg.Format("No Editor For this Node type %s",toString());
	   AfxMessageBox(msg);
	   return FALSE;
   }
   virtual BOOL Init();

   // replace existing node with new node
   static BOOL ReplaceNode(Node *in,Node *replace);

   int size() { return numberOfChildren; }
   virtual int allowedChildren() = 0;
   
   
   Node *root;
   Node **children;
   int  numberOfChildren;
   int  type;
   int  bitmapid;
   int sectionOffset;
   C3do *object;
   int  writingOffset;
   int  string_offset;
   int  depth;
   HTREEITEM treeNode;
   int  refCount;
   BOOL written;
   BOOL drawn;
   int  nodeOffset;

   void setParent(Node *parent)
   {
	   parent = parent;
   }

   private:
      Node *parent;
};

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
class EmptyNode: public Node
{
   public:
   EmptyNode(int secOffset,Node *parent):Node(secOffset,parent,0xFFFF)
   {
   }

   void Read(FILE *fp)
   {
	   
   }

   int Write(FILE *fp,int secOffset)
   {
	   writingOffset = -1;
	   //AfxMessageBox("Writer Procedure Not Implement Yet  For this Empty Node");
	   return -1;
   }

   int OverLay(FILE *fp)
   {
	   return -1;
   }

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Empty Node -1");
	   return buffer;
   }

   int allowedChildren() { return 0; }
};


//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

class UnknownNode: public Node
{
   public:
 
   UnknownNode(int cmd,int secOffset,Node *parent):Node(secOffset,parent,cmd)
   {
   }

   void Read(FILE *fp)
   {
	   
   }

   int Write(FILE *fp,int secOffset)
   {
	   AfxMessageBox("Writer Procedure Not Implement Yet Unknown Data");
	   return -1;
   }

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Unknown Node %x (%d)",type,type);
	   return buffer;
   }

   int allowedChildren() { return 0; }
};

class NodePtr: public Node
{
   public:
 
   NodePtr(int secOffset,Node *parent,int ptr):Node(secOffset,parent,0xFFFE),ptr(ptr)
   {
	   bitmapid = IDB_UNK;
	   pointToNode = NULL;
   }

   void Read(FILE *fp)
   { 
   }

   int Write(FILE *fp,int secOffset);
   int WriteSpecial(FILE *fp,int secOffset);
   
   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Node Ptr %x (%d)  %x",ptr,ptr,pointToNode);
	   if (pointToNode)
	   {		   
		   buffer+=pointToNode->toString();;
	   }
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int allowedChildren() { return 0; }
   int ptr;

   Node *pointToNode;
};


//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
typedef struct
{
	float x;
	float y;
	float z;
	float a;
	float b;
	float c;
	float scale;
	int   offset;
} t_FlavorThree;

class Node3doObjectPosition: public Node
{
   t_FlavorThree info;
   public:
   Node3doObjectPosition(int secOffset,Node *parent):Node(secOffset,parent,0x13)
   {
	   bitmapid = IDB_OBJECT;
	   memset(&info,0,sizeof(info));
   }

   ~Node3doObjectPosition()
   {

   }

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   int allowedChildren() { return 1; }

   void Read(FILE *fp);

   

   int OverLay(FILE *fp);

   int Write(FILE *fp,int secOffset)
   {
	    sectionOffset = secOffset;
		WriteAllChildren(fp,secOffset);
		int foff = ftell(fp);
	    if (children[0]->writingOffset!=-1){
	     info.offset=children[0]->writingOffset-getOffset();
	    }
	    else {
		 info.offset = -1;
	    }		
		sendCommandVal(fp,0x13);
		fwrite(&info,1,sizeof(t_FlavorThree),fp);
		return foff;
   }

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Position 3DO Object Type=%d Children=%d x=%2.2f y=%2.2f z=%2.2f a=%2.2f b=%2.2f c=%2.2f scale=%2.2f",type,numberOfChildren,info.x,info.y,info.z,info.a,info.b,info.c,info.scale);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   float getX();
   float getY();
   float getZ();
   float getRotX();
   float getRotY();
   float getRotZ();

   BOOL Edit();

   
};

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
class Node3doObject: public Node
{
   public:
   Node3doObject(int secOffset,Node *parent):Node(secOffset,parent,0xE)
   {
	   bitmapid = IDB_OBJECT;
	   number =0;
	   subObject =NULL;
	   found = TRUE;
   }

   void Read(FILE *fp);

   

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   sendCommandVal(fp,0xE);
	   WritePCDWord(fp,string_offset);
	   WritePCDWord(fp,number);
	   return foff;
   }

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);
   void Draw(C3do* object,Display *display,CView *doview);
 
   virtual CString toString();

   int OverLay(FILE *fp);	   
   

   BOOL Edit();

   int number;
   int allowedChildren() { return 0; }

   C3do * subObject;
   BOOL found;
};


//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
class NodeSrb: public Node
{
   public:
   NodeSrb(int secOffset,Node *parent):Node(secOffset,parent,0x3)
   {
	   bitmapid = IDB_SRB;
	   textureSrb = NULL;
	   number =0;
	   color =0;
	   child_offset =0;
	   scale = 1.0f;
   }

   ~NodeSrb();
   
   int  LoadSrb();

   void Read(FILE *fp);

   

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   sendCommandVal(fp,0x3);
	   WritePCDWord(fp,number);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,string_offset);
	   WritePCFloat(fp,scale);
	   WritePCDWord(fp,child_offset);
	   return foff;
   }

   BOOL Edit();

   virtual CString toString();

   virtual HTREEITEM LoadTree(HTREEITEM branch,CObjectTree *tree);

   

   void Draw(C3do* object,Display *display,CView *doview);
   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   int number;
   int color;
   float scale;
   int child_offset;
   int allowedChildren() { return 1; }

   CSrb   *textureSrb;
};


//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
class NodeSelection : public Node
{
	public:
		NodeSelection(int secionOffset,Node *parent,int offset,int mode):
		Node(secionOffset,parent,0x5),mode(mode)
		{
		}

		int mode;		
};

class NodeSelection1: public NodeSelection
{
   public:
   NodeSelection1(int secionOffset,Node *parent,int offset):NodeSelection(secionOffset,parent,0x5,0x1),child_offset(offset)
   {
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);

   

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   WriteAllChildren(fp,secOffset);
	   int foff = ftell(fp);
	   if (children[0]->writingOffset!=-1){
	     child_offset=children[0]->writingOffset-getOffset();
	   }
	   else {
		 child_offset = -1;
	   }

	   sendCommandVal(fp,0x5);
	   WritePCDWord(fp,child_offset);
	   WritePCDWord(fp,0x1);
	   WritePCDWord(fp,string_offset);
	   return foff;
   }

   virtual void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   virtual CString toString();

   

   //int string_offset;
   int child_offset;
};

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
// possibly only a root node
class NodeSelection4: public NodeSelection
{
   public:
   NodeSelection4(int secionOffset,Node *parent,int offset):NodeSelection(secionOffset,parent,0x5,0x4),child_offset(offset)
   {
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);

   

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   BOOL Edit();

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   WriteAllChildren(fp,secOffset);
	   int foff = ftell(fp);
	   if (children[0]->writingOffset!=-1){
	     child_offset=children[0]->writingOffset-getOffset();
	   }
	   else {
		 child_offset = -1;
	   }

	   sendCommandVal(fp,0x5);
	   WritePCDWord(fp,child_offset);
	   WritePCDWord(fp,0x4);
	   WritePCDWord(fp,displayClass);
	   return foff;
   }

   virtual CString toString();

   

   int child_offset;
   int displayClass;
};

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
class NodeSelection5: public NodeSelection
{
   public:
   NodeSelection5(int secionOffset,Node *parent,int offset):NodeSelection(secionOffset,parent,0x5,0x5),child_offset(offset)
   {
	   displayClass = 0;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);

   

   BOOL Edit();

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   WriteAllChildren(fp,secOffset);

	   if (children[0]->writingOffset!=-1){
	     child_offset=children[0]->writingOffset-getOffset();
	   }
	   else {
		 child_offset = -1;
	   }

	   int foff = ftell(fp);
	   sendCommandVal(fp,0x5);
	   WritePCDWord(fp,child_offset);
	   WritePCDWord(fp,0x5);
	   WritePCDWord(fp,string_offset);
	   WritePCDWord(fp,displayClass);
	   return foff;
   }

   virtual CString toString();

   virtual void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   //int string_offset;
   int displayClass;
   int child_offset;
};
//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

class NodeSelection17: public Node
{
   public:
   NodeSelection17(int secionOffset,Node *parent,int offset):Node(secionOffset,parent,0x17),tdo_offset(offset)
   {
	   subObject = NULL;
   }

   int allowedChildren() { return 0; }

   ~NodeSelection17()
   {
	   if (mip_strings) delete[] mip_strings;
   }

   void Read(FILE *fp)
   {
	   number_mip_strings = ReadPCDWord(fp);
	   
	   numberOfChildren = 0;
	   mip_strings = new int[number_mip_strings];
	   // get the child
       for(int i=0;i<number_mip_strings;i++)
	   {
		  mip_strings[i] = ReadPCDWord(fp);
	   }

   }

   int Write(FILE *fp,int secOffset)
   {
	   AfxMessageBox("Writer Procedure Not Implement Yet  For this NodeSelection17");
	   return -1;
   }

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   virtual CString toString();

   //int string_offset;
   int number_mip_strings;
   int *mip_strings;
   int tdo_offset;
   C3do * subObject;
};
//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

class NodeSelection11: public Node
{
   public:
   NodeSelection11(int secionOffset,Node *parent,int offset):Node(secionOffset,parent,0x17),child_offset(offset)
   {
   }

   int allowedChildren() { return 0; }

   void Read(FILE *fp)
   {
	   int number1 = ReadPCDWord(fp);
	   int number2 = ReadPCDWord(fp);
	   int number3 = ReadPCDWord(fp);
	   int number4 = ReadPCDWord(fp);
   }

   int Write(FILE *fp,int secOffset)
   {
	   AfxMessageBox("Writer Procedure Not Implement Yet  For this NodeSelection11");
	   return -1;
   }

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d (%x) %d %d %d %d",type,5,11,number1,number2,number3,number4);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number1;
   int number2;
   int number3;
   int number4;
   int child_offset;
};


//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
class NodeEleven: public Node
{
   public:
   NodeEleven(int secOffset,Node *parent):Node(secOffset,parent,0x11)
   {
	   number1 = 0;
	   number2 = 0;
	   number3 = 0;
   }

   ~NodeEleven()
   {
	   if (child_scales) delete[] child_scales;
   }

   int allowedChildren() { return -1; }

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   void Read(FILE *fp);
   int  Write(FILE *fp, int secOffset);

   CString toString();

   BOOL Edit();

   int number1;
   int number2;
   int number3;
   float *child_scales;
};
//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
/*
typedef struct
{
  int number1;
  int offset1;
  int offset2;
  int offset3;
  int offset4;
} t_FlavourEight;
*/

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
class NodePlane : public Node
{
	public:
		NodePlane(int secionOffset,Node *parent,int type):
		Node(secionOffset,parent,type)
		{
		}

		int number1;
		int number2;
		int number3;
		int number4;
		int number5;
};

class NodeEight: public NodePlane
{
   //t_FlavourEight info;
   //int number1;
   //int number2;
   //int number3;
   //int number4;
   //int number5;
   int number6;
   int offset1;
   int offset2;
   int offset3;
   int offset4;

   public:
   NodeEight(int secOffset,Node *parent):NodePlane(secOffset,parent,0x8)
   {
	   //memset(&info,0,sizeof(info));
	   bitmapid = IDB_PLANE;
   }

   int allowedChildren() { return 4; }

   int getNodePlaneNumber() { return number1; }

   BOOL Init();
   BOOL Edit();

   void Read(FILE *fp);
   void ReadGPL(FILE *fp);
   void ReadN2(FILE *fp);
   int  Write(FILE *fp,int secOffset);
   int  WriteN2(FILE *fp,int secOffset);
   int  WriteGPL(FILE *fp,int secOffset);

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("4 Child Flavor=%d Children=%d Plane ?Number=%d",type,numberOfChildren,number1);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }
};

//////////////////////////////////////////////////////////////
// Node 0x09
//////////////////////////////////////////////////////////////

class NodeNine: public NodePlane
{
   public:
   NodeNine(int secOffset,Node *parent):NodePlane(secOffset,parent,0x09)
   {
	   number1 = 0;
	   bitmapid = IDB_PLANE;
   }

   ~NodeNine()
   {
   }

   BOOL Init();

   int allowedChildren() { return 3; }

   int getNodePlaneNumber() { return number1; }

   BOOL Edit();

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);
   
   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d Children=%d Number=%d",type,numberOfChildren,number1);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   //int number1;
   //int number2;
   //int number3;
   //int number4;
   //int number5;
   int offset1;
   int offset2;
   int offset3;
};

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

class NodeB: public NodePlane
{
   public:
   NodeB(int secOffset,Node *parent):NodePlane(secOffset,parent,0x0B)
   {
	   number1 = 0;
	   bitmapid = IDB_PLANE;
   }

   int allowedChildren() { return 2; }

   BOOL Edit();
   BOOL Init();
   int OverLay(FILE *fp);

   void Read(FILE *fp);

   int Write(FILE *fp,int secOffset);

   int getNodePlaneNumber() { return number1; }

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavour=%d (0x%x) Children=%d Number=%d",type,type,numberOfChildren,number1);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   //int number1;
   //int number2;
   //int number3;
   //int number4;
   //int number5;
   int offset1;
   int offset2;
   int offset3;
};

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

typedef struct
{
   int v0;
   int v1;
   int v2;
   int v3;
   int v4;
   int v5;
   int v6;
   int v7;
   int offset1;
} t_FlavorNineteen;

class NodeNineteen: public Node
{
   t_FlavorNineteen info;

   public:
   NodeNineteen(int secOffset,Node *parent):Node(secOffset,parent,0x19)
   {
	   bitmapid = IDB_ROOM;
   }

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   BOOL Edit();

   int allowedChildren() { return 1; }

   void Read(FILE *fp);

   

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   WriteAllChildren(fp,secOffset);
	   if (children[0]->writingOffset!=-1){
	     info.offset1=children[0]->writingOffset-getOffset();
	   }
	   else {
		 info.offset1 = -1;
	   }
	   
	   int foff = ftell(fp);
	   sendCommandVal(fp,0x19);
	   fwrite(&info,1,sizeof(t_FlavorNineteen),fp);
	   return foff;
   }


   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d Children=%d Verticies= v0=%d v1=%d v2=%d v3=%d v4=%d v5=%d v6=%d v7=%d",
		   type,numberOfChildren,info.v0,info.v1,info.v2,info.v3,info.v4,
		   info.v5,info.v6,info.v7);
	   return buffer;
   }

   
   
};

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
typedef struct
{
   float x;
   float y;
   float z;
   float a;
   float b;
   float c;
   float scale;
} t_FlavorDInfo;

class NodeD: public Node
{
   public:
   NodeD(int secOffset,Node *parent):Node(secOffset,parent,0x0D)
   {
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);

   int Write(FILE *fp,int secOffset);

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);
 
   BOOL Edit();

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d Children=%d Scales= %2.2f %2.2f %2.2f %2.2f %2.2f %2.2f %2.2f ",type,numberOfChildren,info.x,info.y,info.z,info.a,info.b,info.c,info.scale);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   float getX(); 
   float getY(); 
   float getZ(); 
   float getRotX(); 
   float getRotY(); 
   float getRotZ(); 


   t_FlavorDInfo info;

   //float scale1;
   //float scale2;
   //float scale3;
   //float scale4;
   //float scale5;
   //float scale6;
   //float scale7;
   int offset1;



   
};


//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
class NodePolygon: public Node
{
public:
	NodePolygon(int secOffset,Node *parent,int subtype):Node(secOffset,parent,subtype)
	{
		color = 0;
		zero = 0;
		textureMip = NULL;
	}

	virtual ~NodePolygon();

	void Explore()
	{
		Edit();
	}

	void DefaultArgs()
	{
		verticies = new int[4];
		numberOfVerticies = 4;
		verticies[0] = 0;
		verticies[1] = 1;
		verticies[2] = 2;
		verticies[3] = 3;

		xtexture = new float[4];
		ytexture = new float[4];

		xtexture[0]  = ytexture[0] = 0.0f;
		xtexture[1]  = ytexture[3] = 0.0f;
		xtexture[3]  = ytexture[1] = 1.0f;
		xtexture[2]  = ytexture[2] = 1.0f;
	}

	int allowedChildren() { return 0; }

	void CalculateABCD();
	
    int zero;
    int numberOfVerticies;
    int color;
    int *verticies;
    float *xtexture;
    float *ytexture;
	int *vertexnormals;
	int *vertexcolors;

    CMip   *textureMip;

	float A;
	float B;
	float C;
	float D;
};

class NodePolygonTexture81f: public NodePolygon
{
   public:
   NodePolygonTexture81f(int secOffset,Node *parent,int subtype):NodePolygon(secOffset,parent,subtype)
   {
	   bitmapid = IDB_POLYGONS_TEXT;
	   textureMip = NULL;
   }

   virtual ~NodePolygonTexture81f()
   {
	   if (verticies) delete[] verticies;
	   if (xtexture)  delete[] xtexture;
	   if (ytexture)  delete[] ytexture;
	   //if (textureMip) delete textureMip;
   }

   virtual void Read(FILE *fp);

   

   virtual int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   WritePCDWord(fp,type);
	   WritePCDWord(fp,zero);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   for(int i=0;i<numberOfVerticies;i++)
	   {
			WritePCFloat(fp,xtexture[i]);
			WritePCFloat(fp,ytexture[i]);
	   }
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   return foff;
   }

   virtual CString toString();

   

   virtual void Draw(C3do* object,Display *display,CView *doview);
   virtual void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   BOOL Edit();

   virtual BOOL Init();
   
};

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

class NodePolygonFlat81b: public NodePolygon
{
   public:
   NodePolygonFlat81b(int secOffset,Node *parent,int subtype):NodePolygon(secOffset,parent,subtype)
   {
	   bitmapid = IDB_POLYGONS;
   }

   ~NodePolygonFlat81b()
   {
	   if (verticies) delete []verticies;
   }

   void Read(FILE *fp);

   

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   WritePCDWord(fp,type);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   return foff;
   }

   virtual CString toString();

   

   void Draw(C3do* object,Display *display,CView *doview);
   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   BOOL Edit();

   virtual BOOL Init();
   
};
//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

class NodeFour: public Node
{
   public:
   NodeFour(int secOffset,Node *parent):Node(secOffset,parent,0x4)
   {
	   numberOfChildren = 0;
   }

   int allowedChildren() { return -1; }

   void Read(FILE *fp);

   

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   WriteAllChildren(fp,secOffset);
	   int foff = ftell(fp);
	   if (foff != nodeOffset)
	   {
		   //cerr << "Offset Mismatch "<< type << " was " << nodeOffset+getOffset() <<" now "<< foff <<endl;
	   }
	   sendCommandVal(fp,0x4);
	   WritePCDWord(fp,numberOfChildren);

	   for(int i=0;i<numberOfChildren;i++)
	   {
		   int offset1;
		   if (children[i]->writingOffset!=-1){
	        offset1=children[i]->writingOffset-getOffset();
	       }
	       else {
		    offset1 = -1;
	      }	   
		  WritePCDWord(fp,offset1);
	   }
	   return foff;
   }

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }
};
//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

class NodeSix: public NodePlane
{
   public:
   NodeSix(int secOffset,Node *parent):NodePlane(secOffset,parent,0x6)
   {
	   bitmapid = IDB_PLANE;
   }

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   int allowedChildren() { return 1; }

   void Read(FILE *fp);

   int Write(FILE *fp,int secOffset);

   int getNodePlaneNumber() { return number1; }

   virtual CString toString();

   BOOL Init();

   BOOL Edit();

   //int number;
   //int number1;
   //int number2;
   //int number3;
   //int number4;
   //int number5;
};
//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

class NodeTen: public Node
{
   public:
   NodeTen(int secOffset,Node *parent):Node(secOffset,parent,0x10)
   {
   }

   ~NodeTen()
   {
	   if (entries) delete[] entries;
   }

   BOOL Init();
   
   int allowedChildren() { return 0; }

   void AddEntry(float val)
   {
	   number++;
	   DataValue *values = new DataValue[number];

	   for(int i=0;i<number-1;i++)
	   {
		   values[i].data = entries[i].data;
	   }
	   values[number-1].data = val;
	   delete entries;
	   entries = values;
   }

   void SetEntry(int pos,float val)
   {
	   entries[pos].data = val;
   }

   void Read(FILE *fp);
   

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   sendCommandVal(fp,0x10);
	   WritePCDWord(fp,number);
	   for(int i=0;i<number;i++)
	   {
	     WritePCDWord(fp,entries[i].getVal());
	   }
	   return foff;
   }

   virtual HTREEITEM LoadTree(HTREEITEM branch,CObjectTree *tree)
   {
	   HTREEITEM treeParent = tree->insertInfoNode(branch, (LPSTR)(LPCSTR)toString(),TO_ID(bitmapid));
	   tree->m_ObjectTree.SetItemData(treeParent,(DWORD)this);

	   for(int i=0;i<number;i++)
	   {
		 CString buffer;
		 buffer.Format("[%d]=%d",i,entries[i].getVal());
		 HTREEITEM entry= tree->insertInfoNode(treeParent, (LPSTR)(LPCSTR) buffer,TO_ID(bitmapid));
		 tree->m_ObjectTree.SetItemData(entry,(DWORD)&entries[i]);

	   }
	   return treeParent;
   }

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d Number of Entries=%d ",type,number);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number;
   DataValue *entries;
};
//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////
typedef struct
{
  int number1;
  int offset1;
  int number2;
  int offset2;
  int number3;
  int offset3;
  int number4;
  int offset4;
  int offset5;
  int offset6;
  /*
  int number1;
  int number2;
  int number3;
  int offset1;
  int number4;
  int number5;
  int number6;
  int number7;
  int offset2;
  int offset3;
  */
  int zero1;
  int zero2;
  int zero3;
  int zero4;
  int numItem;
} t_FlavourFifteen;

typedef struct
{
  int x;
  int y;
  int z;
  int offset;
} t_FlavourFifteenChild;

#define FLAVOR_FIFTEEN_NODES 6

class NodeFifteen: public Node
{  
   t_FlavourFifteen info;
   t_FlavourFifteenChild *child_info;

   public:

   NodeFifteen(int secOffset,Node *parent):Node(secOffset,parent,0xF)
   {
	   child_info = NULL;
   }

   void DisplayNodeAndChildren(ostream &os);

   int allowedChildren() { return FLAVOR_FIFTEEN_NODES; }

   ~NodeFifteen()
   {
	  if (child_info) delete[] child_info;
   }

   void Read(FILE *fp);
  
   int Write(FILE *fp,int secOffset);

   virtual void Draw(C3do* object,Display *display,CView *doview);
   virtual void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);
   

   virtual CString toString();

   //BOOL OffsetRefs[5];

   //Node *OffsetPts[5];
};
//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

class NodeLine815: public Node
{
   public:
   NodeLine815(int secOffset,Node *_parent):Node(secOffset,_parent,0x815)
   {
	   bitmapid = IDB_LINES;
	   v = NULL;
   }

   ~NodeLine815();

   void Read(FILE *fp);

   int allowedChildren() { return 0; }
   	
   int Write(FILE *fp,int secOffset);

   

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Line (%x) number=%d %d -> %d",type,number,v[0],v[1]);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   BOOL Edit();

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   int number;
   int numberofVerticies;
   int *v;
};


class NodeSeventeen: public Node
{
   public:
   NodeSeventeen(int secOffset,Node *parent):Node(secOffset,parent,0x17)
   {
   }

   ~NodeSeventeen()
   {
	   if (numbers) delete[] numbers;
   }

   int allowedChildren() { return 0; }

   void Read(FILE *fp);

   int Write(FILE *fp,int secOffset);

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Multi Flavor=%d Numbers=%d ",type,numNumbers);
	   for(int i=0;i<numNumbers;i++)
	   {
		   CString buffer1;
		   buffer1.Format("[%d]=%d",i,numbers[i]);
		   buffer+=buffer1;
	   }
	   return buffer;
   }

   int child_offset;
   int *numbers;
   int  numNumbers;
};



class NodeFourteen: public Node
{
   public:
   NodeFourteen(int secOffset,Node *parent):Node(secOffset,parent,0x14)
   {

	   
   }

   int allowedChildren() { return 0; }

   void Read(FILE *fp)
   {
	    number1  = ReadPCDWord(fp);
		number2  = ReadPCDWord(fp);
   }

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   sendCommandVal(fp,0x14);
	   WritePCDWord(fp,number1);
	   WritePCDWord(fp,number2);
	   return foff;
   }

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d number1=%d number2=%d ",type,number1,number2);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }
;
   int  number1;
   int  number2;
};

class NodePolygonTexture820: public NodePolygonTexture81f
{
   public:
   NodePolygonTexture820(int secOffset,Node *parent);
   
   ~NodePolygonTexture820();
   
   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);
   virtual CString toString();

   BOOL Init();

   //int *vertexcolors;
};


class NodePolygonTexture821: public NodePolygonTexture81f
{
   public:
   NodePolygonTexture821(int secOffset,Node *parent);
   
   ~NodePolygonTexture821();

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);
   
   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);
   virtual CString toString();

   BOOL Init();

   //int *vertexnormals;
};

class NodePolygonTexture20: public NodePolygonTexture81f
{
   public:
   NodePolygonTexture20(int secOffset,Node *parent);
   
   ~NodePolygonTexture20();
   
   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);
   virtual CString toString();

   int *vertexcolors;
};


class NodePolygonTexture1f: public NodePolygonTexture81f
{
   public:
   NodePolygonTexture1f(int secOffset,Node *parent);
   
   ~NodePolygonTexture1f();
   
   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);
   virtual CString toString();

   //int *vertexcolors;
};


class NodePolygonTexture21: public NodePolygonTexture81f
{
   public:
   NodePolygonTexture21(int secOffset,Node *parent);
   
   ~NodePolygonTexture21();
   
   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);
   virtual CString toString();

   //int *vertexnormals;
};


class NodePolygonFlat81c: public NodePolygonFlat81b
{
   public:
   NodePolygonFlat81c(int secOffset,Node *parent);

   virtual ~NodePolygonFlat81c();  
   virtual void Read(FILE *fp);  
   virtual int Write(FILE *fp,int secOffset);
   virtual CString toString();
   

   virtual BOOL Init();
};

class NodePolygonFlat1c: public NodePolygonFlat81b
{
   public:
   NodePolygonFlat1c(int secOffset,Node *parent);

   virtual ~NodePolygonFlat1c();  
   virtual void Read(FILE *fp);  
   virtual int Write(FILE *fp,int secOffset);
   virtual CString toString();
   

   virtual BOOL Init();
};

class NodePolygonFlat81d: public NodePolygonFlat81b
{
   public:
   NodePolygonFlat81d(int secOffset,Node *parent);

   virtual ~NodePolygonFlat81d();  
   virtual void Read(FILE *fp);  
   virtual int Write(FILE *fp,int secOffset);
   virtual CString toString();

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);
   

   virtual BOOL Init();
};

class NodePolygonFlat81e: public NodePolygonFlat81b
{
   public:
   NodePolygonFlat81e(int secOffset,Node *parent);

   virtual ~NodePolygonFlat81e();  
   virtual void Read(FILE *fp);  
   virtual int Write(FILE *fp,int secOffset);
   virtual CString toString();

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);
   //int *vertexnormals;
   //int *vertexcolors;

   virtual BOOL Init();
};

class NodePolygonN2_FL2: public NodePolygonTexture81f
{
   public:
   NodePolygonN2_FL2(int secOffset,Node *parent);

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   virtual ~NodePolygonN2_FL2();  
   virtual void Read(FILE *fp);  
   virtual int Write(FILE *fp,int secOffset);
   virtual CString toString();

   virtual BOOL Init();

   int mode;
   int *vinfo;
};



class NodeSelection0c: public Node
{
   public:
   NodeSelection0c(int secionOffset,Node *parent,int offset);

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   

   int Write(FILE *fp,int secOffset);

   virtual CString toString();

   int number1;
   int number2;
   int number3;
   float number4;
   int child_offset;
};

class NodeSelection8: public Node
{
   public:
   NodeSelection8(int secionOffset,Node *parent,int offset);

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   

   int Write(FILE *fp,int secOffset);

   virtual CString toString();

   int number1;
   int number2;
   int number3;
   float number4;
   int child_offset;
};



class NodeSelection0d: public Node
{
   public:
   NodeSelection0d(int secionOffset,Node *parent,int offset);

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   

   int Write(FILE *fp,int secOffset);

   virtual CString toString();

   int number1;
   int number2;
   float number4;
   int child_offset;
   float scale;
};



class NodeSelection20: public Node
{
   public:
   NodeSelection20(int secionOffset,Node *parent,int offset);

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   

   int Write(FILE *fp,int secOffset);

   virtual CString toString();

   int number1;
   int number2;
   int number3;
   int number4;
   int child_offset;
};


class NodeSelection24: public Node
{
   public:
   NodeSelection24(int secionOffset,Node *parent,int offset);

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   

   int Write(FILE *fp,int secOffset);

   virtual CString toString();

   int number1;
   int number2;
   int child_offset;
};

class NodeSelection2c: public Node
{
   public:
   NodeSelection2c(int secionOffset,Node *parent,int offset);

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   

   int Write(FILE *fp,int secOffset);

   virtual CString toString();

   int number1;
   int number2;
   int number3;
   float number4;
   int child_offset;
};


class NodeSeven: public NodePlane
{
   public:
   NodeSeven(int secOffset,Node *parent):NodePlane(secOffset,parent,0x7)
   {
	   bitmapid = IDB_PLANE;
   }
   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   int allowedChildren() { return -1; }

   void Read(FILE *fp);

   BOOL Init();
   BOOL Edit();

   int Write(FILE *fp,int secOffset);

   int getNodePlaneNumber() { return number1; }

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   //int number1;
   //int number2;
   //int number3;
   //int number4;
   //int number5;
};

//////////////////////////////////////////////////////////////
// Node
//////////////////////////////////////////////////////////////

typedef struct
{
   float x;
   float y;
   float z;
   float a;
   float b;
   float c;
   float scale;
   int offset1;
} t_FlavorSixteen;

class NodeSixteen: public Node
{
   t_FlavorSixteen info;

   public:
   NodeSixteen(int secOffset,Node *parent):Node(secOffset,parent,0x16)
   {
   }

   int allowedChildren() { return 1; }

   BOOL Edit();

   void Read(FILE *fp);
	   
   
   

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   WriteAllChildren(fp,secOffset);
	   if (children[0]->writingOffset!=-1){
	     info.offset1=children[0]->writingOffset-getOffset();
	   }
	   else {
		 info.offset1 = -1;
	   }
	   
	   int foff = ftell(fp);
	   sendCommandVal(fp,0x16);
	   fwrite(&info,1,sizeof(t_FlavorSixteen),fp);
	   return foff;
   }

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);


   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d Children=%d Scales= %2.2f %2.2f %2.2f %2.2f %2.2f %2.2f %2.2f",
		   type,numberOfChildren,info.x,info.y,info.z,info.a,info.b,
		   info.c,info.scale);
	   return buffer;
   }

   
   
};

class NodeA03: public Node
{
   public:
   NodeA03(int secOffset,Node *parent):Node(secOffset,parent,0x3)
   {
	   bitmapid = IDB_SRB;
	   //textureSrb = NULL;
	   number =0;
	   color =0;
	   child_offset =0;
	   scale = 1.0f;
   }

   BOOL Edit();
   int OverLay(FILE *fp);

   //int  LoadSrb();

   void Read(FILE *fp);

   

   int Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   sendCommandVal(fp,0xA03);
	   WritePCDWord(fp,number);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,string_offset);
	   WritePCFloat(fp,scale);
	   WritePCDWord(fp,child_offset);
	   return foff;
   }

   //BOOL Edit();

   virtual CString toString();

   //virtual HTREEITEM LoadTree(HTREEITEM branch,CObjectTree *tree);

   

   //void Draw(C3do* object,Display *display,CView *doview);
   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   int number;
   int color;
   float scale;
   int child_offset;
   int allowedChildren() { return 1; }

   //CSrb   *textureSrb;
};

class NodeC: public Node
{
   public:
   NodeC(int secOffset,Node *parent):Node(secOffset,parent,0xC)
   {
	   numberOfChildren = 0;
   }

   int allowedChildren() { return -1; }

   void Read(FILE *fp);

   

   int Write(FILE *fp,int secOffset);

   

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Multi Node Flavor=%d Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }
};
class NodeA: public NodePlane
{
   public:
   NodeA(int secOffset,Node *parent):NodePlane(secOffset,parent,0x0A)
   {
	   number1 = 0;
	   bitmapid = IDB_PLANE;
   }

   ~NodeA()
   {
   }

   BOOL Init();

   int OverLay(FILE *fp);

   int allowedChildren() { return 3; }

   int getNodePlaneNumber() { return number1; }

   BOOL Edit();

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);
   
   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%d Children=%d Number=%d",type,numberOfChildren,number1);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   //int number1;
   //int number2;
   //int number3;
   //int number4;
   //int number5;
   int offset1;
   int offset2;
   int offset3;
};

class NodeFourICR2: public Node
{
   public:
   C3do *subObject;

   NodeFourICR2(int secOffset,Node *parent):Node(secOffset,parent,0x8000004)
   {
	   numberOfChildren = 1;
	   subObject = NULL;
   }

   int allowedChildren() { return 1; }

   CString Export();

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   void Read(FILE *fp)
   {
	    string_offset        = ReadPCDWord(fp);
		number               = ReadPCDWord(fp);
	    children = (Node**)malloc(numberOfChildren*sizeof(Node*));
	
		int child_offset  = ReadPCDWord(fp);
		 
		if (child_offset && child_offset!=-1)
		{
		 int ltoff = ftell(fp);
	     fseek(fp,getOffset()+child_offset,SEEK_SET);
	     children[0]       = getChildType(sectionOffset,this,fp,object);
		 fseek(fp,ltoff,SEEK_SET);
		}
		else
		{
		 children[0] = new EmptyNode(sectionOffset,this);
		}
   }

   int Write(FILE *fp,int secOffset);
   
   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d String_offset=%d Number=%d",type,numberOfChildren,string_offset,number);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number;
};

class NodeSevenICR2: public Node
{
   public:
   NodeSevenICR2(int secOffset,Node *parent):Node(secOffset,parent,0x8000007)
   {
	   numberOfChildren = 3;
   }

   int allowedChildren() { return 3; }

   CString Export();

   void Read(FILE *fp)
   {
		number1               = ReadPCDWord(fp);
		number2               = ReadPCDWord(fp);
		number3               = ReadPCDWord(fp);
		number4               = ReadPCDWord(fp);
		number5               = ReadPCDWord(fp);
	    children = (Node**)malloc(numberOfChildren*sizeof(Node*));
	
		 offset1  = ReadPCDWord(fp);
		 offset2  = ReadPCDWord(fp);
		 offset3  = ReadPCDWord(fp);
		 
		if (offset1 && offset1!=-1)
		{
		 int ltoff = ftell(fp);
	     fseek(fp,getOffset()+offset1,SEEK_SET);
	     children[0]       = getChildType(sectionOffset,this,fp,object);
		 fseek(fp,ltoff,SEEK_SET);
		}
		else
		{
		 children[0] = new EmptyNode(sectionOffset,this);
		}

		if (offset2 && offset2!=-1)
		{
		 int ltoff = ftell(fp);
	     fseek(fp,getOffset()+offset2,SEEK_SET);
	     children[1]       = getChildType(sectionOffset,this,fp,object);
		 fseek(fp,ltoff,SEEK_SET);
		}
		else
		{
		 children[1] = new EmptyNode(sectionOffset,this);
		}

		if (offset3 && offset3!=-1)
		{
		 int ltoff = ftell(fp);
	     fseek(fp,getOffset()+offset3,SEEK_SET);
	     children[2]       = getChildType(sectionOffset,this,fp,object);
		 fseek(fp,ltoff,SEEK_SET);
		}
		else
		{
		 children[2] = new EmptyNode(sectionOffset,this);
		}
   }

   int Write(FILE *fp,int secOffset);
   

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number1;
   int number2;
   int number3;
   int number4;
   int number5;
   int offset1;
   int offset2;
   int offset3;
};



class NodeFiveICR2: public Node
{
   public:
   NodeFiveICR2(int secOffset,Node *parent):Node(secOffset,parent,0x8000005)
   {
	   numberOfChildren = 1;
   }

   CString Export();

   int allowedChildren() { return 1; }

   void Read(FILE *fp)
   {
		number1               = ReadPCDWord(fp);
		number2               = ReadPCDWord(fp);
		number3               = ReadPCDWord(fp);
		number4               = ReadPCDWord(fp);
		number5               = ReadPCDWord(fp);
	    children = (Node**)malloc(numberOfChildren*sizeof(Node*));
	
		offset1  = ReadPCDWord(fp);
		 
		if (offset1 && offset1!=-1)
		{
		 int ltoff = ftell(fp);
	     fseek(fp,getOffset()+offset1,SEEK_SET);
	     children[0]       = getChildType(sectionOffset,this,fp,object);
		 fseek(fp,ltoff,SEEK_SET);
		}
		else
		{
		 children[0] = new EmptyNode(sectionOffset,this);
		}

		
   }

   int Write(FILE *fp,int secOffset);
   

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number1;
   int number2;
   int number3;
   int number4;
   int number5;
   int offset1;
};

class NodePolygonOneICR2: public NodePolygon
{
   public:
   NodePolygonOneICR2(int secOffset,Node *parent,int subtype):NodePolygon(secOffset,parent,subtype)
   {
	   bitmapid = IDB_POLYGONS;
   }

   CString Export();

   ~NodePolygonOneICR2()
   {
	   if (verticies) delete []verticies;
   }

   void Read(FILE *fp);

   

   int Write(FILE *fp,int secOffset);
   

   virtual CString toString();
   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

};

class NodePolygonTwoICR2: public NodePolygon
{
   public:
   NodePolygonTwoICR2(int secOffset,Node *parent,int subtype):NodePolygon(secOffset,parent,subtype)
   {
	   bitmapid = IDB_POLYGONS;
   }

   ~NodePolygonTwoICR2()
   {
	   if (verticies) delete []verticies;
   }

   void Read(FILE *fp);

   CString Export();

   int Write(FILE *fp,int secOffset);
   
   virtual CString toString();
   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

};

class NodeTenICR2: public Node
{
   public:
   NodeTenICR2(int secOffset,Node *parent):Node(secOffset,parent,0x8000010)
   {
	   numberOfChildren = 1;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);

   CString Export();

   
   int Write(FILE *fp,int secOffset);

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number;
};

class NodeTwelveICR2: public Node
{
   public:
   NodeTwelveICR2(int secOffset,Node *parent):Node(secOffset,parent,0x8000012)
   {
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);

   
   int Write(FILE *fp,int secOffset);

   virtual CString toString();

   CString Export();

   int number1;
   int number2;
   int number3;
   int number4;
   int number5;
   int number6;
};

class NodeFifteenICR2: public Node
{
   public:
   NodeFifteenICR2(int secOffset,Node *parent):Node(secOffset,parent,0x800000f)
   {
	   numberOfChildren = 0;
	   subObject = NULL;
	   found = TRUE;
   }

   int allowedChildren() { return 0; }

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);

   virtual CString toString();

   CString Export();

   void DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture);

   int number1;
   int number2;
   int number3;
   int number4;
   int number5;
   int number6;
   int offset1;

   C3do *subObject;
   double x,y,z;
   double a,b,c;
   BOOL found;
};


class NodeDICR2: public Node
{
   public:
   NodeDICR2(int secOffset,Node *parent):Node(secOffset,parent,0x800000d)
   {
	   numberOfChildren = 1;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);

   
   int Write(FILE *fp,int secOffset);


   CString Export();
   

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d number1=%d (%d %d)(%d %d)(%d %d)(%d %d)",type,numberOfChildren,number1,number2,offset1,number3,offset2,number4,offset3,number5,offset4);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number1;
   int number2;
   int number3;
   int number4;
   int number5;
   int offset1;
   int offset2;
   int offset3;
   int offset4;

   
};

class NodeAICR2: public Node
{
   public:
   NodeAICR2(int secOffset,Node *parent):Node(secOffset,parent,0x800000a)
   {
	   numberOfChildren = 1;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);
   int OverLay(FILE *fp);

   CString Export();

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number1;
   int number2;
   int number3;
   int number4;
   int number5;
   int offset1;
   int offset2;
};

class NodeThreeICR2: public Node
{
   public:
   NodeThreeICR2(int secOffset,Node *parent):Node(secOffset,parent,0x8000003)
   {
	   numberOfChildren = 1;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);


   CString Export();

   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number;
};

class NodeSixICR2: public Node
{
   public:
   NodeSixICR2(int secOffset,Node *parent):Node(secOffset,parent,0x8000006)
   {
	   numberOfChildren = 1;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);
   
   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   CString Export();

   int number1;
   int number2;
   int number3;
   int number4;
   int number5;
   int offset1;
   int offset2;
};

class NodeEightICR2: public Node
{
   public:
   NodeEightICR2(int secOffset,Node *parent):Node(secOffset,parent,0x8000008)
   {
	   numberOfChildren = 1;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);
   
   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   CString Export();

   int number1;
   int number2;
   int number3;
   int number4;
   int number5;
   int offset1;
   int offset2;
   int offset3;
};

class NodeBICR2: public Node
{
   public:
   NodeBICR2(int secOffset,Node *parent):Node(secOffset,parent,0x800000b)
   {
	   numberOfChildren = 1;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);

   CString Export();
   
   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number;
};

class NodeNineICR2: public Node
{
   public:
   NodeNineICR2(int secOffset,Node *parent):Node(secOffset,parent,0x8000009)
   {
	   numberOfChildren = 1;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);

   CString Export();
   
   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number1;
   int number2;
   int number3;
   int number4;
   int number5;
   int offset1;
   int offset2;
   int offset3;
   int offset4;
};

class NodeCICR2: public Node
{
   public:
   NodeCICR2(int secOffset,Node *parent):Node(secOffset,parent,0x800000c)
   {
	   numberOfChildren = 1;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);

   CString Export();
   
   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number;
};

class NodeEICR2: public Node
{
   public:
   NodeEICR2(int secOffset,Node *parent):Node(secOffset,parent,0x800000e)
   {
	   numberOfChildren = 1;
   }

   int allowedChildren() { return 1; }

   void Read(FILE *fp);
   int Write(FILE *fp,int secOffset);

   CString Export();
   
   virtual CString toString()
   {
       CString buffer;
	   buffer.Format("Flavor=%x Children=%d",type,numberOfChildren);
	   //if (getObject()->showOffset)
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
   }

   int number;
};






#endif