#include "stdafx.h"
#include "Node.h"

#include "DatView.h"
#include "3DOEditorView.h"

#include "EditObjPos.h"
#include "EditTwoVar.h"
#include "EditOneVar.h"

#include <gl\glaux.h>
#include <gl\gl.h>
#include <gl\glu.h>

CString Node3doObject::toString()
{
       CString buffer;
	   CString name = getObject()->findStrName(string_offset);
	   buffer.Format("Select 3DO Object File Type=%d Children=%d String_offset=%d(%s) number=%d",type,numberOfChildren,string_offset,name,number);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

BOOL Node3doObject::Edit()
{

	CEditTwoVar *dlg = new CEditTwoVar();

	dlg->m_VarVal1 = string_offset;
	dlg->m_VarVal2 = number;
	dlg->m_Var1.Format("String Offset");
	dlg->m_Var2.Format("Number?");
	dlg->m_Desc.Format("3do Object\nString offset is ptr to name of 3do file");
	
	int result = dlg->DoModal();

	if (result == IDOK)
	{
		string_offset = dlg->m_VarVal1;
		number = dlg->m_VarVal2;
	}

	if (dlg) delete dlg;
	return TRUE;
}

BOOL Node3doObjectPosition::Edit()
{
	CEditObjPos *dlg = new CEditObjPos();

	dlg->m_X = info.x;
	dlg->m_Y = info.y;
	dlg->m_Z = info.z;
	dlg->m_A = info.a;
	dlg->m_B = info.b;
	dlg->m_C = info.c;
	dlg->m_Scale = info.scale;

	int result = dlg->DoModal();

	if (result == IDOK)
	{
		info.x = dlg->m_X;
		info.y = dlg->m_Y;
		info.z = dlg->m_Z;
		info.a = dlg->m_A;
		info.b = dlg->m_B;
		info.c = dlg->m_C;
		info.scale = dlg->m_Scale;
	}

	if (dlg) delete dlg;
	return TRUE;
}

void Node3doObject::Draw(C3do* object,Display *display,CView *doview)
{
	   DatView *view = (DatView*)doview;

	   if (!view->showObjects) return;

	   Node3doObjectPosition *position = (Node3doObjectPosition*)getParent();

	   display->drawBitmap(IDB_OBJECT,position->getX(),position->getY());

	   CString name = object->findStrName(string_offset);
	   display->drawText(position->getX(),position->getY(),name);
}



void Node3doObject::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	   CMy3DOEditorView *view = (CMy3DOEditorView*)doview;

	   if (!view->GetDocument()->showObjects) return;

	   BOOL isN2 = (getObject()->getVersion() == t_N2) ? TRUE : FALSE;
	   GLdouble wx,wy,wz;
	   GLdouble ax,ay,az;

	   wx = wy = wz = 0.0;
	   ax = ay = az = 0.0;

	   /*
	   Node *parentalNode = getParent();
	   int   parentalType = parentalNode->type;

	   if (isN2)
	   {
		if (parentalType  == 0xD)
		{
	     NodeD *position = (NodeD*)getParent();
		 wx = position->getX();
	     wy = position->getY();
	     wz = position->getZ();
		 ax = position->getRotX();
	     ay = position->getRotY();
	     az = position->getRotZ();
		}
		else
		{         
		}
	   }
	   else
	   {
		Node3doObjectPosition *position = (Node3doObjectPosition*)getParent();
		wx = position->getX();
	    wy = position->getY();
	    wz = position->getZ();
		ax = 0;
		ay = 0;
		az = 0;
	   }
	   */
	   
	   if (view->GetDocument()->showSubObjectLocations)
	   {
       glPointSize(5.0f);
	   glBegin(GL_POINTS);	   
	   glVertex3d(wx,wy,wz);
	   glEnd();
	   glPointSize(1.0f);
	   }
	   glPushMatrix();
	   //glTranslated(wx,wy,wz);
	   //glRotatef((float)ax,1.0f,0.0f,0.0f);
	   //glRotatef((float)ay,0.0f,1.0f,0.0f);
	   //glRotatef((float)az,0.0f,0.0f,1.0f);


	   if (subObject == NULL)
	   {
	    CString name = object->findStrName(string_offset);
	    if (!stricmp(name,"horiz") || !stricmp(name,"sky")) 
		{
		   glPopMatrix();
		   return;
		}
		subObject = object->FindObjectFile(name);
		if (subObject){
		}
        else{
		   cerr << "Failed to load subObject " << name << endl;
		}
	   }

	   if (subObject)
	   {
		 subObject->GetFile(FALSE);
		 if (subObject->topDrawlocation)
		 {
		   //subObject->topDrawlocation->ClearDrawnStatus();
		   for(int i=0;i<subObject->topDrawlocation->size();i++)
		   {
			  Node * topDraw = (Node*) subObject->topDrawlocation->elementAt(i);
			  topDraw->DrawGL(hdc,subObject,doview,currentTexture);
		   }
	       
		 }
	   }
	   
	   glPopMatrix();
}


float Node3doObjectPosition::getX() 
{ return info.x; }
float Node3doObjectPosition::getY() 
{ return info.y; }
float Node3doObjectPosition::getZ() 
{ return info.z; }
float Node3doObjectPosition::getRotX() 
{ return info.b; } 
float Node3doObjectPosition::getRotY() 
{ return info.c; }
float Node3doObjectPosition::getRotZ() 
{ return info.a; }

 CString NodeSelection17::toString()
{
	   CString obj = object->findStrName(tdo_offset);
       CString buffer;
	   buffer.Format("Flavor=%d (%x) 3doOffset=%d(%s) Mips=%d ",type,17,tdo_offset,obj,number_mip_strings);
	   CString buffer1;
	   for(int i=0;i<number_mip_strings;i++)
	   {
	     buffer1.Format("mip[%d]=%d ",i,mip_strings[i]);
		 buffer+=buffer1;
	   }
	   return buffer;
}

void NodeSelection17::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	   glPushMatrix();

	   if (subObject == NULL)
	   {
	    CString name = object->findStrName(0);
	    if (!stricmp(name,"horiz") || !stricmp(name,"sky")) 
		{
		   glPopMatrix();
		   return;
		}
		subObject = object->FindObjectFile(name);
		if (subObject){
		}
        else{
		   cerr << "Failed to load subObject " << name << endl;
		}
	   }

	   if (subObject)
	   {
		 subObject->GetFile(FALSE);
		 if (subObject->topDrawlocation)
		 {
		   //subObject->topDrawlocation->ClearDrawnStatus();
		   for(int i=0;i<subObject->topDrawlocation->size();i++)
		   {
			  Node * topDraw = (Node*) subObject->topDrawlocation->elementAt(i);
			  topDraw->DrawGL(hdc,subObject,doview,mip_strings[0]);
		   }
		 }
	   }
	   
	   glPopMatrix();
}

BOOL NodeSixteen::Edit()
{
	CEditObjPos *dlg = new CEditObjPos();

	dlg->m_X = info.x;
	dlg->m_Y = info.y;
	dlg->m_Z = info.z;
	dlg->m_A = info.a;
	dlg->m_B = info.b;
	dlg->m_C = info.c;
	dlg->m_Scale = info.scale;

	int result = dlg->DoModal();

	if (result == IDOK)
	{
		info.x = dlg->m_X;
		info.y = dlg->m_Y;
		info.z = dlg->m_Z;
		info.a = dlg->m_A;
		info.b = dlg->m_B;
		info.c = dlg->m_C;
		info.scale = dlg->m_Scale;
	}

	if (dlg) delete dlg;
	return TRUE;
}


void NodeSixteen::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	   CMy3DOEditorView *view = (CMy3DOEditorView*)doview;

	   GLdouble wx,wy,wz;
	   GLdouble ax,ay,az;

	   wx = info.x;
	   wy = info.y;
	   wz = info.z;

	   ax = info.c;
	   ay = info.b;
	   az = info.a;

	   if (view->GetDocument()->showSubObjectLocations)
	   {
	   glPointSize(5.0f);
	   glBegin(GL_POINTS);	   
	   glVertex3d(wx,wy,wz);
	   glEnd();
	   glPointSize(1.0f);
	   }
	   glPushMatrix();
	   
	   glRotatef((float)ax,1.0f,0.0f,0.0f);
	   glRotatef((float)ay,0.0f,1.0f,0.0f);
	   glRotatef((float)az,0.0f,0.0f,1.0f);
	   glTranslated(wx,wy,wz);

	   Node::DrawGL(hdc,object,doview,currentTexture);
			   
	   glPopMatrix();
}

void Node3doObjectPosition::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	   CMy3DOEditorView *view = (CMy3DOEditorView*)doview;

	   GLdouble wx,wy,wz;
	   GLdouble ax,ay,az;

	   wx = getX();
	   wy = getY();
	   wz = getZ();

	   ax = getRotX();
	   ay = getRotY();
	   az = getRotZ();

	   glPointSize(5.0f);
	   glBegin(GL_POINTS);
	   
	   glVertex3d(wx,wy,wz);
	   glEnd();
	   glPointSize(1.0f);
	   glPushMatrix();
	   
	   glRotatef((float)ax,1.0f,0.0f,0.0f);
	   glRotatef((float)ay,0.0f,1.0f,0.0f);
	   glRotatef((float)az,0.0f,0.0f,1.0f);
	   glTranslated(wx,wy,wz);

	   Node::DrawGL(hdc,object,doview,currentTexture);
			   
	   glPopMatrix();
}

void Node3doObjectPosition::Read(FILE *fp)
{
	    //fread(&info,1,sizeof(t_FlavorThree),fp);
	    fread(&info,1,sizeof(t_FlavorThree),fp);
		numberOfChildren = 1;
	    children = (Node**)malloc(sizeof(Node*));
		fseek(fp,getOffset()+info.offset,SEEK_SET);
		//object->trackBufferPos = getOffset()+info.offset;
		children[0] = getChildType(sectionOffset,this,fp,object);
		//if (children[0]) children[0]->Read(fp);
}

void Node3doObject::Read(FILE *fp)
{
	    string_offset  = ReadPCDWord(fp);
	    number         = ReadPCDWord(fp);
}