#include "stdafx.h"
#include "Node.h"
#include "math.h"

#include "DatView.h"
#include "3DOEditorView.h"
#include "CMip.h"


#include "PolygonEditor.h"
#include "NodeNineteenEdit.h"

#include <gl\glaux.h>
#include <gl\gl.h>
#include <gl\glu.h>

#include "Palette.h"

extern void WritePCWord(FILE *fp,int val);
extern void setGLColor(float r,float g,float b);
extern void setRandomColor();

NodePolygon::~NodePolygon()
{
		
}

CString NodePolygonTexture81f::toString()
{
       CString buffer;
	   CalculateABCD();
	   buffer.Format("Textured Polygon (%x) Color=%s Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

CString NodePolygonFlat81b::toString()
{
       CString buffer;
	   CalculateABCD();
	   buffer.Format("Flat Polygon (%x) Color=%s Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

static int polygoncount=1;

#define PI 3.1415927

void DrawMyPlane(double xo,double yo,double zo,float r,float g,float b,CPlanes *plane,double normalSize)
{
    double A = plane->getA();  
	double B = plane->getB();
	double C = plane->getC();
	double D = plane->getD();

	GLdouble x1,y1,z1,ang1,dis1;
	GLdouble x2,y2,z2,ang2,dis2;
	GLdouble x3,y3,z3;

	glPushMatrix();
    x1=A*-D; y1=B*-D; z1=C*-D; 
	//ignoring y 
	ang1=atan2(z1,x1); 
	dis1=sqrt((x1*x1)+(z1*z1)); 
	x2=dis1; 
	y2=y1; 
	z2=0; 

	//ignoring z (which is now zero) 
	ang2=atan2(y2,x2); 
	dis2=sqrt((x2*x2)+(y2*y2)); 
	x3=dis2; 
	y3=0; 
	z3=0; 

	glColor3d(r,g,b);
	glBegin(GL_LINES);

	glVertex3d(A*-D, B*-D, C*-D);
	glVertex3d((A*-D)+A,(B*-D)+B,(C*-D)+C); 
	//glVertex3d(A,B,C); 
	glEnd();
	
	setGLColor(r,g,b);

	//and rotate around the y axis by angle 1 
	glRotated(-(ang1*180.0/PI), 0.0,1.0,0.0); 
	//and rotate around the z axis by angle 2 		
	glRotated((ang2*180.0/PI), 0.0,0.0,1.0); 
	//now take four 2d points in yz plane: 
	glBegin(GL_POLYGON); 

			
	glVertex3d(xo+x3,yo+normalSize,zo+normalSize); 
	glVertex3d(xo+x3,yo+normalSize,zo-normalSize); 
	glVertex3d(xo+x3,yo-normalSize,zo-normalSize); 
	glVertex3d(xo+x3,yo-normalSize,zo+normalSize); 
	glEnd(); 

	// line(a*-d, b*-d, c*-d)-((a*-d)+a,(b*-d)+b,(c*-d)+c) 
	
				
	glPopMatrix();
}

void NodePolygonTexture81f::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
			//if (drawn) return;
	        CMy3DOEditorView *view = (CMy3DOEditorView*)doview;
			int textureidx = -1;
			BOOL isN2 = getObject()->isN2Version();
			int width,height;

			width = height = 2;

			if (view->GetDocument()->showTextures)
			{ 
			   //int string_offset = getTextureStringOffset();
			   int string_offset = currentTexture;
			   if  (string_offset!=-1)
			   {
			    textureidx   = object->findStrNameId(string_offset);
			    CString name = object->findStrName(string_offset);
			    if (textureMip==NULL)
			    {	
				    // you haven't loaded it previously
			        if((CMip*)object->datFileParent)
			        {			    
						// we can get it from the dat file
			            textureMip = (CMip*)object->datFileParent->FindImageFile(name);
					}

					if (textureMip==NULL)
					{
					   // oh dear we still don't have it how about locally on the disk
				       CString FileName;
					   FileName.Format("%s.mip",name);
					   textureMip = new CMip();
				       BOOL loadedMipLocal = textureMip->Read(FileName);
				       if (!loadedMipLocal)
				       {
						 //stil could not find the mip file
					     cerr << "Failed to Find Image File (either in track or locally)" << name << ".mip" << endl;
				       }
			        }
			   }
			
			   if (textureidx != -1 && textureMip)
			   {
			      textureMip->GetFile();
			   
			      if(textureMip->getBitmap())
			      {	     
			      BMP  *bmpA  = textureMip->getBitmap();
			      width  = bmpA->ImageWidth();
			      height = bmpA->ImageHeight();
			      if (textureMip->bits==NULL)
			      {
				    textureMip->bitmapData.CreateDIB(textureMip->cbmp);
				    textureMip->output.LoadBitmap(IDB_TESTTEXTURE_128);
			        Mak256SquareImage(textureMip->cbmp,textureMip->output,width,height,128,128,128,128);
			      }			  
			      CDC *pDC = CDC::FromHandle(hdc);
			      textureMip->bits = TextureMap(pDC,textureMip->output,128,128,textureMip->bits);
			      glEnable(GL_TEXTURE_2D);
			      }			  
			      else
			      {
				  // cannot find the bitmap
			      glDisable(GL_TEXTURE_2D);
			      }			 
			   }
			  }
			}

			int red,green,blue;

			if (color >= 0 && color < 255)
			{
	         red    = colorIndex[color].r;
			 green  = colorIndex[color].g;
			 blue   = colorIndex[color].b;

			 if (color != 0 && red==0 && blue==0 && green==0 && color!=31)
			 {
			  CString msg;
			  msg.Format("(%d)",color);
			  cerr << msg;
			 }
			}
			else
			{
			  red   = (color & 0x00FF0000) >> 16;
		      green = (color & 0x0000FF00) >> 8;
		      blue  = (color & 0x000000FF);
			}

			if (getObject()->selection == this)
			{
				red = 255;
				green = 0;
				blue = 0;
			}

			if (view->GetDocument()->showRandomColors)
			{
				setRandomColor();
			}
			else
			{
				setGLColor((float)(red/256.0),(float)(green/256.0),(float)(blue/256.0));
			}
			
			if (view->renderMode == GL_SELECT)
			{
				glLoadName((DWORD)this);
				polygoncount++;
			}

            if (view->GetDocument()->viewType != t_WireFrame) 
				glBegin(GL_POLYGON);
			else 
				glBegin(GL_LINE_LOOP);

			for(int i=0;i<numberOfVerticies;i++)
			{
			 int idxa = verticies[i];
			 if (idxa < 0 || idxa >= object->numPts) continue;
			 Point3D *ptA = object->ptsArray[idxa];
			 GLdouble wx,wy,wz,tx,ty;
			 wx = (double)ptA->getX();
	         wy = (double)ptA->getY();
	         wz = (double)ptA->getZ();
			 
			 if (xtexture)
			 {
			  if (isN2)
			  {
			    tx = (double)xtexture[i]/width; 
			    ty = (double)ytexture[i]/height; 
			  }
			  else
			  {
				tx = (double)xtexture[i]; 
			    ty = (double)ytexture[i]; 
			  }
			 }
			 else
			 {
			  switch(i)
			  {
			     case 0:tx=0.0;ty=0.0;break;
				 case 1:tx=1.0;ty=0.0;break;
				 case 2:tx=1.0;ty=1.0;break;
				 case 3:tx=0.0;ty=1.0;break;
			  }
			  
			 }
			 glTexCoord2f((float)tx,(float)(1.0-ty));
			 glVertex3d(wx,wy,wz);
			}
			glEnd();


			if (view->GetDocument()->viewType == t_PolygonOutlines) 
			{
				glBegin(GL_LINE_LOOP);

				glColor3f(255.0f,255.0f,255.0f);
				for(int i=0;i<numberOfVerticies;i++)
				{
				int idxa = verticies[i];
				Point3D *ptA = object->ptsArray[idxa];
				GLdouble wx,wy,wz;
				wx = (double)ptA->getX();
				wy = (double)ptA->getY();
				wz = (double)ptA->getZ();
				glVertex3d(wx,wy,wz);
				}
				glEnd();
			}

			if (view->GetDocument()->showTextures)
			{
			  glDisable(GL_TEXTURE_2D);
			}


	if (view->GetDocument()->showVertexNormals && !getObject()->isN2Version())
	{
	   int m_PlaneNumber = getPlaneNumber();
	   NodePlane *planeNode = getPlaneNode();
	   CPlanes *plane = object->Plans[m_PlaneNumber];
	   if (planeNode)
	   {
	   double cx,cy,cz;
	   cx = cy = cz = 0.0;
	   for(int i=0;i<numberOfVerticies;i++)
	   {
		int idxa = verticies[i];
		Point3D *ptA = object->ptsArray[idxa];
		
	    cx+= (double)ptA->getX();
	    cy+= (double)ptA->getY();
	    cz+= (double)ptA->getZ();
	   }
	   cx /= numberOfVerticies;
	   cy /= numberOfVerticies;
	   cz /= numberOfVerticies;
	   glColor3f(0.0,0.0,1.0);
	   glPointSize(5.0);
	   glBegin(GL_POINTS);
	   
	   glVertex3d(cx,cy,cz);
	   glEnd();
	   glPointSize(1.0);
	   //DrawMyPlane(cx,cy,cz,1.0,0.0,0.0,plane,2.0);
	   }
	}

	}

BOOL NodePolygonFlat81b::Init()
{
   numberOfVerticies = 4;
   verticies = new int[numberOfVerticies];
   verticies[0] = 0;
   verticies[1] = 1;
   verticies[2] = 2;
   verticies[3] = 3;
   return TRUE;
}


void NodePolygonFlat81b::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
			//if (drawn) return;
	        CMy3DOEditorView *view = (CMy3DOEditorView*)doview;


			int red= (color & 0x00FF0000) >> 16;
		    int green = (color & 0x0000FF00) >> 8;
		    int blue=(color & 0x000000FF);

			if (view->GetDocument()->showRandomColors)
			{
				setRandomColor();
			}
			else
			{
				setGLColor((float)(red/256.0),(float)(green/256.0),(float)(blue/256.0));
			}
			
            if (view->GetDocument()->viewType != t_WireFrame) 
				glBegin(GL_POLYGON);
			else 
				glBegin(GL_LINE_LOOP);

			for(int i=0;i<numberOfVerticies;i++)
			{
			 int idxa = verticies[i];
			 if (idxa <=0 || idxa >= object->numPts) continue;
			 Point3D *ptA = object->ptsArray[idxa];
			 GLdouble wx,wy,wz;
	         wx = (double)ptA->getX();
	         wy = (double)ptA->getY();
	         wz = (double)ptA->getZ();
			 glVertex3d(wx,wy,wz);
			}
			glEnd();

			if (view->GetDocument()->viewType == t_PolygonOutlines) 
			{
				glBegin(GL_LINE_LOOP);

				glColor3f(255.0f,255.0f,255.0f);
				for(int i=0;i<numberOfVerticies;i++)
				{
				int idxa = verticies[i];
				Point3D *ptA = object->ptsArray[idxa];
				GLdouble wx,wy,wz;
				wx = (double)ptA->getX();
				wy = (double)ptA->getY();
				wz = (double)ptA->getZ();
				glVertex3d(wx,wy,wz);
				}
				glEnd();
			}
}

BOOL NodePolygonTexture81f::Init()
{
   numberOfVerticies = 4;
   verticies = new int[numberOfVerticies];
   verticies[0] = 0;
   verticies[1] = 1;
   verticies[2] = 2;
   verticies[3] = 3;
   return TRUE;
}

BOOL NodePolygonTexture820::Init()
{
   numberOfVerticies = 4;
   verticies = new int[numberOfVerticies];
   vertexcolors = new int[numberOfVerticies];
   verticies[0] = 0;
   verticies[1] = 1;
   verticies[2] = 2;
   verticies[3] = 3;
   vertexcolors[0] = 0;
   vertexcolors[1] = 0;
   vertexcolors[2] = 0;
   vertexcolors[3] = 0;
   return TRUE;
}

BOOL NodePolygonTexture821::Init()
{
   numberOfVerticies = 4;
   verticies = new int[numberOfVerticies];
   vertexnormals = new int[numberOfVerticies];
   verticies[0] = 0;
   verticies[1] = 1;
   verticies[2] = 2;
   verticies[3] = 3;
   vertexnormals[0] = 0;
   vertexnormals[1] = 0;
   vertexnormals[2] = 0;
   vertexnormals[3] = 0;
   return TRUE;
}

BOOL NodePolygonTexture81f::Edit()
{
	CPolygonEditor *dlg = new CPolygonEditor(this);
	dlg->DoModal();
	if (dlg) delete dlg;
	return TRUE;
}

BOOL NodePolygonFlat81b::Edit()
{
	// numberOfVerticies
	// color
	// verticies
	CPolygonEditor *dlg = new CPolygonEditor(this);
	dlg->DoModal();
	if (dlg) delete dlg;
	return TRUE;
}

BOOL NodeLine815::Edit()
{
	// number v1 v2
	return TRUE;
}

void NodePolygonTexture81f::Draw(C3do* object,Display *display,CView *doview)
   {
	   if (object->ptsArray)
	   {
		   for(int i=0;i<numberOfVerticies-1;i++)
		   {
			   int idxa = verticies[i];
			   int idxb = verticies[i+1];
			   Point3D *pt = object->ptsArray[idxa];
			   Point3D *ptb = object->ptsArray[idxb];
			   double x=pt->getX();
		       double y=pt->getY();
			   display->drawLine(x,y,ptb->getX(),ptb->getY());
		   }
	   }
   }


void NodePolygonFlat81b::Draw(C3do* object,Display *display,CView *doview)
   {
	   if (object->ptsArray)
	   {
		   for(int i=0;i<numberOfVerticies-1;i++)
		   {
			   int idxa = verticies[i];
			   int idxb = verticies[i+1];
			   Point3D *pt = object->ptsArray[idxa];
			   Point3D *ptb = object->ptsArray[idxb];
			   double x=pt->getX();
		       double y=pt->getY();
			   display->drawLine(x,y,ptb->getX(),ptb->getY());
		   }
	   }
   }


void NodePolygon::CalculateABCD()
{
	A = 0.0f;
	B = 0.0f;
	C = 0.0f;
	D = 0.0f;

	C3do *object = getObject();

	if (object->ptsArray)
	{
	 if (numberOfVerticies > 2)
	 {
	  if (verticies[0] < object->numPts && verticies[1] < object->numPts && verticies[2] < object->numPts)
	  {
	  float x1 = object->ptsArray[verticies[0]]->getX();
	  float x2 = object->ptsArray[verticies[1]]->getX();
	  float x3 = object->ptsArray[verticies[2]]->getX();

	  float y1 = object->ptsArray[verticies[0]]->getY();
	  float y2 = object->ptsArray[verticies[1]]->getY();
	  float y3 = object->ptsArray[verticies[2]]->getY();

	  float z1 = object->ptsArray[verticies[0]]->getZ();
	  float z2 = object->ptsArray[verticies[1]]->getZ();
	  float z3 = object->ptsArray[verticies[2]]->getZ();

	  A = y1 *(z2 - z3) + y2 *(z3 - z1) + y3 *(z1 - z2);
      B = z1 *(x2 - x3) + z2 *(x3 - x1) + z3 *(x1 - x2);
      C = x1 *(y2 - y3) + x2 *(y3 - y1) + x3 *(y1 - y2);
      D = x1 *(y2 *z3 - y3 *z2) + x2 *(y3 *z1 - y1 *z3) + x3 *(y1 *z2 - y2 *z1);

	  float m = 1.0f / (float)(sqrt( A * A + B * B + C * C ));

      A = m*A;
      B = m*B;
      C = m*C;
	  D = -1*(m*D);
	  }
	}
	}
}


NodePolygonTexture820::NodePolygonTexture820(int secOffset,Node *parent):NodePolygonTexture81f(secOffset,parent,0x820)
{
	   bitmapid = IDB_POLYGONS_TEXT;
	   textureMip = NULL;
}

NodePolygonTexture820::~NodePolygonTexture820()
{
	   if (vertexcolors) delete[] vertexcolors;
}

void NodePolygonTexture820::Read(FILE *fp)
{
	    zero              = ReadPCDWord(fp);
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		xtexture =  new float[numberOfVerticies];
		ytexture =  new float[numberOfVerticies];
		vertexcolors = new int[numberOfVerticies];

		
		for(int i=0;i<numberOfVerticies;i++)
		{
			xtexture[i] = ReadPCFloat(fp);
			ytexture[i] = ReadPCFloat(fp);
		}
		fread(verticies,numberOfVerticies,sizeof(int),fp);
		fread(vertexcolors,numberOfVerticies,sizeof(int),fp);
}

int NodePolygonTexture820::Write(FILE *fp,int secOffset)
{
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   sendCommandVal(fp,0x820);
	   WritePCDWord(fp,zero);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   for(int i=0;i<numberOfVerticies;i++)
	   {
			WritePCFloat(fp,xtexture[i]);
			WritePCFloat(fp,ytexture[i]);
	   }
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   fwrite(vertexcolors,numberOfVerticies,sizeof(int),fp);
	   return foff;
}

CString NodePolygonTexture820::toString()
{
	   CString buffer;
	   CalculateABCD();
	   buffer.Format("Textured Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}


NodePolygonFlat81c::NodePolygonFlat81c(int secOffset,Node *parent):
NodePolygonFlat81b(secOffset,parent,0x81c)
{
}

BOOL NodePolygonFlat81c::Init()
{
   numberOfVerticies = 4;
   verticies = new int[numberOfVerticies];

   vertexcolors = new int[numberOfVerticies];
   memset(vertexcolors,4,sizeof(int));
   
   verticies[0] = 0;
   verticies[1] = 1;
   verticies[2] = 2;
   verticies[3] = 3;
   return TRUE;
}

NodePolygonFlat81c::~NodePolygonFlat81c()
{
	   if (vertexcolors) delete[] vertexcolors;
}

void NodePolygonFlat81c::Read(FILE *fp)
{
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		vertexcolors = new int[numberOfVerticies];

		fread(verticies,numberOfVerticies,sizeof(int),fp);
		fread(vertexcolors,numberOfVerticies,sizeof(int),fp);
}

int NodePolygonFlat81c::Write(FILE *fp,int secOffset)
{
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   WritePCDWord(fp,0x81c);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   fwrite(vertexcolors,numberOfVerticies,sizeof(int),fp);
	   return foff;
}

CString NodePolygonFlat81c::toString()
{
       CString buffer;
	   CalculateABCD();
	   buffer.Format("Flat Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}


NodePolygonTexture20::NodePolygonTexture20(int secOffset,Node *parent):NodePolygonTexture81f(secOffset,parent,0x20)
{
	   bitmapid = IDB_POLYGONS_TEXT;
	   textureMip = NULL;
}

NodePolygonTexture20::~NodePolygonTexture20()
{
	   if (vertexcolors) delete[] vertexcolors;
}

void NodePolygonTexture20::Read(FILE *fp)
{
	    zero              = ReadPCDWord(fp);
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		xtexture =  new float[numberOfVerticies];
		ytexture =  new float[numberOfVerticies];
		vertexcolors = new int[numberOfVerticies];

		
		for(int i=0;i<numberOfVerticies;i++)
		{
			xtexture[i] = ReadPCFloat(fp);
			ytexture[i] = ReadPCFloat(fp);
		}
		fread(verticies,numberOfVerticies,sizeof(int),fp);
		fread(vertexcolors,numberOfVerticies,sizeof(int),fp);
}

int NodePolygonTexture20::Write(FILE *fp,int secOffset)
{
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   sendCommandVal(fp,0x20);
	   WritePCDWord(fp,zero);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   for(int i=0;i<numberOfVerticies;i++)
	   {
			WritePCFloat(fp,xtexture[i]);
			WritePCFloat(fp,ytexture[i]);
	   }
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   fwrite(vertexcolors,numberOfVerticies,sizeof(int),fp);
	   return foff;
}

CString NodePolygonTexture20::toString()
{
	   CString buffer;
	   CalculateABCD();
	   buffer.Format("Textured Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

NodePolygonFlat81d::NodePolygonFlat81d(int secOffset,Node *parent):
NodePolygonFlat81b(secOffset,parent,0x81d)
{
}

BOOL NodePolygonFlat81d::Init()
{
   numberOfVerticies = 4;
   verticies = new int[numberOfVerticies];

   vertexnormals = new int[numberOfVerticies];
   memset(vertexnormals,4,sizeof(int));

   verticies[0] = 0;
   verticies[1] = 1;
   verticies[2] = 2;
   verticies[3] = 3;
   return TRUE;
}

NodePolygonFlat81d::~NodePolygonFlat81d()
{
	   if (vertexnormals) delete[] vertexnormals;
}

void NodePolygonFlat81d::Read(FILE *fp)
{
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		vertexnormals = new int[numberOfVerticies];

		fread(verticies,numberOfVerticies,sizeof(int),fp);
		fread(vertexnormals,numberOfVerticies,sizeof(int),fp);
}

int NodePolygonFlat81d::Write(FILE *fp,int secOffset)
{
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   WritePCDWord(fp,0x81d);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   fwrite(vertexnormals,numberOfVerticies,sizeof(int),fp);
	   return foff;
}

CString NodePolygonFlat81d::toString()
{
       CString buffer;
	   CalculateABCD();
	   buffer.Format("Flat Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}




NodePolygonN2_FL2::NodePolygonN2_FL2(int secOffset,Node *parent):
NodePolygonTexture81f(secOffset,parent,0x2)
{
   vinfo = NULL;
}

BOOL NodePolygonN2_FL2::Init()
{
   numberOfVerticies = 4;
   verticies = new int[numberOfVerticies];
   verticies[0] = 0;
   verticies[1] = 1;
   verticies[2] = 2;
   verticies[3] = 3;
   return TRUE;
}

NodePolygonN2_FL2::~NodePolygonN2_FL2()
{
   if (mode == 3)
   {
     if (vinfo) delete vinfo;
   }
}

void NodePolygonN2_FL2::Read(FILE *fp)
{
		mode              = ReadPCDWord(fp);
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		//vertexnormals = new int[numberOfVerticies];

		fread(verticies,numberOfVerticies,sizeof(int),fp);
		xtexture=NULL;
		ytexture=NULL;
		if (mode >= 1 && mode <= 3)
		{
		  xtexture= new float[numberOfVerticies];
		  ytexture= new float[numberOfVerticies];
		  for(int i=0;i<numberOfVerticies;i++)
		  {
			  xtexture[i] = (float)ReadPCWord(fp);
			  ytexture[i] = (float)ReadPCWord(fp);
		  }
		  bitmapid = IDB_POLYGONS_TEXT;
		}
		else
		{
			//cerr << "Unknown Polygon Mode " << mode << endl;
			bitmapid = IDB_POLYGONS;
		}
		if (mode == 3)
	   {
		  vinfo = new int[numberOfVerticies];

		  for(int i=0;i<numberOfVerticies;i++)
		  {
			 vinfo[i] = ReadPCDWord(fp);
		  }
	   }
		//fread(vertexnormals,numberOfVerticies,sizeof(int),fp);
}

int NodePolygonN2_FL2::Write(FILE *fp,int secOffset)
{
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   //WritePCDWord(fp,type);
	   sendCommandVal(fp,type);
	   WritePCDWord(fp,mode);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);	   
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);

	   if (mode >=1 && mode <=3)
	   {
	      for(int i=0;i<numberOfVerticies;i++)
		  {
			  WritePCWord(fp,(int)xtexture[i]);
			  WritePCWord(fp,(int)ytexture[i]);
		  }
	   }
	   
	   if (mode == 3)
	   {
		  for(int i=0;i<numberOfVerticies;i++)
		  {
			 WritePCDWord(fp,vinfo[i]);
		  }
	   }
	   return foff;
}

#define TO_METERS(x) ((x)/19685.0f)
#define NORMAL_SCALE 5000.0

void NodePolygonN2_FL2::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	CMy3DOEditorView *view = (CMy3DOEditorView*)doview;

	if (mode != 0)
		NodePolygonTexture81f::DrawGL(hdc,object,doview,currentTexture);
	else {
			
			int red,green,blue;

			if (color >= 0 && color < 255)
			{
	         red    = colorIndex[color].r;
			 green  = colorIndex[color].g;
			 blue   = colorIndex[color].b;

			 if (color != 0 && red==0 && blue==0 && green==0 && color!=31)
			 {
			  CString msg;
			  msg.Format("(%d)",color);
			  //AfxMessageBox(msg);
			  cerr << msg;
			 }
			}
			else
			{
			  red   = (color & 0x00FF0000) >> 16;
		      green = (color & 0x0000FF00) >> 8;
		      blue  = (color & 0x000000FF);
			}

			if (view->GetDocument()->showRandomColors)
			{
				setRandomColor();
			}
			else
			{
				setGLColor((float)(red/256.0),(float)(green/256.0),(float)(blue/256.0));
			}
			
            if (view->GetDocument()->viewType != t_WireFrame) 
				glBegin(GL_POLYGON);
			else 
				glBegin(GL_LINE_LOOP);

			for(int i=0;i<numberOfVerticies;i++)
			{
			 int idxa = verticies[i];
			 Point3D *ptA = object->ptsArray[idxa];
			 GLdouble wx,wy,wz;
	         wx = (double)ptA->getX();
	         wy = (double)ptA->getY();
	         wz = (double)ptA->getZ();
			 glVertex3d(wx,wy,wz);
			}
			glEnd();
			
			if (view->GetDocument()->viewType == t_PolygonOutlines) 
			{
				glBegin(GL_LINE_LOOP);

				glColor3f(255.0f,255.0f,255.0f);
				for(int i=0;i<numberOfVerticies;i++)
				{
				int idxa = verticies[i];
				Point3D *ptA = object->ptsArray[idxa];
				GLdouble wx,wy,wz;
				wx = (double)ptA->getX();
				wy = (double)ptA->getY();
				wz = (double)ptA->getZ();
				glVertex3d(wx,wy,wz);
				}
				glEnd();
			}
	}

	if (view->GetDocument()->showVertexNormals)
	{
	   NodePlane *planeNode = getPlaneNode();
	   if (planeNode)
	   {
	   double cx,cy,cz;
	   cx = cy = cz = 0.0;
	   for(int i=0;i<numberOfVerticies;i++)
	   {
		int idxa = verticies[i];
		Point3D *ptA = object->ptsArray[idxa];
		
	    cx+= (double)ptA->getX();
	    cy+= (double)ptA->getY();
	    cz+= (double)ptA->getZ();
	   }
	   cx /= numberOfVerticies;
	   cy /= numberOfVerticies;
	   cz /= numberOfVerticies;
	   glColor3d(1.0,0.0,1.0);
	   glPointSize(5.0);
	   glBegin(GL_POINTS);
	   double gl1 = TO_METERS(((double)planeNode->number1/NORMAL_SCALE));
	   double gl2 = TO_METERS(((double)planeNode->number2/NORMAL_SCALE));
	   double gl3 = TO_METERS(((double)planeNode->number3/NORMAL_SCALE));
	   glVertex3d(cx,cy,cz);
	   glVertex3d(cx+gl1,cy+gl2,cz+gl3);
	   glEnd();
	   glBegin(GL_LINES);
	   glVertex3d(cx,cy,cz);
	   glVertex3d(cx+gl1,cy+gl2,cz+gl3);
	   glEnd();
	   glPointSize(1.0);
	   }
	}
}

CString NodePolygonN2_FL2::toString()
{
       CString buffer;
	   CalculateABCD();
	   buffer.Format("N2 Textured Polygon (%x) Color=%s Verticies=%d Mode=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,mode,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

NodeLine815::~NodeLine815()
{
	if (v) delete v;
}

void NodeLine815::Read(FILE *fp)
{
	if (object->getVersion() != t_N2)
	{
		v = new int[2];
	    number     = ReadPCDWord(fp);
		v[0]       = ReadPCDWord(fp);
		v[1]       = ReadPCDWord(fp);
		numberofVerticies = 2;
	}
	else
	{
		int zero   = ReadPCDWord(fp);
		number     = ReadPCDWord(fp);
		numberofVerticies     = ReadPCDWord(fp);
		v = new int[numberofVerticies];
		for(int i=0;i<numberofVerticies;i++)
		{
			v[i]     = ReadPCDWord(fp);
		}
	}
}

void NodeLine815::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	//if (drawn) return;
	glColor3f(0.0f,0.0f,0.0f);
	glBegin(GL_LINES);
	for(int i=0;i<numberofVerticies;i++)
	{
		int idxa = v[i];
		Point3D *ptA = object->ptsArray[idxa];
		GLdouble wx,wy,wz;
	    wx = (double)ptA->getX();
	    wy = (double)ptA->getY();
	    wz = (double)ptA->getZ();
		glVertex3d(wx,wy,wz);
	}
	glEnd();
}


NodePolygonTexture821::NodePolygonTexture821(int secOffset,Node *parent):NodePolygonTexture81f(secOffset,parent,0x821)
{
	   bitmapid = IDB_POLYGONS_TEXT;
	   textureMip = NULL;
}

NodePolygonTexture821::~NodePolygonTexture821()
{
	   if (vertexnormals) delete[] vertexnormals;
}

void NodePolygonFlat81b::Read(FILE *fp)
{
		//C3do *obj         = getObject();
		//obj->trackBufferPos    = ftell(fp);
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];

		fread(verticies,numberOfVerticies,sizeof(int),fp);
		//fseek(fp,obj->trackBufferPos,SEEK_SET);
}

void NodePolygonTexture821::Read(FILE *fp)
{
		C3do *obj         = getObject();
		//obj->trackBufferPos    = ftell(fp);
	    zero              = ReadPCDWord(fp);
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		xtexture =  new float[numberOfVerticies];
		ytexture =  new float[numberOfVerticies];
		vertexnormals = new int[numberOfVerticies];

		
		for(int i=0;i<numberOfVerticies;i++)
		{
			xtexture[i] = ReadPCFloat(fp);
			ytexture[i] = ReadPCFloat(fp);
		}
		fread(verticies,numberOfVerticies,sizeof(int),fp);
		fread(vertexnormals,numberOfVerticies,sizeof(int),fp);
		//fseek(fp,obj->trackBufferPos,SEEK_SET);
}

int NodePolygonTexture821::Write(FILE *fp,int secOffset)
{
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   sendCommandVal(fp,0x821);
	   WritePCDWord(fp,zero);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   for(int i=0;i<numberOfVerticies;i++)
	   {
			WritePCFloat(fp,xtexture[i]);
			WritePCFloat(fp,ytexture[i]);
	   }
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   fwrite(vertexnormals,numberOfVerticies,sizeof(int),fp);
	   return foff;
}

CString NodePolygonTexture821::toString()
{
	   CString buffer;
	   CalculateABCD();
	   buffer.Format("Textured Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

NodePolygonTexture21::NodePolygonTexture21(int secOffset,Node *parent):NodePolygonTexture81f(secOffset,parent,0x821)
{
	   bitmapid = IDB_POLYGONS_TEXT;
	   textureMip = NULL;
}

NodePolygonTexture21::~NodePolygonTexture21()
{
	   if (vertexnormals) delete[] vertexnormals;
}

void NodePolygonTexture21::Read(FILE *fp)
{
	    zero              = ReadPCDWord(fp);
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		xtexture =  new float[numberOfVerticies];
		ytexture =  new float[numberOfVerticies];
		vertexnormals = new int[numberOfVerticies];

		
		for(int i=0;i<numberOfVerticies;i++)
		{
			xtexture[i] = ReadPCFloat(fp);
			ytexture[i] = ReadPCFloat(fp);
		}
		fread(verticies,numberOfVerticies,sizeof(int),fp);
		fread(vertexnormals,numberOfVerticies,sizeof(int),fp);
}

int NodePolygonTexture21::Write(FILE *fp,int secOffset)
{
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   sendCommandVal(fp,0x21);
	   WritePCDWord(fp,zero);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   for(int i=0;i<numberOfVerticies;i++)
	   {
			WritePCFloat(fp,xtexture[i]);
			WritePCFloat(fp,ytexture[i]);
	   }
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   fwrite(vertexnormals,numberOfVerticies,sizeof(int),fp);
	   return foff;
}

CString NodePolygonTexture21::toString()
{
	   CString buffer;
	   CalculateABCD();
	   buffer.Format("Textured Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

NodePolygonFlat81e::NodePolygonFlat81e(int secOffset,Node *parent):
NodePolygonFlat81b(secOffset,parent,0x81e)
{
}

BOOL NodePolygonFlat81e::Init()
{
   numberOfVerticies = 4;
   verticies = new int[numberOfVerticies];

   vertexcolors = new int[numberOfVerticies];
   memset(vertexcolors,4,sizeof(int));
   vertexnormals = new int[numberOfVerticies];
   memset(vertexnormals,4,sizeof(int));

   verticies[0] = 0;
   verticies[1] = 1;
   verticies[2] = 2;
   verticies[3] = 3;
   return TRUE;
}

NodePolygonFlat81e::~NodePolygonFlat81e()
{
	   if (vertexnormals) delete[] vertexnormals;
	   if (vertexcolors)  delete[] vertexcolors;
}

void NodePolygonFlat81e::Read(FILE *fp)
{
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		vertexnormals = new int[numberOfVerticies];
		vertexcolors = new int[numberOfVerticies];

		fread(verticies,numberOfVerticies,sizeof(int),fp);
		fread(vertexcolors,numberOfVerticies,sizeof(int),fp);
		fread(vertexnormals,numberOfVerticies,sizeof(int),fp);
}

int NodePolygonFlat81e::Write(FILE *fp,int secOffset)
{
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   WritePCDWord(fp,0x81e);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   fwrite(vertexcolors,numberOfVerticies,sizeof(int),fp);
	   fwrite(vertexnormals,numberOfVerticies,sizeof(int),fp);
	   return foff;
}

CString NodePolygonFlat81e::toString()
{
       CString buffer;
	   CalculateABCD();
	   buffer.Format("Flat Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

NodePolygonTexture1f::NodePolygonTexture1f(int secOffset,Node *parent):NodePolygonTexture81f(secOffset,parent,0x1f)
{
	   bitmapid = IDB_POLYGONS_TEXT;
	   textureMip = NULL;
}

NodePolygonTexture1f::~NodePolygonTexture1f()
{
}

void NodePolygonTexture1f::Read(FILE *fp)
{
	    zero              = ReadPCDWord(fp);
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		xtexture =  new float[numberOfVerticies];
		ytexture =  new float[numberOfVerticies];
		
		
		for(int i=0;i<numberOfVerticies;i++)
		{
			xtexture[i] = ReadPCFloat(fp);
			ytexture[i] = ReadPCFloat(fp);
		}
		fread(verticies,numberOfVerticies,sizeof(int),fp);
}

int NodePolygonTexture1f::Write(FILE *fp,int secOffset)
{
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   sendCommandVal(fp,0x1f);
	   WritePCDWord(fp,zero);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   for(int i=0;i<numberOfVerticies;i++)
	   {
			WritePCFloat(fp,xtexture[i]);
			WritePCFloat(fp,ytexture[i]);
	   }
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   return foff;
}

CString NodePolygonTexture1f::toString()
{
	   CString buffer;
	   CalculateABCD();
	   buffer.Format("Textured Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

NodePolygonFlat1c::NodePolygonFlat1c(int secOffset,Node *parent):
NodePolygonFlat81b(secOffset,parent,0x1c)
{
}

BOOL NodePolygonFlat1c::Init()
{
   numberOfVerticies = 4;
   verticies = new int[numberOfVerticies];

   vertexcolors = new int[numberOfVerticies];
   memset(vertexcolors,4,sizeof(int));
   
   verticies[0] = 0;
   verticies[1] = 1;
   verticies[2] = 2;
   verticies[3] = 3;
   return TRUE;
}

NodePolygonFlat1c::~NodePolygonFlat1c()
{
	   if (vertexcolors) delete[] vertexcolors;
}

void NodePolygonFlat1c::Read(FILE *fp)
{
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		vertexcolors = new int[numberOfVerticies];

		fread(verticies,numberOfVerticies,sizeof(int),fp);
		fread(vertexcolors,numberOfVerticies,sizeof(int),fp);
}

int NodePolygonFlat1c::Write(FILE *fp,int secOffset)
{
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   WritePCDWord(fp,0x1c);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   fwrite(vertexcolors,numberOfVerticies,sizeof(int),fp);
	   return foff;
}

CString NodePolygonFlat1c::toString()
{
       CString buffer;
	   CalculateABCD();
	   buffer.Format("Flat Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

int NodePolygonOneICR2::Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   WritePCDWord(fp,0x80000001);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   // not quite right just yet!
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   return foff;
   }

void NodePolygonOneICR2::Read(FILE *fp)
{
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp)+1;
		verticies = new int[numberOfVerticies];

		fread(verticies,numberOfVerticies,sizeof(int),fp);

		int offset = getObject()->ICR2DataSectionStart;

		for(int i=0;i<numberOfVerticies;i++)
		{
			 int vidx = verticies[i]; 
			 verticies[i] = getObject()->GetVertexPoint(fp,vidx,FALSE);
			 //int idx = (vidx)/20;
			 //verticies[i] = idx;
		}
}

void NodePolygonOneICR2::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
			//if (drawn) return;
	        CMy3DOEditorView *view = (CMy3DOEditorView*)doview;

			int red,green,blue;

			if (color >= 0 && color < 255)
			{
	         red    = colorIndex[color].r;
			 green  = colorIndex[color].g;
			 blue   = colorIndex[color].b;
			}
			else
			{
			 red   = (color & 0x00FF0000) >> 16;
		     green = (color & 0x0000FF00) >> 8;
		     blue  = (color & 0x000000FF);
			}

			//int red= (color & 0x00FF0000) >> 16;
		    //int green = (color & 0x0000FF00) >> 8;
		    //int blue=(color & 0x000000FF);

			if (view->GetDocument()->showRandomColors)
			{
				setRandomColor();
			}
			else
			{
				setGLColor((float)(red/256.0),(float)(green/256.0),(float)(blue/256.0));
			}
			//glColor3f((float)(red/256.0),(float)(green/256.0),(float)(blue/256.0));

            if (view->GetDocument()->viewType != t_WireFrame) 
				glBegin(GL_POLYGON);
			else 
				glBegin(GL_LINE_LOOP);

			for(int i=0;i<numberOfVerticies;i++)
			{
			 int idxa = verticies[i];
			 if (idxa < object->numPts)
			 {
			  Point3D *ptA = object->ptsArray[idxa];
			  GLdouble wx,wy,wz;
	          wx = (double)ptA->getX();
	          wy = (double)ptA->getY();
	          wz = (double)ptA->getZ();
			  glVertex3d(wx,wy,wz);
			 }
			}
			glEnd();

			if (view->GetDocument()->viewType == t_PolygonOutlines) 
			{
				glBegin(GL_LINE_LOOP);

				glColor3f(255.0f,255.0f,255.0f);
				for(int i=0;i<numberOfVerticies;i++)
				{
				int idxa = verticies[i];
				Point3D *ptA = object->ptsArray[idxa];
				GLdouble wx,wy,wz;
				wx = (double)ptA->getX();
				wy = (double)ptA->getY();
				wz = (double)ptA->getZ();
				glVertex3d(wx,wy,wz);
				}
				glEnd();
			}
}

CString NodePolygonOneICR2::toString()
{
       CString buffer;
	   CalculateABCD();
	   buffer.Format("Flat Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   for(int i=0;i<numberOfVerticies;i++)
	   {
		   CString msg;
		   msg.Format("[%d]",verticies[i]);
		   buffer += msg;
	   }
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

int NodePolygonTwoICR2::Write(FILE *fp,int secOffset)
   {
	   sectionOffset = secOffset;
	   int foff = ftell(fp);
	   WritePCDWord(fp,0x80000002);
	   WritePCDWord(fp,color);
	   WritePCDWord(fp,numberOfVerticies);
	   // not quite right just yet!
	   fwrite(verticies,numberOfVerticies,sizeof(int),fp);
	   return foff;
   }

void NodePolygonTwoICR2::Read(FILE *fp)
{
		int textureFlag       = ReadPCDWord(fp);
	    color             = ReadPCDWord(fp);		
		numberOfVerticies = ReadPCDWord(fp)+1;
		verticies = new int[numberOfVerticies];

		fread(verticies,numberOfVerticies,sizeof(int),fp);

		for(int i=0;i<numberOfVerticies;i++)
		{
			 int vidx = verticies[i]; 
			 //int idx = (vidx)/20;
			 verticies[i] = getObject()->GetVertexPoint(fp,vidx,TRUE);
			 //verticies[i] = idx;
		}
}

void NodePolygonTwoICR2::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
			//if (drawn) return;
	        CMy3DOEditorView *view = (CMy3DOEditorView*)doview;

			int red,green,blue;

			if (color >= 0 && color < 255)
			{
	         red    = colorIndex[color].r;
			 green  = colorIndex[color].g;
			 blue   = colorIndex[color].b;
			}
			else
			{
			 red   = (color & 0x00FF0000) >> 16;
		     green = (color & 0x0000FF00) >> 8;
		     blue  = (color & 0x000000FF);
			}

			//int red= (color & 0x00FF0000) >> 16;
		    //int green = (color & 0x0000FF00) >> 8;
		    //int blue=(color & 0x000000FF);

			if (view->GetDocument()->showRandomColors)
			{
				setRandomColor();
			}
			else
			{
				setGLColor((float)(red/256.0),(float)(green/256.0),(float)(blue/256.0));
			}
			//glColor3f((float)(red/256.0),(float)(green/256.0),(float)(blue/256.0));

            if (view->GetDocument()->viewType != t_WireFrame) 
				glBegin(GL_POLYGON);
			else 
				glBegin(GL_LINE_LOOP);

			for(int i=0;i<numberOfVerticies;i++)
			{
			 int idxa = verticies[i];
			 if (idxa < object->numPts)
			 {
			  Point3D *ptA = object->ptsArray[idxa];
			  GLdouble wx,wy,wz;
	          wx = (double)ptA->getX();
	          wy = (double)ptA->getY();
	          wz = (double)ptA->getZ();
			  glVertex3d(wx,wy,wz);
			 }
			}
			glEnd();

			if (view->GetDocument()->viewType == t_PolygonOutlines) 
			{
				glBegin(GL_LINE_LOOP);

				glColor3f(255.0f,255.0f,255.0f);
				for(int i=0;i<numberOfVerticies;i++)
				{
				int idxa = verticies[i];
				Point3D *ptA = object->ptsArray[idxa];
				GLdouble wx,wy,wz;
				wx = (double)ptA->getX();
				wy = (double)ptA->getY();
				wz = (double)ptA->getZ();
				glVertex3d(wx,wy,wz);
				}
				glEnd();
			}
}

CString NodePolygonTwoICR2::toString()
{
       CString buffer;
	   CalculateABCD();
	   buffer.Format("Textured Polygon (%x) Color=%s Colored Verticies=%d (ABCD=%f,%f,%f,%f)",type,ToRgb(color),numberOfVerticies,A,B,C,D);
	   for(int i=0;i<numberOfVerticies;i++)
	   {
		   CString msg;
		   msg.Format("[%d]",verticies[i]);
		   buffer += msg;
	   }
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

void NodeNineteen::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	CMy3DOEditorView *view = (CMy3DOEditorView*)doview;

	glColor3d(1.0,0.0,1.0);
	if (view->GetDocument()->showRooms)
	{
	Point3D *ptA;
	glBegin(GL_LINE_LOOP);
    ptA= object->ptsArray[info.v0];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v1];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v2];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v3];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	glEnd();

	glBegin(GL_LINE_LOOP);
    ptA= object->ptsArray[info.v4];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v5];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v6];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v7];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	glEnd();

	glBegin(GL_LINE_LOOP);
    ptA= object->ptsArray[info.v0];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v1];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v5];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v4];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	glEnd();

	glBegin(GL_LINE_LOOP);
    ptA= object->ptsArray[info.v2];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v3];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v7];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v6];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	glEnd();

	glBegin(GL_LINE_LOOP);
    ptA= object->ptsArray[info.v0];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v3];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v7];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v4];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	glEnd();

	glBegin(GL_LINE_LOOP);
    ptA= object->ptsArray[info.v1];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v2];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v6];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	ptA= object->ptsArray[info.v5];
	glVertex3d((double)ptA->getX(),(double)ptA->getY(),(double)ptA->getZ());
	glEnd();
	}

	Node::DrawGL(hdc,object,doview,currentTexture);
}

BOOL NodeNineteen::Edit()
{
	CNodeNineteenEdit *dlg = new CNodeNineteenEdit();

	dlg->m_V0 = info.v0;
	dlg->m_V1 = info.v1;
	dlg->m_V2 = info.v2;
	dlg->m_V3 = info.v3;
	dlg->m_V4 = info.v4;
	dlg->m_V5 = info.v5;
	dlg->m_V6 = info.v6;
	dlg->m_V7 = info.v7;

	int result = dlg->DoModal();

	if (result == IDOK)
	{
		info.v0 = dlg->m_V0;
		info.v1 = dlg->m_V1;
		info.v2 = dlg->m_V2;
		info.v3 = dlg->m_V3;
		info.v4 = dlg->m_V4;
		info.v5 = dlg->m_V5;
		info.v6 = dlg->m_V6;
		info.v7 = dlg->m_V7;
		return TRUE;
	}
	return FALSE;
}

void NodePolygonTexture821::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	NodePolygonTexture81f::DrawGL(hdc,object,doview,currentTexture);

	CMy3DOEditorView *view = (CMy3DOEditorView*)doview;
	   
	if (view->GetDocument()->showVertexNormals)
	{
	 for(int i=0;i<numberOfVerticies;i++)
	 {
			 int idxa = verticies[i];
			 Point3D *ptA = object->ptsArray[idxa];
			 GLdouble wx,wy,wz;
			 wx = (double)ptA->getX();
	         wy = (double)ptA->getY();
	         wz = (double)ptA->getZ();

			 glColor3d(0.0,0.0,0.0);
			 glBegin(GL_LINES);

			 int nidx = vertexnormals[i];
			 CNormals *norm = object->Normals[nidx];
			 double A = view->GetDocument()->normalScale*norm->A.data;
			 double B = view->GetDocument()->normalScale*norm->B.data;
			 double C = view->GetDocument()->normalScale*norm->C.data;
			 double D = view->GetDocument()->normalScale*norm->D.data;

			 glVertex3d(wx,wy,wz);
			 glVertex3d(wx+B,wy+C,wz+D);
			 //glVertex3d(A*-D, B*-D, C*-D);
			 //glVertex3d((A*-D)+A,(B*-D)+B,(C*-D)+C); 
			 glEnd();
	 }
	}
}

void NodePolygonFlat81e::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	NodePolygonFlat81b::DrawGL(hdc,object,doview,currentTexture);

	CMy3DOEditorView *view = (CMy3DOEditorView*)doview;
	   
	if (view->GetDocument()->showVertexNormals)
	{
	 for(int i=0;i<numberOfVerticies;i++)
	 {
			 int idxa = verticies[i];
			 Point3D *ptA = object->ptsArray[idxa];
			 GLdouble wx,wy,wz;
			 wx = (double)ptA->getX();
	         wy = (double)ptA->getY();
	         wz = (double)ptA->getZ();

			 glColor3d(0.0,0.0,0.0);
			 glBegin(GL_LINES);

			 int nidx = vertexnormals[i];
			 CNormals *norm = object->Normals[nidx];
			 double A = view->GetDocument()->normalScale*norm->A.data;
			 double B = view->GetDocument()->normalScale*norm->B.data;
			 double C = view->GetDocument()->normalScale*norm->C.data;
			 double D = view->GetDocument()->normalScale*norm->D.data;

			 glVertex3d(wx,wy,wz);
			 glVertex3d(wx+B,wy+C,wz+D); 
			 glEnd();
	 }
	}
}

void NodePolygonFlat81d::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	NodePolygonFlat81b::DrawGL(hdc,object,doview,currentTexture);

	CMy3DOEditorView *view = (CMy3DOEditorView*)doview;
	   
	if (view->GetDocument()->showVertexNormals)
	{
	 for(int i=0;i<numberOfVerticies;i++)
	 {
			 int idxa = verticies[i];
			 Point3D *ptA = object->ptsArray[idxa];
			 GLdouble wx,wy,wz;
			 wx = (double)ptA->getX();
	         wy = (double)ptA->getY();
	         wz = (double)ptA->getZ();

			 glColor3d(0.0,0.0,0.0);
			 glBegin(GL_LINES);

			 int nidx = vertexnormals[i];
			 CNormals *norm = object->Normals[nidx];
			 double A = view->GetDocument()->normalScale*norm->A.data;
			 double B = view->GetDocument()->normalScale*norm->B.data;
			 double C = view->GetDocument()->normalScale*norm->C.data;
			 double D = view->GetDocument()->normalScale*norm->D.data;

			 glVertex3d(wx,wy,wz);
			 glVertex3d(wx+B,wy+C,wz+D); 
			 glEnd();
	 }
	}
}

void NodePolygonTexture81f::Read(FILE *fp)
{
	    //C3do *obj         = getObject();
		//obj->trackBufferPos = ftell(fp);
	    zero              = ReadPCDWord(fp);
	    color             = ReadPCDWord(fp);
		numberOfVerticies = ReadPCDWord(fp);
		verticies = new int[numberOfVerticies];
		xtexture =  new float[numberOfVerticies];
		ytexture =  new float[numberOfVerticies];

		
		for(int i=0;i<numberOfVerticies;i++)
		{
			xtexture[i] = ReadPCFloat(fp);
			ytexture[i] = ReadPCFloat(fp);
		}
		fread(verticies,numberOfVerticies,sizeof(int),fp);
		//fseek(fp,obj->trackBufferPos,SEEK_SET);
}

void NodeFour::Read(FILE *fp)
{
	    numberOfChildren        = ReadPCDWord(fp);
		if (numberOfChildren > 1000)
		{
			cerr << "Possibly too many children in this section" << endl;
			numberOfChildren = 0;
		}
	    children = (Node**)malloc(numberOfChildren*sizeof(Node*));
		
		int toff = ftell(fp);	   	   
		//int toff = object->trackBufferPos;
		for(int i=0;i<numberOfChildren;i++)
		{
		 int child_offset  = ReadPCDWord(fp);
		 
		 if (child_offset && child_offset!=-1)
		 {
		 int ltoff = ftell(fp);
		 //int ltoff = object->trackBufferPos;
	     fseek(fp,getOffset()+child_offset,SEEK_SET);
		 //object->trackBufferPos =getOffset()+child_offset; 
	     children[i]       = getChildType(sectionOffset,this,fp,object);
		 //if (children[i]) children[i]->Read(fp);
		 fseek(fp,ltoff,SEEK_SET);
		 //object->trackBufferPos = ltoff;
		 }
		 else
		 {
			 children[i] = new EmptyNode(sectionOffset,this);
		 }
		}
		fseek(fp,toff,SEEK_SET);
		//object->trackBufferPos = toff;
}


void NodeNineteen::Read(FILE *fp)
{
	    numberOfChildren = 1;
		fread(&info,1,sizeof(t_FlavorNineteen),fp);		
	    children = (Node**)malloc(numberOfChildren*sizeof(Node*));
		
		int toff = ftell(fp);
		//int toff = object->trackBufferPos;
	   	if (info.offset1 != -1)
		{
		 fseek(fp,getOffset()+info.offset1,SEEK_SET);
		 //object->trackBufferPos = getOffset()+info.offset1;
	     children[0]       = getChildType(sectionOffset,this,fp,object);
		 //if (children[0]) children[0]->Read(fp);
		}
		else
		{
		 children[0] = new EmptyNode(sectionOffset,this);
		}

		fseek(fp,toff,SEEK_SET);
		//object->trackBufferPos = toff;
}

void NodeSixteen::Read(FILE *fp)
   {
	    numberOfChildren = 1;
		fread(&info,1,sizeof(t_FlavorSixteen),fp);		
	    children = (Node**)malloc(numberOfChildren*sizeof(Node*));
		
		int toff = ftell(fp);
		//int toff = object->trackBufferPos;
	   	if (info.offset1 != -1)
		{
		 fseek(fp,getOffset()+info.offset1,SEEK_SET);
		 //object->trackBufferPos = getOffset()+info.offset1;
	     children[0]       = getChildType(sectionOffset,this,fp,object);
		 //if (children[0]) children[0]->Read(fp);
		}
		else
		{
		 children[0] = new EmptyNode(sectionOffset,this);
		}

		fseek(fp,toff,SEEK_SET);
		//object->trackBufferPos = toff;

   }

void NodeTen::Read(FILE *fp)	   
{
	    number        = ReadPCDWord(fp);
		entries       = new DataValue[number];
		for(int i=0;i<number;i++)
		{
            entries[i].data = (float)ReadPCDWord(fp);
		}
}
