
#include "stdafx.h"
#include "Node.h"

#include "DatView.h"
#include "3DOEditorView.h"
#include "CSrb.h"

#include "EditSrb.h"

#include <gl\glaux.h>
#include <gl\gl.h>
#include <gl\glu.h>

void setGLColor(float r,float g,float b);

NodeSrb::~NodeSrb()
{
	   if (textureSrb) {
		   //delete textureSrb;
		   //textureSrb = NULL;
	   }
}

int  NodeSrb::LoadSrb()
{
	   int textureidx = -1;

	   if (string_offset!=-1)
	   {
		C3do *objView = getObject();	   
		textureidx   = objView->findStrNameId(string_offset);
		CString name = objView->findStrName(string_offset);
		if (textureSrb==NULL)
		{	
			 // you haven't loaded it previously
			 if((CMip*)objView->datFileParent)
			 {			    
				 // we can get it from the dat file
			     textureSrb = (CSrb*)objView->datFileParent->FindSrbImageFile(name);
			 }

			 if (textureSrb==NULL)
			 {
					// oh dear we still don't have it how about locally on the disk
				    CString FileName;
					FileName.Format("%s.srb",name);
					textureSrb = new CSrb();
				    BOOL loadedMipLocal = textureSrb->Read(FileName);
				    if (!loadedMipLocal)
				     {
					  //stil could not find the mip file
					  cerr << "Failed to Find Image File (either in track or locally)" << name << ".mip" << endl;
				     }
			 }
		}
	   }
	   return textureidx;
}

CString NodeSrb::toString()
{
       CString buffer;
	   CString name = getObject()->findStrName(string_offset);
	   buffer.Format("(Srb) Flavor=%d Children=%d Vertex Number=%d Color=%x String=%d(%s) Scale=%f unk=%d?",type,numberOfChildren,number,color,string_offset,name,scale,child_offset);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

HTREEITEM NodeSrb::LoadTree(HTREEITEM branch,CObjectTree *tree)
{
	   HTREEITEM treeParent = tree->insertInfoNode(branch, (LPSTR)(LPCSTR)toString(),TO_ID(bitmapid));
	   tree->m_ObjectTree.SetItemData(treeParent,(DWORD)this);


	   int textureidx = -1;

	   if (string_offset!=-1)
	   {
		C3do *objView = getObject();	   
		textureidx   = objView->findStrNameId(string_offset);
		CString name = objView->findStrName(string_offset);
		if (textureSrb==NULL)
		{	
			 // you haven't loaded it previously
			 if((CMip*)objView->datFileParent)
			 {			    
				 // we can get it from the dat file
			     textureSrb = (CSrb*)objView->datFileParent->FindSrbImageFile(name);
			 }

			 if (textureSrb==NULL)
			 {
					// oh dear we still don't have it how about locally on the disk
				    CString FileName;
					FileName.Format("%s.srb",name);
					textureSrb = new CSrb();
				    BOOL loadedMipLocal = textureSrb->Read(FileName);
				    if (!loadedMipLocal)
				     {
					  //stil could not find the mip file
					  cerr << "Failed to Find Image File (either in track or locally)" << name << ".mip" << endl;
				     }
			 }
		}
	   }

	   if (textureSrb)
	   {
	    for(int i=0;i<6;i++)
	    {
		 CString buffer;
		 buffer.Format("SXY[%d]=%f",i,textureSrb->SXY[i]);
		 tree->insertInfoNode(treeParent, (LPSTR)(LPCSTR) buffer,TO_ID(bitmapid));
	    }
	   }
	   return treeParent;
 }


void NodeSrb::Draw(C3do* object,Display *display,CView *doview)
{
	//DatView *view = (DatView*)doview;

	//if (!view->showObjects) return;

	//Node3doObjectPosition *position = (Node3doObjectPosition*)getParent();

	//display->drawBitmap(IDB_SRB,position->getX(),position->getY());

	//CString name = object->findStrName(string_offset);
	//display->drawText(position->getX(),position->getY(),name);
}

void NodeSrb::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	CMy3DOEditorView *view = (CMy3DOEditorView*)doview;
	CDC *pDC = CDC::FromHandle(hdc);

	BOOL isN2 = (getObject()->getVersion() == t_N2) ? TRUE : FALSE;
	if (isN2) return;

	int textureidx = LoadSrb();

	if (view->GetDocument()->showTextures)
	{			   
		if (textureidx != -1 && textureSrb)
		{
			textureSrb->GetFile();
			   
			if(textureSrb->cmip && textureSrb->cmip->getBitmap())
			{	     
			   BMP  *bmpA  = textureSrb->cmip->getBitmap();
			   int width  = bmpA->ImageWidth();
			   int height = bmpA->ImageHeight();
			   if (textureSrb->cmip->bits==NULL)
			   {
				 textureSrb->cmip->bitmapData.CreateDIB(textureSrb->cmip->cbmp);
				 textureSrb->cmip->output.LoadBitmap(IDB_TESTTEXTURE_128);
			     Mak256SquareImage(textureSrb->cmip->cbmp,textureSrb->cmip->output,width,height,128,128,128,128);
			   }
			   //file->valid = TRUE;
			   textureSrb->cmip->bits = TextureMap(pDC,textureSrb->cmip->output,128,128,textureSrb->cmip->bits);
			   glEnable(GL_TEXTURE_2D);
			   }			  
			   else
			   {
				// cannot find the bitmap
			    glDisable(GL_TEXTURE_2D);
			   }			 
			 }
		}
		else
		{
			 glDisable(GL_TEXTURE_2D);			 
		}

		int red   = (color & 0x00FF0000) >> 16;
		int green = (color & 0x0000FF00) >> 8;
		int blue  =(color & 0x000000FF);

		if (!view->GetDocument()->showTextures)
		{
	      red = green = blue = 0;
		}


		if (view->GetDocument()->showSrbs)
		{
			setGLColor((float)(red/256.0),(float)(green/256.0),(float)(blue/256.0));
			//glColor3f((float)(red/256.0),(float)(green/256.0),(float)(blue/256.0));

            if (view->GetDocument()->viewType != t_WireFrame) 
				glBegin(GL_POLYGON);
			else 
				glBegin(GL_LINE_LOOP);

		GLdouble xpos,ypos,zpos;

		Point3D *pt = object->ptsArray[number];
		xpos = pt->getX();
		ypos = pt->getY();
		zpos = pt->getZ();

		float width = 16.0f/32.0f;
		float height = 16.0f/32.0f;
		if (textureSrb!=NULL)
		{
			width = textureSrb->SXY[2]/32.0f;
			height = textureSrb->SXY[3]/32.0f;
		}

		 
			glTexCoord2f(0.0f,0.0f);
			glVertex3f((float)(xpos-width/2.0f),(float)ypos,(float)zpos);
			glTexCoord2f(1.0f,0.0f);
			glVertex3f((float)(xpos+width/2.0f),(float)ypos,(float)zpos);
			glTexCoord2f(1.0f,1.0f);
			glVertex3f((float)(xpos+width/2.0f),(float)ypos,(float)(zpos+height));
			glTexCoord2f(0.0f,1.0f);
			glVertex3f((float)(xpos-width/2.0f),(float)ypos,(float)(zpos+height));
			glEnd();
			glDisable(GL_TEXTURE_2D);
		}
}


BOOL NodeSrb::Edit()
{
	CEditSrb *dlg = new CEditSrb();

	dlg->m_String = string_offset;
	dlg->m_Vertex = number;
	dlg->m_Scale  = scale;
	dlg->m_Unk    =child_offset;
	dlg->m_Alpha= (color & 0xFF000000) >> 24;
	dlg->m_Red= (color & 0x00FF0000) >> 16;
	dlg->m_Green = (color & 0x0000FF00) >> 8;
	dlg->m_Blue=(color & 0x000000FF);
	
	int result = dlg->DoModal();

	if (result == IDOK)
	{
		string_offset = dlg->m_String;
		number = dlg->m_Vertex;
		scale = dlg->m_Scale;
		child_offset = dlg->m_Unk;
		color = (dlg->m_Alpha << 24) + ((dlg->m_Red << 16) + (dlg->m_Green <<8) + dlg->m_Blue);
	}

	if (dlg) delete dlg;
	return TRUE;
}

BOOL NodeA03::Edit()
{
	CEditSrb *dlg = new CEditSrb();

	dlg->m_String = string_offset;
	dlg->m_Vertex = number;
	dlg->m_Scale  = scale;
	dlg->m_Unk    =child_offset;
	dlg->m_Alpha= (color & 0xFF000000) >> 24;
	dlg->m_Red= (color & 0x00FF0000) >> 16;
	dlg->m_Green = (color & 0x0000FF00) >> 8;
	dlg->m_Blue=(color & 0x000000FF);
	
	int result = dlg->DoModal();

	if (result == IDOK)
	{
		string_offset = dlg->m_String;
		number = dlg->m_Vertex;
		scale = dlg->m_Scale;
		child_offset = dlg->m_Unk;
		color = (dlg->m_Alpha << 24) + ((dlg->m_Red << 16) + (dlg->m_Green <<8) + dlg->m_Blue);
	}

	if (dlg) delete dlg;
	return TRUE;
}

CString NodeA03::toString()
{
       CString buffer;
	   CString name = getObject()->findStrName(string_offset);
	   buffer.Format("(A03) Flavor=%d Children=%d Vertex Number=%d Color=%x String=%d(%s) Scale=%f unk=%d?",type,numberOfChildren,number,color,string_offset,name,scale,child_offset);
	   {
		  CString msg;	  
		  msg.Format("[%x] %s",nodeOffset,buffer);
		  return msg;
	   }
	   return buffer;
}

void NodeA03::DrawGL(HDC hdc,C3do* object,CView *doview,int currentTexture)
{
	CMy3DOEditorView *view = (CMy3DOEditorView*)doview;
	CDC *pDC = CDC::FromHandle(hdc);

	
		int red   = (color & 0x00FF0000) >> 16;
		int green = (color & 0x0000FF00) >> 8;
		int blue  =(color & 0x000000FF);

		if (!view->GetDocument()->showTextures)
		{
	      red = green = blue = 0;
		}


		glPointSize(5.0f);
		glBegin(GL_POINTS);
		GLdouble xpos,ypos,zpos;

		Point3D *pt = object->ptsArray[number];
		xpos = pt->getX();
		ypos = pt->getY();
		zpos = pt->getZ();


		 
		glVertex3f((float)xpos,(float)ypos,(float)zpos);
		glEnd();
		glPointSize(1.0f);
}

void NodeSrb::Read(FILE *fp)
{
	    number        = ReadPCDWord(fp);
	    color         = ReadPCDWord(fp);
	    string_offset = ReadPCDWord(fp);
	    scale         = ReadPCFloat(fp);
	    child_offset  = ReadPCDWord(fp);

		LoadSrb();
}

void NodeA03::Read(FILE *fp)
   {
	    number        = ReadPCDWord(fp);
	    color         = ReadPCDWord(fp);
	    string_offset = ReadPCDWord(fp);
	    scale         = ReadPCFloat(fp);
	    child_offset  = ReadPCDWord(fp);

		//LoadSrb();
   }
