// CarTree.cpp : implementation file
//

#include "stdafx.h"
#include "3DOEditor.h"
#include "ObjectTree.h"
#include "3DOEditorDoc.h"
#include "Observer.h"
#include "fstream.h"
#include "NewNode.h"
#include "EditOneVar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define PROFILE(x,y,b) AfxGetApp()->GetProfileInt(CString(#y), CString(#x),b);
#define WR_PROFILE(x,y,b) AfxGetApp()->WriteProfileInt(CString(#y), #x,b);

extern Node * getChildNode(int cmdval,int sectionOffset,Node *parent,FILE *fp,C3do *obj);
Node *getNewNodeType(int off,Node *par);

extern void totals();

/////////////////////////////////////////////////////////////////////////////
// CObjectTree 

CObjectTree ::CObjectTree ()
{
	clipBoardNode = NULL;
}

CObjectTree ::~CObjectTree ()
{
}


void CObjectTree ::loadTree()
{
    TV_INSERTSTRUCT tvstruct;


	tvstruct.hParent = NULL;
	tvstruct.hInsertAfter = TVI_FIRST;
	tvstruct.item.iImage = TO_ID(IDB_OBJECT);
	tvstruct.item.iSelectedImage = TO_ID(IDB_OBJECT);
	tvstruct.item.pszText = _T("All Objects");
	tvstruct.item.mask    = TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_TEXT;
	HTREEITEM hTreeRoot   = m_ObjectTree.InsertItem(&tvstruct);

    m_ObjectTree.Expand(m_ObjectTree.GetRootItem(), TVE_EXPAND);
}






BEGIN_MESSAGE_MAP(CObjectTree , CSizingControlBar)
	//{{AFX_MSG_MAP(CObjectTree )
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_NOTIFY(NM_DBLCLK, IDC_OBJECTTREE, OnDblclkCartree)
	ON_NOTIFY(TVN_SELCHANGED, IDC_OBJECTTREE, OnclkCartree)
	ON_NOTIFY(TVN_ITEMEXPANDING, IDC_OBJECTTREE, OnItemexpandingTree)
	ON_NOTIFY(NM_RCLICK, IDC_OBJECTTREE, OnRclickTracktree)	
	ON_COMMAND(ID_TREE_OPEN, OnOpen)
	ON_COMMAND(ID_TREEMENU_EDIT, OnEdit)	
	ON_COMMAND(ID_RIGHTPOP_SETTOPDRAWLOCATION, OnTopDrawingTree)
	ON_COMMAND(ID_RIGHTPOP_ADDTOPDRAWLOCATION, OnAddTopDrawingTree)
	ON_COMMAND(ID_RIGHTPOP_SAVESELECTEDFILEAS, OnSaveFileAsTree)
	ON_COMMAND(ID_RIGHTPOP_OUTPUTTREE, OnOutputTree)
	ON_COMMAND(ID_TREEMENU_RENAME, OnRenameImageFile)
	ON_COMMAND(ID_TREEMENU_REMOVENODE, OnRemoveNode)
	ON_COMMAND(ID_TREEMENU_INSERT_INSETPOINT, OnInsertPoint)
	ON_COMMAND(ID_TREEMENU_INSERT_INSERTSTRING, OnInsertString)
	ON_COMMAND(ID_TREEMENU_INSERT_INSERTPLANE, OnInsertPlane)	
	ON_COMMAND(ID_TREEMENU_REMOVE_INSERTNODE, OnInsertNode)
	ON_COMMAND(ID_TREEMENU_INSERT_INSERTCHLDNODE, OnInsertChildNode)
	ON_COMMAND(ID_3DOMENU_VIEW3DOSTATS, OnView3DStats)
	ON_COMMAND(ID_TREEMENU_CUTNODE, OnInsertCutNode)
	ON_COMMAND(ID_TREEMENU_COPYNODE, OnInsertCopyNode)
	ON_COMMAND(ID_TREEMENU_PASTENODE, OnInsertPasteNode)
	ON_COMMAND(ID_INSERT_SELECTPOINT, OnInsertSelectPoint)
	ON_COMMAND(ID_TREEMENU_EXPORTSUBTREE, OnExportSubTree)
	
	
	
	
	
	
	
	
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CObjectTree  message handlers

void CObjectTree ::OnSize(UINT nType, int cx, int cy) 
{
	CSizingControlBar::OnSize(nType, cx, cy);
	CRect rc;
    GetClientRect(rc);

	Reposition(rc);
}

void CObjectTree ::Reposition(CRect rc)
{
	int border = 4;

    int bottom = (IsHorz() || IsFloating())
        ? rc.Height() - (2*border) : rc.Height() - (2*border) - 4;

    m_ObjectTree.SetWindowPos(NULL,rc.left + border, rc.top + border,
            rc.Width() - (2*border), bottom , SWP_NOACTIVATE | SWP_NOZORDER);            
}

int CObjectTree ::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CSizingControlBar::OnCreate(lpCreateStruct) == -1)
		return -1;

	CRect rc;

	GetClientRect(rc);
	rc.left   = PROFILE(TreeLeft,Preferences,10);
	rc.right  = PROFILE(TreeRight,Preferences,100);
	rc.top    = PROFILE(TreeTop,Preferences,10);
	rc.bottom = PROFILE(TreeBottom,Preferences,110);

	int cx=rc.right;
	int cy=rc.right;
	
	if (!m_ObjectTree.CreateEx(WS_EX_CLIENTEDGE,"SysTreeView32","",TVS_HASLINES  | TVS_HASBUTTONS | TVS_SHOWSELALWAYS | TVS_DISABLEDRAGDROP | WS_VISIBLE | WS_CHILD | WS_BORDER ,0,0,cx,cy,GetSafeHwnd(),(HMENU)IDC_OBJECTTREE))
       return -1;


	Reposition(rc);

	LoadImages();

	return 0;
}

void CObjectTree ::LoadImages()
{
	pImageList = (((CMy3DOEditorApp*)AfxGetApp())->getSharedImageList());
    m_ObjectTree.SetImageList(pImageList, TVSIL_NORMAL);

	//ImagesNotLoaded = FALSE;

	loadTree();
}

void CObjectTree ::InsertThreeDOObject(CDocument *doc,ThreeDOObject *object)
{
	//object->pDoc = doc;
//	if (object!=NULL) object->LoadTree(this);
}

void CObjectTree ::Refresh(ThreeDOObject *object)
{
   //if (object->objectNode) m_ObjectTree.DeleteItem(object->objectNode);
   //if (object!=NULL) object->LoadTree(this);
}


HTREEITEM CObjectTree ::insertInfoNode(HTREEITEM parent, LPSTR title,int bitmap)
{
	if (m_ObjectTree)
	{
		TV_INSERTSTRUCT tvstruct;

        tvstruct.hParent      = parent;
	    tvstruct.hInsertAfter = TVI_LAST;
	    tvstruct.item.iImage  = bitmap;
	    tvstruct.item.iSelectedImage = bitmap;
	    tvstruct.item.pszText = _T((char*)title);
	    tvstruct.item.mask    = TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_TEXT;
	    HTREEITEM hThisItem   = m_ObjectTree.InsertItem(&tvstruct);
	    m_ObjectTree.Expand(m_ObjectTree.GetRootItem(), TVE_EXPAND);
		return hThisItem;
	}
	else
	{
	  char msg[256];

	  wsprintf(msg,"m_TrackTree is NULL for adding in %s",title);
	  AfxMessageBox(msg);
	}
	return 0;
}

void CObjectTree ::OnDblclkCartree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if (m_ObjectTree)
	{
	 HTREEITEM TreeNode = m_ObjectTree.GetSelectedItem();

	 DWORD data = m_ObjectTree.GetItemData(TreeNode);
	 
	 Observer *dlg = (Observer*)data;
	 if (dlg != NULL)
	 {
		dlg->Explore();
	 }
	}
	else
	{
		AfxMessageBox("Tree NULL",MB_OK);
	}
	*pResult = 0;
}

void CObjectTree ::OnclkCartree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if (m_ObjectTree)
	{
	 HTREEITEM TreeNode = m_ObjectTree.GetSelectedItem();
	 
	 if (TreeNode)
	 {
	  DWORD data = m_ObjectTree.GetItemData(TreeNode);
	 
	  Observer *dlg = (Observer*)data;
	  if (dlg != NULL)
	  {
		dlg->Select();
		
	  }
	 }
	}
	else
	{
		AfxMessageBox("Tree NULL",MB_OK);
	}
	
	*pResult = 0;
}

void CObjectTree ::OnItemexpandingTree(NMHDR* pNMHDR, LRESULT* pResult)
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	if (m_ObjectTree)
	{
	  TV_ITEM *TreeItem = (TV_ITEM*)(&pNMTreeView->itemNew);

	  if (TreeItem != NULL)
	  {
	   HTREEITEM TreeNode = TreeItem->hItem;
	   if (TreeNode!=0)
	   {
	   DWORD data = m_ObjectTree.GetItemData(TreeNode);
	 
	   Observer *dlg = (Observer*)data;
	   if (dlg != NULL)
	   {
		dlg->Expand(this,TreeNode);
	   }	 
	  }
	 }
	 else
	 {
		 AfxMessageBox("No Selection in tree");
	 }
	}
	*pResult = 0;
}

void CObjectTree::OnRclickTracktree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	// TODO: Add your message handler code here and/or call default
	CMenu menu;
    CMenu *submenu;

	// load the menu
    menu.LoadMenu(IDR_LAUNCH); 

	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj!=NULL)
		{
		  if (obj->isA(t_Image))
		   {
			   submenu = menu.GetSubMenu(4);
		   }
		  else if (obj->isAFile())
		  {
		   if (obj->isA(t_tdo))
		   {
			   submenu = menu.GetSubMenu(3);
		   }
		   else
		   {
		      submenu = menu.GetSubMenu(2);
		   }
		  }
		  else
		  {
	       submenu = menu.GetSubMenu(1);
		  }
		}
		else
		{
		  submenu = menu.GetSubMenu(1);
		}
	}
	else
	{
		submenu = menu.GetSubMenu(1);
	}
         
   CPoint point;

	GetCursorPos(&point);
    
	if (submenu)
	{
    // post the menu
    submenu->TrackPopupMenu(TPM_LEFTALIGN|TPM_RIGHTBUTTON,                  
                              point.x,
                              point.y,
                              this,NULL);
	}


	*pResult = 0;
}

void CObjectTree::OnOpen()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj)
		{
		  obj->Explore();
		}
	}

}

void CObjectTree::OnEdit()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node *objNode = (Node*)obj;
		  if (objNode->Edit())
		    objNode->getObject()->UpdateTree(NULL);
		}
	}

}





void CObjectTree::OnTopDrawingTree()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  Node * root    = objNode->getRoot();
		  C3do * objView = root->getObject(); //object being viewed
		  if (objView)
		  {
			  objView->topDrawlocation->removeAllElements();
		      objView->topDrawlocation->addElementReference(objNode);
			  if (objView->pDoc)
			  {
				objView->pDoc->UpdateAllViews(NULL);
			  }
		  }
		  else
		  {
			  AfxMessageBox("Cannot Find Parent Object");
		  }
		}
	}

}

void CObjectTree::OnAddTopDrawingTree()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  Node * root    = objNode->getRoot();
		  C3do * objView = root->getObject(); //object being viewed
		  if (objView)
		  {
			  if (objView->topDrawlocation->findElementIndex(objNode) < 0)
			  {
		        objView->topDrawlocation->addElementReference(objNode);
			  }
			  if (objView->pDoc)
			  {
				objView->pDoc->UpdateAllViews(NULL);
			  }
		  }
		  else
		  {
			  AfxMessageBox("Cannot Find Parent Object");
		  }
		}
	}

}


void CObjectTree::OnSaveFileAsTree()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isAFile())
		{
		  CGplFile * file = (CGplFile*)obj;
		  if (file)
		  {
			  file->GetFile();
		      file->SaveAs();
		  }
		  else
		  {
			  AfxMessageBox("Cannot Find File to Save");
		  }
		}
	}
}


void CObjectTree::OnOutputTree()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isAFile() && obj->isA(t_tdo))
		{
		  C3do * file = (C3do*)obj;
		  if (file)
		  {
			  ofstream out("tree.txt");
			  for(int i=0;i<file->sections->size();i++)
			  {
				  Section *section = (Section*)file->sections->elementAt(i);
				  if (section->isPRIM())
				  {
					  section->getObject()->primTree->ExportData(out);
				  }
			  }
		      
		  }
		  else
		  {
			  AfxMessageBox("Cannot Output Tree");
		  }
		}
		else
		{
			  AfxMessageBox("Cannot Output Tree");
		}
	}
}

void CObjectTree::OnRenameImageFile()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_Image))
		{
		  CImageFile * file = (CImageFile*)obj;
		  if (file)
		  {
			  file->Rename();
		  }
		  else
		  {
			  AfxMessageBox("Cannot Rename this object");
		  }
		}
	}
}



void CObjectTree::OnInsertNode()
{	
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  Node * root    = objNode->getRoot();
		  C3do * objView = root->getObject(); //object being viewed
		  if (objView)
		  {
			  Node *par = objNode->getParent();
			  int   off = objNode->getOffset();
			  Node *nnode = getNewNodeType(off,par);
			  objNode->ReplaceNode(objNode,nnode);
			  objView->UpdateTree(nnode);
		  }
		}
	}
}

void CObjectTree::OnRemoveNode()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  Node * root    = objNode->getRoot();
		  C3do * objView = root->getObject(); //object being viewed
		  int result = AfxMessageBox("I'm removing a tree Node OK?",MB_YESNO);
		  if (result == IDYES)
		  {
			  Node *par = objNode->getParent();
			  int   off = objNode->getOffset();
			  objNode->ReplaceNode(objNode,new EmptyNode(off,par));
			  objView->UpdateTree(par);
		  }
		}
	}

}

void CObjectTree::OnEditNode()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  if (objNode)
		  {
			  C3do *object = objNode->getObject();
			  objNode->Edit();
			  object->UpdateTree(objNode);
		  }
		}
	}

}


void CObjectTree::OnInsertPoint()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  if (objNode)
		  {
			  C3do *object = objNode->getObject();
			  Point3D *newpt = new Point3D();
			  object->addPoint(newpt);
			  newpt->Explore(NULL);
			  object->UpdateTree(NULL);
		  }
		}
	}
}


void CObjectTree::OnInsertPlane()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  if (objNode)
		  {
			  C3do *object = objNode->getObject();
			  CPlanes *newpt = new CPlanes();
			  object->addPlane(newpt);
			  newpt->Explore();
			  object->UpdateTree(NULL);
		  }
		}
	}
}

void CObjectTree::OnInsertString()
{
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  if (objNode)
		  {
			  C3do *object = objNode->getObject();
			  CImageFile *newstr = object->addString("unknown");
			  newstr->Rename();
			  object->UpdateTree(objNode);
		  }
		}
	}

}



Node *getNewNodeType(int off,Node *par)
{
	CNewNode *dlg = new CNewNode();

	int result = dlg->DoModal();

	if (result==IDOK)
	{
		int cmd = dlg->Flavor;

		if (dlg) delete dlg;

		Node *newnode = getChildNode(cmd,off,par,NULL,NULL);		
		int numChildren = newnode->allowedChildren();
		if (numChildren == -1)
		{
			CEditOneVar *dlg = new CEditOneVar();			
			dlg->m_VarVal1 = 1;
			dlg->m_Var1.Format("Children");
			dlg->m_Desc.Format("Select the number of child objects you require");
			int result = dlg->DoModal();
		
			if (result == IDOK)
			{
               numChildren = dlg->m_VarVal1;
			}
			if (dlg) delete dlg;
		}
		newnode->Init();
		newnode->object = par->getObject();

		if (numChildren !=-1 && numChildren> 0)
		{
			newnode->children = (Node**)malloc(numChildren*sizeof(Node*));
			for(int i=0;i<numChildren;i++)
			{
				newnode->addChild(new EmptyNode(off,newnode));
				//newnode->children[i] = new EmptyNode(off,newnode);
			}
		}
		switch(cmd)
		{
			case 0x81d: 
			case 0x81e: 
			case 0x81f: 
			case 0x820: 
			case 0x821: 
				{
					NodePolygon* poly = (NodePolygon*)newnode;
					poly->DefaultArgs();
				}
				break;
			default:
				break;
		}
		return newnode;
	}
	if (dlg) delete dlg;
	return NULL;
}


void CObjectTree::OnInsertChildNode()
{	
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  Node * root    = objNode->getRoot();
		  C3do * objView = root->getObject(); //object being viewed
		  if (objView)
		  {
			  Node *par = objNode->getParent();
			  int   off = objNode->getOffset();
			  int   allowed = objNode->allowedChildren();
			  int   size    = objNode->size();

			  if (allowed==-1)
			  {
			     objNode->addChild(getNewNodeType(off,par));
				 objView->UpdateTree(objNode);
			  }
			  else
			  {
				 AfxMessageBox("You Cannot Insert a Child to this object\nit is fully populated use replace instead");   
			  }
			  
		  }
		}
	}
}


void CObjectTree::OnView3DStats()
{	
	// get the popup menu
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj)
		{
		  totals();
		}
	}
}

void CObjectTree::OnInsertCutNode()
{
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  Node * root    = objNode->getRoot();
		  C3do * objView = root->getObject(); //object being viewed
		  if (objView)
		  {
			  clipBoardNode = objNode;
			  Node *par = objNode->getParent();
			  int   off = objNode->getOffset();
			  objNode->ReplaceNode(objNode,new EmptyNode(off,par));
			  objView->UpdateTree(par);
		  }
		}
	}
}

void CObjectTree::OnInsertCopyNode()
{
	AfxMessageBox("Copy Node Not Working");
}

void CObjectTree::OnInsertPasteNode()
{
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  Node * root    = objNode->getRoot();
		  C3do * objView = root->getObject(); //object being viewed
		  if (objView)
		  {
			  //Node *parent = objNode->getParent();
			  //objNode = clipBoardNode;
			  //objNode->setParent(parent);
			  //Node *par = objNode->getParent();
			  //int   off = objNode->getOffset();
			  objNode->ReplaceNode(objNode,clipBoardNode);
			  objView->UpdateTree(objNode);
		  }
		}
	}
}


void CObjectTree::OnExportSubTree()
{
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_tree))
		{
		  Node * objNode = (Node*)obj;
		  Node * root    = objNode->getRoot();
		  C3do * objView = root->getObject(); //object being viewed
		  if (objNode)
		  {
			 objView->ExportSubTree(objNode);
		  }
		}
	}
}

void CObjectTree::OnInsertSelectPoint()
{
	HTREEITEM selected = m_ObjectTree.GetSelectedItem();
	
	if (selected != NULL)
	{
		Observer *obj = (Observer*)m_ObjectTree.GetItemData(selected);
		if (obj && obj->isA(t_pt))
		{
		  Point3D * objNode = (Point3D*)obj;
		  C3do * objView = objNode->getObject(); //object being viewed
		  if (objView)
		  {
			  objView->selectedPt = objNode;
			  objView->pDoc->UpdateAllViews(NULL);
		  }
		}
	}
}





