
#ifndef _INCLUDED_POINT_H_
#define _INCLUDED_POINT_H_

#include "Display.h"
#include "Observer.h"

typedef struct
{
  float unk;
  float x;
  float y;
  float z;
} t_Point;

typedef struct
{
  int unk;
  int x;
  int y;
  int z;
} t_N2Point;


typedef struct
{
  int unk;
  int x;
  int y;
  int z;
  unsigned char u;
  unsigned char v;
} t_ICR2Point;

class C3do;

class PointAlias: public CObject
{
public:
	PointAlias(int id,char *_alias):
	id(id)
	{
      alias = strdup(_alias);
	}

	~PointAlias()
	{
		free(alias);
	}

	int id;
	char *alias;
};

#define PSCALE 19685.0f

class Point3D: public Observer
{
public:
	Point3D():Observer(t_pt)
	{
		InitPt();	
	}

	Point3D(float x,float y,float z):Observer(t_pt)
	{
		InitPt();
		pt.x = x;
		pt.y = y;
		pt.z = z;
	}

	void InitPt()
	{
		tdoCount = 0;
		SrbCount = 0;
		pt.x = 0.0f;
		pt.y = 0.0f;
		pt.z = 0.0f;
		pt.unk = 0.0f;
		u = -1;
		v = -1;
		ptAlias = NULL;
		useTexture = FALSE;
	}

	void Read(FILE *fp);
	void ReadN2(FILE *fp);
	void ReadICR2(FILE *fp,BOOL texture);
	void Write(FILE *fp);
	void WriteN2(FILE *fp);
	void WriteICR2(FILE *fp);
	

	float getX()
	{
		return pt.x;
	}

	float getY()
	{
		return pt.y;
	}

	float getZ()
	{
		return pt.z;
	}


	void setX(float x) { pt.x = x; }
	void setY(float y) { pt.y = y; }
	void setZ(float z) { pt.z = z; }
	void setUNK(float unk) { pt.unk = unk; }

	CString toString()
	{
	  CString buffer;
	  if (ptAlias!=NULL)
	  {
	    buffer.Format("(%2.4f,%2.4f,%2.4f)unk=%d %s",pt.x,pt.y,pt.z,(int)pt.unk,ptAlias);
	  }
	  else
	  {
		buffer.Format("(%2.4f,%2.4f,%2.4f)unk=%d",pt.x,pt.y,pt.z,(int)pt.unk);
	  }
	  return buffer;
	}

	void Explore(CDocument *pDoc=NULL);

	void Explore()
	{
		Explore(NULL);
	}

	void Draw2D(Display *display);
	void Draw2DSelected(Display *display);
	
	BOOL isOver(Display *display,CPoint pt);

	void setLocation(Display *display,CPoint pt);

	C3do *getObject();
	
	t_Point   pt;


	int tdoCount;
	int SrbCount;
	int idx;
	C3do *object;
	char *ptAlias;

	void setPointName(char *name)
	{
		ptAlias = strdup(name);
	}

	BOOL useTexture;
	int u;
	int v;
	int pos;
};


#endif