// PolygonEditor.cpp : implementation file
//

#include "stdafx.h"
#include "3DOEditor.h"
#include "PolygonEditor.h"
#include "Node.h"
#include "C3do.h"

#include "EditOneVar.h"
#include "EditTwoVar.h"
#include "EditTwoFVar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPolygonEditor dialog


CPolygonEditor::CPolygonEditor(NodePolygon *node,CWnd* pParent /*=NULL*/)
	: CDialog(CPolygonEditor::IDD, pParent),node(node)
{
	//{{AFX_DATA_INIT(CPolygonEditor)
	m_StrOffset = _T("");
	m_Blue = 0;
	m_Green = 0;
	m_Red = 0;
	m_Alpha = 0;
	m_PlaneA = 0.0f;
	m_PlaneB = 0.0f;
	m_PlaneC = 0.0f;
	m_PlaneD = 0.0f;
	m_PlaneNumber = 0;
	//}}AFX_DATA_INIT
}


void CPolygonEditor::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPolygonEditor)
	DDX_Control(pDX, IDC_VERTEX_NORMALS, m_VertexNormals);
	DDX_Control(pDX, IDC_VERTEX_COLORS, m_VertexColors);
	DDX_Control(pDX, IDC_TEXTURE_COORD, m_TextCoord);
	DDX_Control(pDX, IDC_VERTICIES, m_Verticies);
	DDX_Control(pDX, IDC_TEXTURE_COMBO, m_Textures);
	DDX_Text(pDX, IDC_STR_OFFSET, m_StrOffset);
	DDX_Text(pDX, IDC_BLUE, m_Blue);
	DDX_Text(pDX, IDC_GREEN, m_Green);
	DDX_Text(pDX, IDC_RED, m_Red);
	DDX_Text(pDX, IDC_ALPHA, m_Alpha);
	DDX_Text(pDX, IDC_PLANE_A, m_PlaneA);
	DDX_Text(pDX, IDC_PLANE_B, m_PlaneB);
	DDX_Text(pDX, IDC_PLANE_C, m_PlaneC);
	DDX_Text(pDX, IDC_PLANE_D, m_PlaneD);
	DDX_Text(pDX, IDC_PLANE_NUMBER, m_PlaneNumber);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPolygonEditor, CDialog)
	//{{AFX_MSG_MAP(CPolygonEditor)
	ON_BN_CLICKED(IDC_DOWN, OnDown)
	ON_BN_CLICKED(IDC_UP, OnUp)
	ON_BN_CLICKED(IDC_CHOOSE, OnChoose)
	ON_LBN_DBLCLK(IDC_TEXTURE_COORD, OnDblclkTextureCoord)
	ON_LBN_DBLCLK(IDC_VERTICIES, OnDblclkVerticies)
	ON_BN_CLICKED(IDC_ADDPT, OnAddpt)
	ON_BN_CLICKED(IDC_REMOVEPT, OnRemovept)
	ON_BN_CLICKED(IDC_EDITPT, OnEditpt)
	ON_BN_CLICKED(IDC_EDITPT2, OnEditTexturept)
	ON_BN_CLICKED(IDC_UP2, OnUpTexture)
	ON_BN_CLICKED(IDC_DOWN2, OnDownTexture)
	ON_BN_CLICKED(IDC_BUTTON1, OnEditPlane)
	ON_BN_CLICKED(IDC_BUTTON2, OnReCalculatePlane)
	ON_BN_CLICKED(IDC_EDITPT4, OnEditNormal)
	ON_LBN_SELCHANGE(IDC_VERTEX_NORMALS, OnSelchangeVertexNormals)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPolygonEditor message handlers

void CPolygonEditor::OnDown() 
{
	int idx   = m_Verticies.GetCurSel();
	int count = m_Verticies.GetCount();

	if (idx != LB_ERR && idx < count-1)
	{
		int a = idx;
		int b = idx+1;

		int tx = node->verticies[a];
		node->verticies[a] = node->verticies[b];
		node->verticies[b] = tx;
		m_Verticies.SetCurSel(b);
	}
	LoadListBoxes();
}

void CPolygonEditor::OnUp() 
{
	int idx = m_Verticies.GetCurSel();

	if (idx != LB_ERR && idx > 0)
	{
		int a = idx;
		int b = idx-1;

		int tx = node->verticies[a];
		node->verticies[a] = node->verticies[b];
		node->verticies[b] = tx;
		m_Verticies.SetCurSel(b);
	}
	LoadListBoxes();
}

BOOL CPolygonEditor::OnInitDialog() 
{
	CDialog::OnInitDialog();
	m_Alpha= (node->color & 0xFF000000) >> 24;
	m_Red= (node->color & 0x00FF0000) >> 16;
	m_Green = (node->color & 0x0000FF00) >> 8;
	m_Blue=(node->color & 0x000000FF);



	C3do *object = node->getObject();

	for(int i=0;i<object->strs->size();i++)
	{
		CImageFile *file = (CImageFile*)object->strs->elementAt(i);
		if (file)
		{
		  m_Textures.AddString(file->filename);
		}
	}

	m_StrOffset.Format("(%d)",node->getTextureStringOffset());

	m_PlaneNumber = node->getPlaneNumber();

	int numPlanes = node->getObject()->numPlanes;

	if (m_PlaneNumber >= 0 && m_PlaneNumber <numPlanes)
	{
	   CPlanes *plane = node->getObject()->Plans[m_PlaneNumber];
	   m_PlaneA = plane->A.data;
	   m_PlaneB = plane->B.data;
	   m_PlaneC = plane->C.data;
	   m_PlaneD = plane->D.data;
	}
	else
	{
		m_PlaneA = -99;
		m_PlaneB = -99;
		m_PlaneC = -99;
		m_PlaneD = -99;
	}

	LoadListBoxes();

	
	UpdateData(FALSE);	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL hasVertextNormals(int type)
{
	if (type == 0x81e) return TRUE;
	if (type == 0x81d) return TRUE;
	if (type == 0x821) return TRUE;
	return FALSE;
}

BOOL hasVertextColors(int type)
{
	if (type == 0x81c) return TRUE;
	if (type == 0x81e) return TRUE;
	if (type == 0x820) return TRUE;
	return FALSE;
}

void CPolygonEditor::LoadListBoxes()
{
	int curSelV = m_Verticies.GetCurSel();
	int curSelT = m_TextCoord.GetCurSel();
	m_Verticies.ResetContent();
	m_TextCoord.ResetContent();
	m_VertexColors.ResetContent();
	m_VertexNormals.ResetContent();

	C3do *object = node->getObject();

	for(int i=0;i<node->numberOfVerticies;i++)
	{
	  int verid = node->verticies[i];
	  CString buff;

	  float x = object->ptsArray[verid]->getX();
	  float y = object->ptsArray[verid]->getY();
	  float z = object->ptsArray[verid]->getZ();

	  buff.Format("[%d] x=%2.4f y=%2.4f z=%2.4f",verid,x,y,z);
	  m_Verticies.AddString(buff);
	}

	// hack to find out if it is a texture polygon
	//FIXME
	if (node->bitmapid == IDB_POLYGONS_TEXT)
	{
	 for(i=0;i<node->numberOfVerticies;i++)
	 {
	  if (node->xtexture && node->ytexture)
	  {
	   float xt = node->xtexture[i];
	   float yt = node->ytexture[i];
	   CString buff;


	   buff.Format("u=%2.4f v=%2.4f",xt,yt);
	   m_TextCoord.AddString(buff);
	  }
	 }
	}
	else
	{
		m_TextCoord.EnableWindow(FALSE);
	}

	if (hasVertextColors(node->type))
	{
	 for(i=0;i<node->numberOfVerticies;i++)
	 {
	  int color = node->vertexcolors[i];
	  CString buff;

	  buff.Format("color=%d",color);
	  m_VertexColors.AddString(buff);
	 }
	}
	else
	{
	  m_VertexColors.EnableWindow(FALSE);
	}

	if (hasVertextNormals(node->type))
	{
	 for(i=0;i<node->numberOfVerticies;i++)
	 {
	  int normal = node->vertexnormals[i];
	  CString buff;

	  buff.Format("normal=%d",normal);
	  m_VertexNormals.AddString(buff);
	 }
	}
	else
	{
	  m_VertexNormals.EnableWindow(FALSE);
	}



	if (curSelV!=LB_ERR)
	{
		m_Verticies.SetCurSel(curSelV);
	}
	if (curSelT!=LB_ERR)
	{
		m_TextCoord.SetCurSel(curSelT);
	}
	
}

void CPolygonEditor::OnOK() 
{
	// TODO: Add extra validation here
	//node->color = RGB(m_Red,m_Green,m_Blue);
	//m_Red= (node->color & 0x00FF0000) >> 16;
	//m_Green = (node->color & 0x0000FF00) >> 8;
	//m_Blue=(node->color & 0x000000FF);
	UpdateData(TRUE);

	node->color = (m_Alpha << 24) + ((m_Red << 16) + (m_Green <<8) + m_Blue);
	
	CDialog::OnOK();
}

void CPolygonEditor::OnChoose() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);
	CColorDialog *dlg = new CColorDialog(RGB(m_Red,m_Green,m_Blue));
	int result = dlg->DoModal();

	if (result)
	{
		m_Red   = GetRValue(dlg->GetColor());
		m_Green = GetGValue(dlg->GetColor());
		m_Blue  = GetBValue(dlg->GetColor());
		m_Alpha = 255;
		UpdateData(FALSE);
	}

	if (dlg) delete dlg;

}

void CPolygonEditor::OnDblclkTextureCoord() 
{
	int idx = m_TextCoord.GetCurSel();

	CEditTwoFVar *dlg = new CEditTwoFVar();
	

	dlg->m_VarVal1 = node->xtexture[idx];
	dlg->m_VarVal2 = node->ytexture[idx];
	dlg->m_Var1.Format("X Texture Coord");
	dlg->m_Var2.Format("Y Texture Coord");

	int result = dlg->DoModal();

	if (result == IDOK)
	{
	  node->xtexture[idx] = dlg->m_VarVal1; 
	  node->ytexture[idx] = dlg->m_VarVal2; 
	}
	

}

void CPolygonEditor::OnDblclkVerticies() 
{
	int idx = m_Verticies.GetCurSel();

	CEditOneVar *dlg = new CEditOneVar();

	dlg->m_VarVal1 = node->verticies[idx];
	dlg->m_Var1.Format("Vextex");

	int result = dlg->DoModal();

	if (result == IDOK)
	{
	  node->verticies[idx] = dlg->m_VarVal1; 
	  LoadListBoxes();
	}
}

void CPolygonEditor::OnAddpt() 
{
	node->numberOfVerticies++;
	int *newverticies = new int[node->numberOfVerticies];

	for(int i=0;i<node->numberOfVerticies-1;i++)
	{
      newverticies[i] = node->verticies[i];
	}
	newverticies[node->numberOfVerticies-1] = 0;
	delete node->verticies;
	node->verticies = newverticies;

	if (node->bitmapid == IDB_POLYGONS_TEXT)
	{
		float *newxtexture = new float[node->numberOfVerticies];
		float *newytexture = new float[node->numberOfVerticies];
		for(int i=0;i<node->numberOfVerticies-1;i++)
		{
			newxtexture[i] = node->xtexture[i];
			newytexture[i] = node->ytexture[i];
		}
		newxtexture[node->numberOfVerticies-1] = 0.0f;
		newytexture[node->numberOfVerticies-1] = 0.0f;
		delete node->xtexture;
		delete node->ytexture;
		node->xtexture = newxtexture;
		node->ytexture = newytexture;
	}

	LoadListBoxes();
}

void CPolygonEditor::OnRemovept() 
{
	int idx = m_Verticies.GetCurSel();
	node->numberOfVerticies--;

	for(int i=idx+1;i<node->numberOfVerticies;i++)
	{
      node->verticies[i] = node->verticies[i+1];
	}

	LoadListBoxes();
}

void CPolygonEditor::OnEditpt() 
{
	OnDblclkVerticies();
}

void CPolygonEditor::OnEditTexturept() 
{
	OnDblclkTextureCoord();
}

void CPolygonEditor::OnUpTexture() 
{
	int idx = m_TextCoord.GetCurSel();

	if (idx != LB_ERR && idx > 0)
	{
		int a = idx;
		int b = idx-1;

		float tx = node->xtexture[a];
		node->xtexture[a] = node->xtexture[b];
		node->xtexture[b] = tx;

		float ty = node->ytexture[a];
		node->ytexture[a] = node->ytexture[b];
		node->ytexture[b] = ty;
		m_TextCoord.SetCurSel(b);
	}
	LoadListBoxes();
}

void CPolygonEditor::OnDownTexture() 
{
	int idx   = m_TextCoord.GetCurSel();
	int count = m_TextCoord.GetCount();

	if (idx != LB_ERR && idx < count-1)
	{
		int a = idx;
		int b = idx+1;

		float tx = node->xtexture[a];
		node->xtexture[a] = node->xtexture[b];
		node->xtexture[b] = tx;

		float ty = node->ytexture[a];
		node->ytexture[a] = node->ytexture[b];
		node->ytexture[b] = ty;
		m_TextCoord.SetCurSel(b);
	}
	
	LoadListBoxes();
}

void CPolygonEditor::OnEditPlane() 
{
	int numPlanes = node->getObject()->numPlanes;

	if (m_PlaneNumber >= 0 && m_PlaneNumber <numPlanes)
	{
	   CPlanes *plane = node->getObject()->Plans[m_PlaneNumber];
	   plane->Explore();
	}
	else
	{
		AfxMessageBox("Confused over plane number");	
	}
}

void CPolygonEditor::OnReCalculatePlane() 
{
	int numPlanes = node->getObject()->numPlanes;

	if (m_PlaneNumber >= 0 && m_PlaneNumber <numPlanes)
	{
	   CPlanes *plane = node->getObject()->Plans[m_PlaneNumber];
	   node->CalculateABCD();
	   m_PlaneA = plane->A.data = node->A;
	   m_PlaneB = plane->B.data = node->B;
	   m_PlaneC = plane->C.data = node->C;
	   m_PlaneD = plane->D.data = node->D;
	   UpdateData(FALSE);
	}
	else
	{
		AfxMessageBox("Confused over plane number");	
	}
}

void CPolygonEditor::OnEditNormal() 
{
	int curSel = m_VertexNormals.GetCurSel();
	int numNormals = node->getObject()->numNormals;

	if (curSel >= 0 && curSel <numNormals)
	{
	   CNormals *norm = node->getObject()->Normals[curSel];
	   norm->Explore();
	}
	else
	{
		AfxMessageBox("Confused over Normal number");	
	}
}

void CPolygonEditor::OnSelchangeVertexNormals() 
{
	int idx = m_VertexNormals.GetCurSel();

	CEditOneVar *dlg = new CEditOneVar();

	dlg->m_VarVal1 = node->vertexnormals[idx];
	dlg->m_Var1.Format("Normal");

	int result = dlg->DoModal();

	if (result == IDOK)
	{
	  node->vertexnormals[idx] = dlg->m_VarVal1; 
	  LoadListBoxes();
	}
}
