
#ifndef _INCLUDED_SECTION_H_
#define _INCLUDED_SECTION_H_

#include "resource.h"
#include "Node.h"
#include "C3do.h"

class Node;

typedef struct
{
  char id[4];
  int  unk1;
  int  length;
} t_SectionHeader;

#define STRS_ID "NRTS"
#define XYZ_ID "SZYX"
#define PRIM_ID "MIRP"
#define PLAN_ID "NALP"
#define HAND_ID "DNAH"
#define NORM_ID "MRON"

class Section: public Observer
{
	C3do *object;

    public:
	Section(FILE *fp,C3do *obj):fp(fp),Observer(t_section)
	{
		subTree = NULL;
		//primTree = NULL;
		object = obj;
	}

	void setInfo(char *id,int len)
	{
		strncpy(info.id,id,4);
		info.unk1 = 0;
		info.length = len;
	}



	virtual ~Section();

	C3do* getObject()
	{
		return object;
	}

	CString reverName()
	{
		CString buffer;
		buffer.Format("%c%c%c%c",info.id[3],info.id[2],info.id[1],info.id[0]);
		return buffer;
	}

	void LoadTree(HTREEITEM parent,CObjectTree *tree,C3do *obj)
	{
		object = obj;
		CString buffer;
		buffer.Format("%s \"%s\" Length=%d Offset=0x%x",info.id,reverName(),info.length,offset);
		SectionNode = tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buffer,TO_ID(IDB_OFFSET));
		tree->m_ObjectTree.SetItemData(SectionNode,(DWORD)this);
		subTree = tree->insertInfoNode(SectionNode, (LPSTR)(LPCSTR)"subTree",TO_ID(IDB_OFFSET));
	}

	int ReadHeader(FILE *fp)
	{
		while(1)
		{
		int sizeHeader = fread(&info,1,sizeof(t_SectionHeader),fp);
		if (sizeHeader != sizeof(t_SectionHeader))
		{
			return -1;
		}
		if (info.id[0] != 0x20 ) break;
		int size = sizeof(t_SectionHeader)-1;
		fseek(fp,-size,SEEK_CUR);
		}
		
		offset = ftell(fp);
		fseek(fp,info.length,SEEK_CUR);
		int offset1 = ftell(fp);
		return offset+info.length;
	}

	void WriteHeader(FILE *fp)
	{
	   fwrite(&info,1,sizeof(t_SectionHeader),fp);
	}

	BOOL isXYZ()
	{
		BOOL isGPL3do = (!strcmp(info.id,XYZ_ID));
		return (isGPL3do);
	}

	BOOL isPERP()
	{
		BOOL isN23do  = (!strcmp(info.id,"PERP"));
		return (isN23do);
	}

	CString GetName()
	{
		CString name("");
		name += info.id[3];
		name += info.id[2];
		name += info.id[1];
		name += info.id[0];
		return name;
	}

	BOOL isSTR()
	{
		return (!strcmp(info.id,STRS_ID));
	}

	BOOL isPLAN()
	{
		return (!strcmp(info.id,PLAN_ID));
	}

	BOOL isPRIM()
	{
		return (!strcmp(info.id,PRIM_ID));
	}

	BOOL isHAND()
	{
		return (!strcmp(info.id,HAND_ID));
	}

	BOOL isNORM()
	{
		return (!strcmp(info.id,NORM_ID));
	}

	void LoadSubTree(CObjectTree *tree);

	void Expand(CObjectTree *tree,HTREEITEM node)
	{
		   if (subTree){
			   tree->m_ObjectTree.DeleteItem(subTree);
			   LoadSubTree(tree);
			   subTree = NULL;
		   }
	}

	void parsePrimTree(FILE *fp,HTREEITEM SectionNode, CObjectTree *tree);
	void readCmd(HTREEITEM parent, CObjectTree *tree);
	Node *readPrim(int ReadOffset,int Handle_offset);


	t_SectionHeader info;
	int offset;
	HTREEITEM SectionNode;
	
	int RootOffset;

	private:
	FILE *fp;
	
};


#endif