
#include "stdafx.h"
#include "Trk.h"
#include "stdio.h"
#include "math.h"
#include "register.h"

#include "3DOEditor.h"
#include "Progress.h"
#include "C3do.h"
#include "Section.h"
#include "3DOEditorView.h"

#include "resource.h"

#include <gl\glaux.h>
#include <gl\gl.h>
#include <gl\glu.h>

BOOL CheckRegistrationName();

#define PI 3.14159265

double DegToRad(double angle)
{
	return angle*PI/180.0;
}


extern void WritePCDWord(FILE *fp,int value);
extern int ReadPCDWord(FILE *fp);
void setGLColor(float r,float g,float b);

CTrk::~CTrk()
	{
	  if (loaded)
	  {
	   //if (Altitudes) delete Altitudes;
	   delete []Altitudes;
	   delete []Walls;
	   delete []TrackSections;
	   if (Sections) delete Sections;
	  }
}

BOOL CTrk::Read(const char *filename)
{
	FILE *fp = FOPEN(filename,"rb");

	if (fp == NULL)
	{
		CString msg;
		msg.Format("Cannot find file: %s",filename);
		AfxMessageBox(msg);
		return FALSE;
	}
	Read(fp);
	return TRUE;
}

BOOL CTrk::Read(FILE *fp)
{
	int offset = ftell(fp);
	fread(&Header,1,sizeof(t_Trk_Header),fp);

	Sections = new t_TrkSection[Header.sections];

	numAltitudes  = Header.sections*Header.lateral_count;
	if (isICR2())
	{
	  numWalls      = Header.wallLength/12;
	}
	else
	{
	  numWalls      = Header.wallLength/32;
	}
	Altitudes     =  new AltitudeSection[numAltitudes];
	Walls         =  new WallSection[numWalls];
	TrackSections =  new TrackSection[Header.sections];
	loaded = TRUE;

	if (isICR2())
	{
	  lateral = (int*) malloc(10*sizeof(int));
	  for(int i=0;i<10;i++)
	  {
		lateral[i] = ReadPCDWord(fp);
	  }
	}
	else
	{
	  lateral = (int*) malloc(16*sizeof(int));
	  for(int i=0;i<16;i++)
	  {
		lateral[i] = ReadPCDWord(fp);
	  }
	}

	

	for(int i=0;i<Header.sections;i++)
	{
		fread(&(Sections[i]),1,sizeof(t_TrkSection),fp);
	}

	for(i=0;i<numAltitudes;i++)
	{
        Altitudes[i].setParent(this);
		Altitudes[i].Read(fp,this);
	    //fread(&(Altitudes[i]),1,sizeof(t_AltitudeSection),fp);
	}

	for(i=0;i<numWalls;i++)
	{
		Walls[i].setParent(this);
		Walls[i].Read(fp,this);
	}

	for(i = 0;i<Header.sections;i++)
	{
		TrackSections[i].setParent(this);
		TrackSections[i].Read(fp,this);
	}
	return TRUE;
}


BOOL CTrk::Write(FILE *fp)
{
	// only write out at offset if part of dat file
	// old style of editor
	if (!datFileParent->isPackFile())
	{
	  fseek(fp,filepos,SEEK_SET);
	}
	fwrite(&Header,1,sizeof(t_Trk_Header),fp);

	if (isICR2())
	{
	  for(int i=0;i<10;i++)
	  {
		 WritePCDWord(fp,lateral[i]);
	  }
	}
	else
	{
	  for(int i=0;i<16;i++)
	  {
		 WritePCDWord(fp,lateral[i]);
	  }
	}

	for(int i=0;i<Header.sections;i++)
	{
		fwrite(&(Sections[i]),1,sizeof(t_TrkSection),fp);
	}

	for(i=0;i<numAltitudes;i++)
	{
	    fwrite(&(Altitudes[i]),1,sizeof(t_AltitudeSection),fp);
	}

	for(i=0;i<numWalls;i++)
	{
		Walls[i].Write(fp,this);
	}

	for(i = 0;i<Header.sections;i++)
	{
		TrackSections[i].Write(fp,this);
	}
	return TRUE;
}



void CTrk::Explore()
{	
	//LoadTree(FileNode,((CMy3DOEditorApp*)AfxGetApp())->m_MainTree);
}

double OrientationToDegrees(long orientation)
{
   return (static_cast<unsigned long>(orientation) / 4294967296.0) * 360.0;
}

double CurvatureToDegrees(long orientation)
{
   return (orientation / 4294967296.0) * 720.0;
}


int getTrkIcon(TrackSection *section)
{
     if (section->data.type == 2)
	 {
		 if (section->data.unk4 < 0) 
			 return IDB_CURVE_RIGHT;
		 else
		     return IDB_CURVE_LEFT;
	 }
	 return IDB_TRACK;
}

void CTrk::LoadTree(HTREEITEM parent,CObjectTree *tree)
{
	subTree = tree->insertInfoNode(parent, "SubTree",TO_ID(IDB_FILE));
}

CBrush *getWallBrush(int val,Display *display)
{
   val = val & 0x7FF;
   switch(val)
   {
      case 1:return new CBrush(RGB(0xC0,0xC0,0xC0));
	  case 4:return new CBrush(RGB(0x00,0xA0,0x00));
	  case 5:return new CBrush(RGB(0x00,0x00,0xFF));
      case 6:return new CBrush(RGB(0xC0,0xC0,0x00));
	  case 10:return new CBrush(RGB(0x00,0x80,0x00));
	  case 2059:return new CBrush(RGB(0x80,0x80,0x80));
	  case 2056:return new CBrush(RGB(0x80,0x80,0x80));
   }
   cerr << "Unknown Surface Type " << val << endl;
   return NULL;
}

#define GLRGB(r,g,b) glColor3d(((double)r/256.0),((double)g/256.0),((double)b/256.0))

void glgetWallBrush(int val)
{
   val = val & 0x7FF;
   switch(val)
   {
      case 1:  GLRGB(0xC0,0xC0,0xC0); break;
	  case 4:  GLRGB(0x00,0xA0,0x00);break;
	  case 5:  GLRGB(0x00,0x00,0xFF);break;
      case 6:  GLRGB(0xC0,0xC0,0x00);break;
	  case 10:  GLRGB(0x00,0x80,0x00);break;
	  case 2059:GLRGB(0x80,0x80,0x80);break;
	  case 2056:GLRGB(0x80,0x80,0x80);break;
   }
}

void getSurface(int val,CString &surf)
{
	val = val & 0x7FF;
	switch(val)
	{
	case 1:
		surf = "Road";
		break;
    case 4:
		surf = "Grass";
		break;
    case 5:
		surf = "Kerb";
		break;
	case 2056:
		surf = "Barrier/Fence";
		break;
    case 2059:
		surf = "Slip Road (WG)";
		break;
	case 6:
		surf = "Gravel";
		break;
    case 10:
		surf = "Forest";
		break;
	default:
		CString msg;
		msg.Format("Unknown Surface Value %d val");
		surf = msg;
		TRACE1("%s\n",msg);
		break;
	}
}

void CTrk::LoadSubTree(HTREEITEM parent,CObjectTree *tree)
{
	CProgress *dlg = new CProgress();
	dlg->UpdateProgress("Loading Laterals",0);

	GetFile();
	tree->m_ObjectTree.SetRedraw(FALSE);
	CString buff;
	int i;

	buff.Format("Version %d ",Header.version);tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));
	buff.Format("Track Length %d Meters (%d)",Header.track_length/19685,Header.track_length);tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));
	buff.Format("Number Of Laterals %d ",Header.lateral_count);tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));
	buff.Format("Number Of Section? %d ",Header.sections);tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));
	
	buff.Format("Laterals %d",Header.lateral_count);
	HTREEITEM latparent = tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));
	for( i=0;i<Header.lateral_count;i++)
	{
	  buff.Format("Laterals[%d] %d ",i,lateral[i]);
	  tree->insertInfoNode(latparent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));
	}

	dlg->UpdateProgress("Loading Altitudes",10);

	buff.Format("Sections Offsets %d",Header.sections);
	HTREEITEM secparent = tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));

	if (Sections ==NULL)
	{
		AfxMessageBox("Cannot Load TRK file for this type");
		dlg->UpdateProgress("Aborted",100);
	    delete dlg;
		return;
	}

	for( i=0;i<Header.sections;i++)
	{
	  buff.Format("Track Section Offsets[%d] %d ",i,Sections[i]);
	  tree->insertInfoNode(secparent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));
	}

	buff.Format("Altitudes %d",numAltitudes);
	HTREEITEM altparent = tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));

	for( i=0;i<numAltitudes;i++)
	{	  
	  buff.Format("[%d]Altitudes for Section[%d (%d of %d)] start=%d ",i,(i/Header.lateral_count),1+i-(Header.lateral_count*(i/Header.lateral_count)),Header.lateral_count,Altitudes[i].data.start);
	  HTREEITEM alnode = tree->insertInfoNode(altparent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));
	  tree->m_ObjectTree.SetItemData(alnode,(DWORD)&Altitudes[i]);

	  buff.Format("Delta 1 = %d",Altitudes[i].data.delta1);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Delta 2 = %d",Altitudes[i].data.delta2);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Delta 3 = %d",Altitudes[i].data.delta3);	  
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Start Height = %d",Altitudes[i].data.start);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Delta 4 = %d",Altitudes[i].data.delta4);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Delta 5 = %d",Altitudes[i].data.delta5);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Unknown (X?) = %d",Altitudes[i].data.X);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Unknown (Y?) = %d",Altitudes[i].data.Y);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	}

	dlg->UpdateProgress("Loading Walls",50);

	buff.Format("Walls %d",numWalls);
	HTREEITEM wallparent = tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));

	for( i=0;i<numWalls;i++)
	{
	  buff.Format("Walls[%d] ",i);
	  HTREEITEM alnode = tree->insertInfoNode(wallparent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_FILE));
	  tree->m_ObjectTree.SetItemData(alnode,(DWORD)&Walls[i]);


	  buff.Format("From = %d",Walls[i].data.from);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("To = %d",Walls[i].data.to);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));

	  CString surf;
	  getSurface(Walls[i].data.surface,surf);
	  buff.Format("Surface = %d = %s",Walls[i].data.surface,surf);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Height = %d",Walls[i].data.height);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Unk1 = %d",Walls[i].data.unk1);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Unk2 = %d",Walls[i].data.unk2);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	}

	dlg->UpdateProgress("Loading Sections",80);

	buff.Format("Track Sections %d",Header.sections);
	HTREEITEM tsecparent = tree->insertInfoNode(parent, (LPSTR)(LPCSTR)buff,TO_ID(IDB_TRACK));

	for( i=0;i<Header.sections;i++)
	{
	  buff.Format("Section[%d] ",i);
	  HTREEITEM alnode = tree->insertInfoNode(tsecparent, (LPSTR)(LPCSTR)buff,TO_ID(getTrkIcon(&TrackSections[i])));
	  tree->m_ObjectTree.SetItemData(alnode,(DWORD)&TrackSections[i]);

	  buff.Format("Type = %d (1=straight 2=curved)",TrackSections[i].data.type);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));

	  buff.Format("Start = %d",TrackSections[i].data.start);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));

	  buff.Format("Length = %d",TrackSections[i].data.length);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));

	  buff.Format("Orientation = %d (%f%c)",TrackSections[i].data.orientation,OrientationToDegrees(TrackSections[i].data.orientation),176);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));

	  buff.Format("Unk1 = %d",TrackSections[i].data.XC);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Unk2 = %d",TrackSections[i].data.YC);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Unk3 = %d",TrackSections[i].data.unk3);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Curvature = %d %f%c",TrackSections[i].data.unk4,CurvatureToDegrees(TrackSections[i].data.unk4),176);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Unk5 = %d",TrackSections[i].data.unk5);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Unk6 = %f",TrackSections[i].data.unk6);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));

	  buff.Format("Altitude Index = %d",TrackSections[i].data.altitudes_index);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Wall Count = %d",TrackSections[i].data.wall_count);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	  buff.Format("Wall Index Start = %d",TrackSections[i].data.wall_index);
	  tree->insertInfoNode(alnode, (LPSTR)(LPCSTR)buff,TO_ID(IDB_OFFSET));
	}

	dlg->UpdateProgress("Finished",100);
	delete dlg;

	tree->m_ObjectTree.SetRedraw(TRUE);
}



double getLeftSideX(double angle,double length)
{
    return length*cos(angle-PI/2);
}

double getLeftSideY(double angle,double length)
{
    return length*sin(angle-PI/2);
}

double getRightSideX(double angle,double length)
{
    return length*cos(angle+PI/2);
}

double getRightSideY(double angle,double length)
{
    return  length*sin(angle+PI/2);
}

#define TRK_WIDTH 20





void CTrk::ExportCSV(FILE *fp)
{
	for(int i=0;i<Header.sections;i++)
	{
		TrackSections[i].ExportCSV(fp);		
	}
}

void CTrk::updateTrack()
{
    int start  = TrackSections[0].data.start;
	cout << "updating Track..";
	double angle=0;
	for(int i=0;i<Header.sections;i++)
	{
	  double TrackAngle = CurvatureToDegrees(TrackSections[i].data.unk4);
 	  if (TrackSections[i].data.type == 2)
	  {
		  angle += TrackAngle;
	  }
	  if (TrackSections[i].data.start != start) 
	  {
		  // it seems that the track has been modified
		  // so set the file so it will be saved next time
		  SetModified();
	  }

	  TrackSections[i].data.start = start;
	  int length  = TrackSections[i].data.length;
	  start+=length;
	}
	cout << "done" << endl;

	cout << "Old Track Length=" << Header.track_length << " = " << ToMeters(Header.track_length) << " Meters" << endl;
	Header.track_length = start;
	cout << "New Track Length=" << Header.track_length << " = " << ToMeters(Header.track_length) << " Meters" <<endl;
	cout << "Total Track Angle = " <<angle << endl;
	
}  

TrackSection * CTrk::SectorForLength(double len)
{
	double totalLength=0;
	
	for(int i=0;i<Header.sections;i++)
    {
        int st  = TrackSections[i].data.start;
		
		totalLength += ToMeters(TrackSections[i].data.length);		
		if (totalLength > len) return &TrackSections[i];
	}
	return NULL;
}


void CTrk::Draw2D(Display *display, DatView *view,double x,double y)
{
	//BOOL showWalls =TRUE;
	//BOOL showPointNumbers = TRUE;
	if (!loaded) return;
	//double x = 0;
	//double y = 0;

	//int first  = TrackSections[4].data.altitudes_index;
	//AltitudeSection *alt = &Altitudes[first];
	//x= ToMeters(alt->data.X);
	//y= ToMeters(alt->data.Y);

	
	double angle = 0.0;
	double sx  = x;
	double sy  = y;
	double nsx =sx;
	double nsy =sy;

	display->SelectGreyPen();
	if (view->showAxis)
	{
	display->drawLine(0,0,4096,0);
	display->drawLine(0,0,-4096,0);
	display->drawLine(0,0,0,4096);
	display->drawLine(0,0,0,-4096);
	}
	CPen *bluePen = new CPen(PS_SOLID,2,RGB(0x00,0x00,0xFF));
	CPen *oldPen = display->pDC->SelectObject(bluePen);

	angle = OrientationToDegrees(TrackSections[0].data.orientation);
	  
	for(int i=0;i<Header.sections;i++)
	{
	  int st  = TrackSections[i].data.start;
	  double length = ToMeters(TrackSections[i].data.length);
	  double TrackAngle = CurvatureToDegrees(TrackSections[i].data.unk4);
 	  double StartTrackAngle = angle*PI/180.0;
	
	  if (TrackSections[i].data.type == 2)
	  {
		  angle += TrackAngle;
	  }

	  double EndTrackAngle = angle*PI/180.0;

	  double anglelen = length*TrackAngle;
      double anglerad = DegToRad(TrackAngle);
      double currrad  = StartTrackAngle;

      double sectorStartAngle = currrad;
      double sectorEndAngle   = currrad;
      double sectorRadius     = 0;

	  double tx,ty;

      if (TrackSections[i].data.type == 2)
      {
		    double xc = ToMeters(TrackSections[i].data.XC);
		    double yc = ToMeters(TrackSections[i].data.YC);
		    double dx = (sx-xc);
		    double dy = (sy-yc);
			
            double beta = TrackAngle/2.0;
            double inrad        = DegToRad(beta);
            sectorRadius       = (length/anglerad);
            double len = 2*(sectorRadius*sin(inrad));

            ty =  len*sin((currrad+inrad));
            tx =  len*cos((currrad+inrad));
            
            sectorEndAngle = currrad+inrad;
      }
      else
      {
            ty = length*sin(currrad);
            tx = length*cos(currrad);
      }
	  

	  nsx += tx;
	  nsy += ty;

	  TrackSections[i].startX = sx;	  
	  TrackSections[i].startY = sy;
	  TrackSections[i].endX = nsx;
	  TrackSections[i].endY = nsy;
	  TrackSections[i].startAngle = sectorStartAngle;
	  TrackSections[i].endAngle = sectorEndAngle;

	  
	  if (TrackSections[i].data.type == 2)
	  {
		  double Cx = sx+getRightSideX(sectorStartAngle,sectorRadius);
		  double Cy = sy+getRightSideY(sectorStartAngle,sectorRadius);
		  display->drawArc(sx,sy,nsx,nsy,Cx,Cy,sectorStartAngle,anglelen,sectorRadius);
	  }
	  else
	  {
		display->drawLine(sx,sy,nsx,nsy);
	  }

	  

	  int wallStart = TrackSections[i].data.wall_index;
	  int wallNum = TrackSections[i].data.wall_count;

	  if (view->showWalls)
	  {
	  for(int j=0;j<wallNum;j++)
	  {
		  if ((wallStart+j) < 0 || (wallStart+j) >= numWalls) break;
		  WallSection *wall = &Walls[wallStart+j];
		  double to   = ToMeters(wall->data.to);
		  double from = ToMeters(wall->data.from);
          double lbsx,lbsy;
		  double lesx,lesy;

		  if (from < 0)
		  {
		   lbsx =sx+getRightSideX(StartTrackAngle,from);
	       lbsy =sy+getRightSideY(StartTrackAngle,from);
		  }
		  else
		  {
		   lbsx =sx+getLeftSideX(StartTrackAngle,-from);
	       lbsy =sy+getLeftSideY(StartTrackAngle,-from);
		  }

		  if (to < 0)
		  {
		   lesx =nsx+getRightSideX(EndTrackAngle,to);
	       lesy =nsy+getRightSideY(EndTrackAngle,to);
		  }
		  else
		  {
		   lesx =nsx+getLeftSideX(EndTrackAngle,-to);
	       lesy =nsy+getLeftSideY(EndTrackAngle,-to);
		  }

		  if (TrackSections[i].data.type == 2)
          {	
			double nrad = sectorRadius-from;
			double Cx = sx+getRightSideX(sectorStartAngle,sectorRadius);
		    double Cy = sy+getRightSideY(sectorStartAngle,sectorRadius);
		    display->drawArc(sx,sy,nsx,nsy,Cx,Cy,sectorStartAngle,anglelen,nrad);		    
		  }
		  else
		  {
			display->drawLine(lbsx,lbsy,lesx,lesy);
		  }

		  display->drawLine(lbsx,lbsy,sx,sy);
		  display->drawLine(lesx,lesy,nsx,nsy);
		  

		  if (view->showFilled)
		  {
		   CBrush *br = getWallBrush(wall->data.surface,display);
		   if (br != NULL)
		   {
		   CBrush * oldbrush = display->pDC->SelectObject(br);
		   display->drawPolygon(sx,sy,lbsx,lbsy,lesx,lesy,nsx,nsy);
		   display->pDC->SelectObject(oldbrush);
		   delete br;
		   }
		  }
	  }
	  }

	  //display->pDC->SelectObject(oldPen);
	  //delete bluePen;
	  display->pDC->SelectStockObject(BLACK_BRUSH);
	  display->drawRectangle(sx-3,sy-3,sx+3,sy+3);
	  if (view->showTrackNumbers)
	  {
		 CString str;
		 str.Format("%d",i);
		 display->drawText(sx+3,sy+3,str);
	  }


	  sx = nsx;
	  sy = nsy;

	  

	}

	/*
	for(i=0;i<Header.sections;i++)
	{
	  int first  = TrackSections[i].data.altitudes_index;
	  for(int j=0;j<8;j++)
	  {
	  AltitudeSection *alt = &Altitudes[first+j];
	  x= ToMeters(alt->data.X);
	  y= ToMeters(alt->data.Y);
	  display->drawLine(x,y,0,0);
	  }
	}
	*/
}

double CTrk::LengthToTrackSector(TrackSection *in)
{
	double totalLength=0;

	int size = Header.sections;
	for(int i=0;i<size;i++)
    {
        TrackSection *t = (TrackSection*)&TrackSections[i];
		if (t == in)
			return totalLength;

		if (t->isCorner() ==0) totalLength+=t->getLength();
        else totalLength+= fabs(t->getLength());

	}
	return (double)ToMeters(totalLength);
}

BOOL CTrk::Create3do(LPCSTR filename)
{
	C3do *tdo = new C3do(NULL);
	strcpy(tdo->Header.id,"4OD3");
	tdo->Header.unk1 = 0;
	tdo->Header.FileSize = 0;

	Section *XYZ   = new Section(NULL,tdo);
	Section *PLAN  = new Section(NULL,tdo);
	Section *STRS  = new Section(NULL,tdo);
	Section *PRIMS = new Section(NULL,tdo);
	//Section *HANDLES = new Section();

	XYZ->setInfo(XYZ_ID,-1);
	PLAN->setInfo(PLAN_ID,-1);
	STRS->setInfo(STRS_ID,-1);
	PRIMS->setInfo(PRIM_ID,-1);

	tdo->addSection(XYZ);
	//tdo->addSection(PLAN);
	//tdo->addSection(STRS);
	tdo->addSection(PRIMS);
	//tdo->addSection(HANDLES);

	tdo->numPts = 1;
	tdo->ptsArray = (Point3D**)malloc(tdo->numPts*sizeof(Point3D*));
	tdo->ptsArray[0] = new Point3D();

	tdo->primTree = new NodeFour(0,NULL);
	tdo->primTree->object = tdo;

	NodeTen *tableA = new NodeTen(0,tdo->primTree);
	tableA->object = tdo;

	tdo->primTree->addChild(tableA);

	int trkSections = 25;

	for(int i=0;i<trkSections;i++)
	{
		NodeFour *trkSectionParent = new NodeFour(0,tableA);
		trkSectionParent->object = tdo;
		tdo->primTree->addChild(trkSectionParent);
		
		// this make a tree for each track section

		// 4 small section
		// 2 medium sections
		// 1 big section

		// I suggest we take a seciton and sub divide it
		// but the lower sections need more data.

		for(int i=0;i<7;i++)
		{
			NodeSix *tableTrk = new NodeSix(0,trkSectionParent);
			tableTrk->object = tdo;
			trkSectionParent->addChild(tableTrk);
			NodeFour *A = new NodeFour(0,NULL);
			A->object = tdo;
			tableTrk->addChild(A);

			
			// whats in this Node Four
			//NodeWalls in Node10
			//TrackRight in Node 5 texture section
			//CenterLine in PolygonN2
			//TrackMiddle in Node 5 texture section
			//PitEdgeLine in Node 5 texture section
			//PitEdge in Polygon N2
			// track markings in Node4
			//	-> list of N2 polygon
			//	-> Lines
			// if 0
			// 4 start lines
		}
		// 4 distances
		NodeTen *distanceTable = new NodeTen(0,trkSectionParent);
		distanceTable->AddEntry(0);
		distanceTable->AddEntry(0);
		distanceTable->AddEntry(0);
		distanceTable->AddEntry(0);
		distanceTable->object = tdo;
		trkSectionParent->addChild(distanceTable);
	}

	tdo->Write(filename);
	return TRUE;
}


void CTrk::DrawGL(CMy3DOEditorView *view,double x,double y,double z)
{
	if (!loaded) return;
	
	double angle = 0.0;
	double sx  = x;
	double sy  = y;
	double sz  = z;
	double nsx =sx;
	double nsy =sy;
	double nsz = sz;

	setGLColor(0.0,0.0,1.0);

	angle = OrientationToDegrees(TrackSections[0].data.orientation);
	  
	for(int i=0;i<Header.sections;i++)
	{
	  int st  = TrackSections[i].data.start;
	  int he  = TrackSections[i].data.altitudes_index;
	  double length = ToMeters(TrackSections[i].data.length);
	  double TrackAngle = CurvatureToDegrees(TrackSections[i].data.unk4);
 	  double StartTrackAngle = angle*PI/180.0;

	  if (TrackSections[i].data.type == 2)
	  {
		  angle += TrackAngle;
	  }

	  double EndTrackAngle = angle*PI/180.0;

	  double anglelen = length*TrackAngle;
      double anglerad = DegToRad(TrackAngle);
      double currrad  = StartTrackAngle;

      double sectorStartAngle = currrad;
      double sectorEndAngle   = currrad;
      double sectorRadius     = 0;

	  double tx,ty;

      if (TrackSections[i].data.type == 2)
      {
		    double xc = ToMeters(TrackSections[i].data.XC);
		    double yc = ToMeters(TrackSections[i].data.YC);
		    double dx = (sx-xc);
		    double dy = (sy-yc);
			
            double beta = TrackAngle/2.0;
            double inrad        = DegToRad(beta);
            sectorRadius       = (length/anglerad);
            double len = 2*(sectorRadius*sin(inrad));

            ty =  len*sin((currrad+inrad));
            tx =  len*cos((currrad+inrad));
            
            sectorEndAngle = currrad+inrad;
      }
      else
      {
            ty = length*sin(currrad);
            tx = length*cos(currrad);
      }
	  

	  nsx += tx;
	  nsy += ty;
	  //nsz += tz;
	  sz   = ToMeters(Altitudes[he+4].data.start);
	  nsz  = ToMeters(Altitudes[he+4].data.start+Altitudes[he].data.delta1+Altitudes[he].data.delta2+Altitudes[he].data.delta3);

	  TrackSections[i].startX = sx;	  
	  TrackSections[i].startY = sy;
	  //TrackSections[i].startZ = sz;
	  TrackSections[i].endX = nsx;
	  TrackSections[i].endY = nsy;
	  //TrackSections[i].endZ = nsz;
	  TrackSections[i].startAngle = sectorStartAngle;
	  TrackSections[i].endAngle = sectorEndAngle;

	  
	  if (TrackSections[i].data.type == 2)
	  {
		  double Cx = sx+getRightSideX(sectorStartAngle,sectorRadius);
		  double Cy = sy+getRightSideY(sectorStartAngle,sectorRadius);
		  glBegin(GL_LINES);
		  glVertex3d(sx,sy,sz);
		  glVertex3d(nsx,nsy,nsz);
		  glEnd();
		  //display->drawArc(sx,sy,nsx,nsy,Cx,Cy,sectorStartAngle,anglelen,sectorRadius);
		  //glPushMatrix();
		  //glTranslated(Cx,Cy,0);
		  //GLUquadricObj *disk = gluNewQuadric();
		  //gluQuadricDrawStyle(disk,GLU_LINE);
		  //gluPartialDisk(disk,sectorRadius,sectorRadius,20,1,sectorStartAngle,anglelen);
		  //glPopMatrix();
	  }
	  else
	  {
		//display->drawLine(sx,sy,nsx,nsy);
		    glBegin(GL_LINES);
			//display->drawLine(lbsx,lbsy,lesx,lesy);
			glVertex3d(sx,sy,sz);
			glVertex3d(nsx,nsy,nsz);
			glEnd();
	  }

	  

	  int wallStart = TrackSections[i].data.wall_index;
	  int wallNum = TrackSections[i].data.wall_count;

	  for(int j=0;j<wallNum;j++)
	  {
		  if ((wallStart+j) < 0 || (wallStart+j) >= numWalls) break;
		  WallSection *wall = &Walls[wallStart+j];
		  double to   = ToMeters(wall->data.to);
		  double from = ToMeters(wall->data.from);
          double lbsx,lbsy,lbsz;
		  double lesx,lesy,lesz;
		  double lastbx,lastby,lastbz;
		  double lastex,lastey,lastez;

		  double height = ToMeters(wall->data.height);

		  if (height > 0)
		  {
			  setGLColor(1.0,0.0,0.0);
		  }
		  else
		  {
			  setGLColor(0.0,0.0,1.0);
		  }

		  lbsz = 0;
		  lesz = 0;

		  if (from < 0)
		  {
		   lbsx =sx+getRightSideX(StartTrackAngle,from);
	       lbsy =sy+getRightSideY(StartTrackAngle,from);
		  }
		  else
		  {
		   lbsx =sx+getLeftSideX(StartTrackAngle,-from);
	       lbsy =sy+getLeftSideY(StartTrackAngle,-from);
		  }

		  if (to < 0)
		  {
		   lesx =nsx+getRightSideX(EndTrackAngle,to);
	       lesy =nsy+getRightSideY(EndTrackAngle,to);
		  }
		  else
		  {
		   lesx =nsx+getLeftSideX(EndTrackAngle,-to);
	       lesy =nsy+getLeftSideY(EndTrackAngle,-to);
		  }

		  lbsz   = ToMeters(Altitudes[he+j].data.start);
		  int endHeight = Altitudes[he+j].data.start+Altitudes[he+j].data.delta1+Altitudes[he+j].data.delta2+Altitudes[he+j].data.delta3;
	      lesz   = ToMeters(endHeight);

		  

		  if (TrackSections[i].data.type == 2)
          {	
			double nrad = sectorRadius-from;
			double Cx = sx+getRightSideX(sectorStartAngle,sectorRadius);
		    double Cy = sy+getRightSideY(sectorStartAngle,sectorRadius);
		    //display->drawArc(sx,sy,nsx,nsy,Cx,Cy,sectorStartAngle,anglelen,nrad);		    
			glBegin(GL_LINES);
			//display->drawLine(lbsx,lbsy,lesx,lesy);
			glVertex3d(lbsx,lbsy,lbsz);
			glVertex3d(lesx,lesy,lesz);
			glEnd();
		  }
		  else
		  {
			glBegin(GL_LINES);
			//display->drawLine(lbsx,lbsy,lesx,lesy);
			glVertex3d(lbsx,lbsy,lbsz);
			glVertex3d(lesx,lesy,lesz);
			glEnd();
		  }

		  if (height > 0)
		  {
			if (view->GetDocument()->showFilled)
			{
			  glgetWallBrush(wall->data.surface);
			  glBegin(GL_POLYGON);
			}
			else
			{
			  glBegin(GL_LINE_LOOP);
			}
			glVertex3d(lbsx,lbsy,lbsz);
			glVertex3d(lesx,lesy,lesz);
			glVertex3d(lesx,lesy,lesz+height);
			glVertex3d(lbsx,lbsy,lbsz+height);
			glVertex3d(lbsx,lbsy,lbsz);
			glEnd();
		  }

		  if (j > 0)
		  {
			glBegin(GL_LINES);
			glVertex3d(lbsx,lbsy,lbsz);
		    glVertex3d(lastbx,lastby,lastbz);
			glVertex3d(lesx,lesy,lesz);
		    glVertex3d(lastex,lastey,lastez);
			glEnd();
		  }

		  /*
		  if (view->GetDocument()->showFilled)
		  {
		   
		   glBegin(GL_POLYGON);
		   glVertex3d(sx,sy,sz);
		   glVertex3d(lbsx,lbsy,lbsz);
		   glVertex3d(lesx,lesy,lesz);
		   glVertex3d(nsx,nsy,nsz);
		   glEnd();
		  }
		  */

		  lastbx = lbsx;
		  lastby = lbsy;
		  lastbz = lbsz;
		  lastex = lesx;
		  lastey = lesy;
		  lastez = lesz;
		  

	  }
	  //}

	  

	  sx = nsx;
	  sy = nsy;
	  sz = nsz;

	  

	}
}
