
#ifndef _INCLUDED_TRK_H_
#define _INCLUDED_TRK_H_

#include "CGPLFile.h"
#include "Display.h"
#include "WallEdit.h"
#include "AlltitudeEditor.h"

class WallSection;
class AltitudeSection;
class TrackSection;

#include "TrackEdit.h"
#include "DatView.h"

// ok this time I'm going to start with 
// with a class

double getRightSideX(double angle,double length);
double getRightSideY(double angle,double length);


extern void WritePCDWord(FILE *fp,int value);

typedef struct
{
	char id[4];	//MIP
	int version;
	int track_length;
	int lateral_count;
	int sections;
	int wallLength;
	int sectionLength;
	//int lateral[16];
} t_Trk_Header;

typedef struct
{
	int offset;
} t_TrkSection;


typedef struct
{
	int delta1;
	int delta2;
	int delta3;
	int start;
	int delta4;
	int delta5;
	int X;
	int Y;
} t_AltitudeSection;

typedef struct
{
	int type;
	int start;
	int length;
	int orientation;
	int unk3;
	int XC;
	int YC;	
	int unk4;
	int unk5;
	int unk6;
	int altitudes_index;
	int wall_count;
	int wall_index;
} t_TrackSection;


typedef struct 
{
	int unk1;
	int unk2;
	int unk3;
	int unk4;
	int unk5;
} t_WallICR2Section;



//#define SCALE 19685

const double ScaleToMeters = 19685.0;

class CMy3DOEditorView;

class CTrk: public CGplFile
{
public:
	BOOL treeLoaded;

	CTrk():CGplFile(t_trk)
	{
		loaded = FALSE;
		Sections = NULL;
		Altitudes = NULL;
		treeLoaded = FALSE;
	}

	~CTrk();

	CString GetDefaultExtension()
	{
		return CString("*.trk");
	}

	void DrawGL(CMy3DOEditorView *view,double x,double y,double z);

	BOOL Read(const char *filename);
	BOOL Read(FILE *fp);
	BOOL Write(FILE *fp);

	void ExportCSV(FILE *fp);

	void Explore();

	void LoadTree(HTREEITEM parent,CObjectTree *tree);
	void LoadSubTree(HTREEITEM parent,CObjectTree *tree);

	void Expand(CObjectTree *tree,HTREEITEM node)
	{
		   if (subTree){
			   tree->m_ObjectTree.DeleteItem(subTree);
			   LoadSubTree(node,tree);
			   subTree = NULL;
		   }
	}

	t_Trk_Header Header;
	int         *lateral;
	t_TrkSection *Sections;
	AltitudeSection *Altitudes;
	WallSection     *Walls;
	TrackSection    *TrackSections;

	void Draw2D(Display *display,DatView *view,double x,double y);

	void updateTrack();

	int numAltitudes;
	int numWalls;
	BOOL loaded;

	double ToMeters(double length)
	{
		return (double)length/ScaleToMeters;
	}

	BOOL isVersion(int version)
	{
		return (Header.version == version) ? TRUE : FALSE;
	}

	BOOL isICR2()
	{
		return isVersion(1);
	}

	BOOL isGPL()
	{
		return isVersion(3000);
	}

	TrackSection * SectorForLength(double len);
	double LengthToTrackSector(TrackSection *in);

	BOOL Create3do(LPCSTR filename);
};

class TrackSection: public Observer
{
   public:
	CTrk *parent;
	double   startX;
	double   startY;
	double   endX;
	double   endY;
	double   startAngle;
	double   endAngle;
	double   angle;

	TrackSection(CTrk *parent=NULL):Observer(t_trktrk),parent(parent)
	{}

	void setParent(CTrk *_parent)
	{
       parent = _parent; 
	}

	~TrackSection()
	{}

	BOOL isCorner()
	{
		return ((data.type == 2) ? TRUE : FALSE);
	}

	void Read(FILE *fp,CTrk *trk)
	{
		if (trk->isICR2())
		{
		  memset(&data,0,sizeof(t_TrackSection));
		  data.type = ReadPCDWord(fp);
		  data.start = ReadPCDWord(fp);
		  data.length = ReadPCDWord(fp);
		  data.orientation = ReadPCDWord(fp);	      
	      data.XC= ReadPCDWord(fp);
	      data.YC= ReadPCDWord(fp);	
	      data.unk4= ReadPCDWord(fp);
		  data.unk3= ReadPCDWord(fp);
	      data.unk5= ReadPCDWord(fp);
		  data.altitudes_index = ReadPCDWord(fp);
	      data.unk6= ReadPCDWord(fp);	      
		  data.wall_index= ReadPCDWord(fp);
	      data.wall_count= ReadPCDWord(fp);
		  ICR2walls = (t_WallICR2Section*)malloc(data.wall_count*sizeof(t_WallICR2Section));
          for(int i=0;i<data.wall_count;i++)
		  {
			  ICR2walls[i].unk1= ReadPCDWord(fp);
			  ICR2walls[i].unk2= ReadPCDWord(fp);
			  ICR2walls[i].unk3= ReadPCDWord(fp);
			  ICR2walls[i].unk4= ReadPCDWord(fp);
			  ICR2walls[i].unk5= ReadPCDWord(fp);
		  }
		}
		else
		{
		 fread(&data,1,sizeof(t_TrackSection),fp);
		}
	}

	void Write(FILE *fp,CTrk *trk)
	{
		if (trk->isICR2())
		{
		  WritePCDWord(fp,data.type);
		  WritePCDWord(fp,data.start);
		  WritePCDWord(fp,data.length);
		  WritePCDWord(fp,data.orientation);	      
	      WritePCDWord(fp,data.XC);
	      WritePCDWord(fp,data.YC);
	      WritePCDWord(fp,data.unk4);
		  WritePCDWord(fp,data.unk3);
	      WritePCDWord(fp,data.unk5);
		  WritePCDWord(fp,data.altitudes_index);
	      WritePCDWord(fp,data.unk6);	      
		  WritePCDWord(fp,data.wall_index);
	      WritePCDWord(fp,data.wall_count);
		  for(int i=0;i<data.wall_count;i++)
		  {
			  WritePCDWord(fp,ICR2walls[i].unk1);
			  WritePCDWord(fp,ICR2walls[i].unk2);
			  WritePCDWord(fp,ICR2walls[i].unk3);
			  WritePCDWord(fp,ICR2walls[i].unk4);
			  WritePCDWord(fp,ICR2walls[i].unk5);
		  }
		}
		else
		{
		  fwrite(&data,1,sizeof(t_TrackSection),fp);
		}
	}

	void ExportCSV(FILE *fp)
	{
		fprintf(fp,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%2.2f\n",data.type,data.start,data.length,data.orientation,data.XC,data.YC,
					data.unk3,data.unk4,data.unk5,data.unk6);
	}

	void Explore()
	{
		CTrackEdit *dlg = new CTrackEdit(parent,this);

		dlg->m_Type = data.type;
		dlg->m_Start = data.start;
		dlg->m_Length = data.length;
		dlg->m_Orientation = data.orientation;
		dlg->m_Unk1 = data.XC;
		dlg->m_Unk2 = data.YC;
		dlg->m_Unk3 = data.unk3;
		dlg->m_Unk4 = data.unk4;
		dlg->m_Unk5 = data.unk5;
		dlg->m_Unk6 = data.unk6;
		dlg->m_AltitudesIndex = data.altitudes_index;
		dlg->m_WallCount = data.wall_count;
		dlg->m_WallIndex = data.wall_index;

		int result = dlg->DoModal();

		if (result == IDOK)
		{
		   data.type = dlg->m_Type;	
		   data.start = dlg->m_Start;	
		   data.length = dlg->m_Length;	
		   data.orientation = dlg->m_Orientation;	
		   data.XC = dlg->m_Unk1;	
		   data.YC = dlg->m_Unk2;	
		   data.unk3 = dlg->m_Unk3;	
		   data.unk4 = dlg->m_Unk4;	
		   data.unk5 = dlg->m_Unk5;	
		   data.unk6 = dlg->m_Unk6;	
		   data.altitudes_index = dlg->m_AltitudesIndex;	
		   data.wall_count = dlg->m_WallCount;	
		   data.wall_index = dlg->m_WallIndex;
		   parent->SetModified();
		}


		delete dlg;
	}

	

	double getLength()
	{
		return data.length;
	}

	t_TrackSection data;
	t_WallICR2Section *ICR2walls;
};

typedef struct
{
	int from;
	int to;
	int surface;
	int unk1;
	int height;
	int unk2;
	int reserved1;
	int reserver2;
} t_WallSection;

class WallSection: public Observer
{
   public:
	CTrk *parent;

	WallSection(CTrk *parent=NULL):Observer(t_trkwall),parent(parent)
	{}

	~WallSection()
	{}

	void setParent(CTrk *_parent)
	{
       parent = _parent; 
	}

	void Read(FILE *fp,CTrk *trk)
	{
		if (trk->isICR2())
		{
		  memset(&data,0,sizeof(t_WallSection));
		  data.from    = ReadPCDWord(fp);
		  data.to      = ReadPCDWord(fp);
		  data.surface = ReadPCDWord(fp);
		}
		else
		{
		  fread(&data,1,sizeof(t_WallSection),fp);
		}
	}

	void Write(FILE *fp,CTrk *trk)
	{
		if (trk->isICR2())
		{
		  WritePCDWord(fp,data.from);
		  WritePCDWord(fp,data.to);
		  WritePCDWord(fp,data.surface);
		}
		else
		{
		  fwrite(&data,1,sizeof(t_WallSection),fp);
		}
	}

	void Explore()
	{
		CWallEdit *dlg = new CWallEdit();

		dlg->m_From = data.from;
		dlg->m_To = data.to;
		dlg->m_Surface = data.surface;
		dlg->m_Height = data.height;
		dlg->m_Unk1 = data.unk1;
		dlg->m_Unk2 = data.unk2;
		int result = dlg->DoModal();

		if (result == IDOK)
		{
			data.from = dlg->m_From;
			data.to = dlg->m_To;
			data.surface = dlg->m_Surface;
			data.height = dlg->m_Height;
			data.unk1 = dlg->m_Unk1;
			data.unk2 = dlg->m_Unk2;
			parent->SetModified();
		}


		delete dlg;
	}

	t_WallSection data;
};

typedef struct
{
	short  data[8];
} t_TrkData;


class AltitudeSection: public Observer
{
   public:
	CTrk *parent;

	AltitudeSection(CTrk *parent=NULL):Observer(t_trkalt),parent(parent)
	{}

	~AltitudeSection()
	{}

	void setParent(CTrk *_parent)
	{
       parent = _parent; 
	}

	void Read(FILE *fp,CTrk *trk)
	{
		fread(&data,1,sizeof(t_AltitudeSection),fp);

	}

	void Write(FILE *fp,CTrk *trk)
	{
		fwrite(&data,1,sizeof(t_AltitudeSection),fp);

	}

	void Explore()
	{
		CAlltitudeEditor *dlg = new CAlltitudeEditor();

		dlg->m_Delta1 = data.delta1;
		dlg->m_Delta2 = data.delta2;
		dlg->m_Delta3 = data.delta3;
		dlg->m_Delta4 = data.delta4;
		dlg->m_Delta5 = data.delta5;
		dlg->m_Start = data.start;
		dlg->m_X = data.X;
		dlg->m_Y = data.Y;
		int result = dlg->DoModal();

		if (result == IDOK)
		{
			data.delta1 = dlg->m_Delta1;
			data.delta2 = dlg->m_Delta2;
			data.delta3 = dlg->m_Delta3;
			data.delta4 = dlg->m_Delta4;
			data.delta5 = dlg->m_Delta5;
			data.start = dlg->m_Start;
			data.X = dlg->m_X;
			data.Y = dlg->m_Y;
			parent->SetModified();
		}


		delete dlg;
	}

	t_AltitudeSection data;
};




#endif